// GENERATED FILE DO NOT EDIT
// $ xsltproc TagKeywords.xsl Part6.xml > gdcmTagKeywords.h

/*=========================================================================

  Program: GDCM (Grassroots DICOM). A DICOM library

  Copyright (c) 2006-2012 Mathieu Malaterre
  All rights reserved.
  See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/

#ifndef GDCMTAGKEYWORDS_H
#define GDCMTAGKEYWORDS_H

#include "gdcmAttribute.h"

namespace gdcm {
namespace Keywords {

    typedef gdcm::Attribute<0x0000, 0x0000> CommandGroupLength;
    typedef gdcm::Attribute<0x0000, 0x0002> AffectedSOPClassUID;
    typedef gdcm::Attribute<0x0000, 0x0003> RequestedSOPClassUID;
    typedef gdcm::Attribute<0x0000, 0x0100> CommandField;
    typedef gdcm::Attribute<0x0000, 0x0110> MessageID;
    typedef gdcm::Attribute<0x0000, 0x0120> MessageIDBeingRespondedTo;
    typedef gdcm::Attribute<0x0000, 0x0600> MoveDestination;
    typedef gdcm::Attribute<0x0000, 0x0700> Priority;
    typedef gdcm::Attribute<0x0000, 0x0800> CommandDataSetType;
    typedef gdcm::Attribute<0x0000, 0x0900> Status;
    typedef gdcm::Attribute<0x0000, 0x0901> OffendingElement;
    typedef gdcm::Attribute<0x0000, 0x0902> ErrorComment;
    typedef gdcm::Attribute<0x0000, 0x0903> ErrorID;
    typedef gdcm::Attribute<0x0000, 0x1000> AffectedSOPInstanceUID;
    typedef gdcm::Attribute<0x0000, 0x1001> RequestedSOPInstanceUID;
    typedef gdcm::Attribute<0x0000, 0x1002> EventTypeID;
    typedef gdcm::Attribute<0x0000, 0x1005> AttributeIdentifierList;
    typedef gdcm::Attribute<0x0000, 0x1008> ActionTypeID;
    typedef gdcm::Attribute<0x0000, 0x1020> NumberOfRemainingSuboperations;
    typedef gdcm::Attribute<0x0000, 0x1021> NumberOfCompletedSuboperations;
    typedef gdcm::Attribute<0x0000, 0x1022> NumberOfFailedSuboperations;
    typedef gdcm::Attribute<0x0000, 0x1023> NumberOfWarningSuboperations;
    typedef gdcm::Attribute<0x0000, 0x1030> MoveOriginatorApplicationEntityTitle;
    typedef gdcm::Attribute<0x0000, 0x1031> MoveOriginatorMessageID;
    typedef gdcm::Attribute<0x0000, 0x0001> CommandLengthToEnd;
    typedef gdcm::Attribute<0x0000, 0x0010> CommandRecognitionCode;
    typedef gdcm::Attribute<0x0000, 0x0200> Initiator;
    typedef gdcm::Attribute<0x0000, 0x0300> Receiver;
    typedef gdcm::Attribute<0x0000, 0x0400> FindLocation;
    typedef gdcm::Attribute<0x0000, 0x0850> NumberOfMatches;
    typedef gdcm::Attribute<0x0000, 0x0860> ResponseSequenceNumber;
    typedef gdcm::Attribute<0x0000, 0x4000> DialogReceiver;
    typedef gdcm::Attribute<0x0000, 0x4010> TerminalType;
    typedef gdcm::Attribute<0x0000, 0x5010> MessageSetID;
    typedef gdcm::Attribute<0x0000, 0x5020> EndMessageID;
    typedef gdcm::Attribute<0x0000, 0x5110> DisplayFormat;
    typedef gdcm::Attribute<0x0000, 0x5120> PagePositionID;
    typedef gdcm::Attribute<0x0000, 0x5130> TextFormatID;
    typedef gdcm::Attribute<0x0000, 0x5140> NormalReverse;
    typedef gdcm::Attribute<0x0000, 0x5150> AddGrayScale;
    typedef gdcm::Attribute<0x0000, 0x5160> Borders;
    typedef gdcm::Attribute<0x0000, 0x5170> Copies;
    typedef gdcm::Attribute<0x0000, 0x5180> CommandMagnificationType;
    typedef gdcm::Attribute<0x0000, 0x5190> Erase;
    typedef gdcm::Attribute<0x0000, 0x51a0> Print;
    typedef gdcm::Attribute<0x0000, 0x51b0> Overlays;
    typedef gdcm::Attribute<0x0008, 0x0001> LengthToEnd;
    typedef gdcm::Attribute<0x0008, 0x0005> SpecificCharacterSet;
    typedef gdcm::Attribute<0x0008, 0x0006> LanguageCodeSequence;
    typedef gdcm::Attribute<0x0008, 0x0008> ImageType;
    typedef gdcm::Attribute<0x0008, 0x0010> RecognitionCode;
    typedef gdcm::Attribute<0x0008, 0x0012> InstanceCreationDate;
    typedef gdcm::Attribute<0x0008, 0x0013> InstanceCreationTime;
    typedef gdcm::Attribute<0x0008, 0x0014> InstanceCreatorUID;
    typedef gdcm::Attribute<0x0008, 0x0016> SOPClassUID;
    typedef gdcm::Attribute<0x0008, 0x0018> SOPInstanceUID;
    typedef gdcm::Attribute<0x0008, 0x001a> RelatedGeneralSOPClassUID;
    typedef gdcm::Attribute<0x0008, 0x001b> OriginalSpecializedSOPClassUID;
    typedef gdcm::Attribute<0x0008, 0x0020> StudyDate;
    typedef gdcm::Attribute<0x0008, 0x0021> SeriesDate;
    typedef gdcm::Attribute<0x0008, 0x0022> AcquisitionDate;
    typedef gdcm::Attribute<0x0008, 0x0023> ContentDate;
    typedef gdcm::Attribute<0x0008, 0x0024> OverlayDate;
    typedef gdcm::Attribute<0x0008, 0x0025> CurveDate;
    typedef gdcm::Attribute<0x0008, 0x002a> AcquisitionDateTime;
    typedef gdcm::Attribute<0x0008, 0x0030> StudyTime;
    typedef gdcm::Attribute<0x0008, 0x0031> SeriesTime;
    typedef gdcm::Attribute<0x0008, 0x0032> AcquisitionTime;
    typedef gdcm::Attribute<0x0008, 0x0033> ContentTime;
    typedef gdcm::Attribute<0x0008, 0x0034> OverlayTime;
    typedef gdcm::Attribute<0x0008, 0x0035> CurveTime;
    typedef gdcm::Attribute<0x0008, 0x0040> DataSetType;
    typedef gdcm::Attribute<0x0008, 0x0041> DataSetSubtype;
    typedef gdcm::Attribute<0x0008, 0x0042> NuclearMedicineSeriesType;
    typedef gdcm::Attribute<0x0008, 0x0050> AccessionNumber;
    typedef gdcm::Attribute<0x0008, 0x0051> IssuerOfAccessionNumberSequence;
    typedef gdcm::Attribute<0x0008, 0x0052> QueryRetrieveLevel;
    typedef gdcm::Attribute<0x0008, 0x0054> RetrieveAETitle;
    typedef gdcm::Attribute<0x0008, 0x0056> InstanceAvailability;
    typedef gdcm::Attribute<0x0008, 0x0058> FailedSOPInstanceUIDList;
    typedef gdcm::Attribute<0x0008, 0x0060> Modality;
    typedef gdcm::Attribute<0x0008, 0x0061> ModalitiesInStudy;
    typedef gdcm::Attribute<0x0008, 0x0062> SOPClassesInStudy;
    typedef gdcm::Attribute<0x0008, 0x0064> ConversionType;
    typedef gdcm::Attribute<0x0008, 0x0068> PresentationIntentType;
    typedef gdcm::Attribute<0x0008, 0x0070> Manufacturer;
    typedef gdcm::Attribute<0x0008, 0x0080> InstitutionName;
    typedef gdcm::Attribute<0x0008, 0x0081> InstitutionAddress;
    typedef gdcm::Attribute<0x0008, 0x0082> InstitutionCodeSequence;
    typedef gdcm::Attribute<0x0008, 0x0090> ReferringPhysicianName;
    typedef gdcm::Attribute<0x0008, 0x0092> ReferringPhysicianAddress;
    typedef gdcm::Attribute<0x0008, 0x0094> ReferringPhysicianTelephoneNumbers;
    typedef gdcm::Attribute<0x0008, 0x0096> ReferringPhysicianIdentificationSequence;
    typedef gdcm::Attribute<0x0008, 0x0100> CodeValue;
    typedef gdcm::Attribute<0x0008, 0x0102> CodingSchemeDesignator;
    typedef gdcm::Attribute<0x0008, 0x0103> CodingSchemeVersion;
    typedef gdcm::Attribute<0x0008, 0x0104> CodeMeaning;
    typedef gdcm::Attribute<0x0008, 0x0105> MappingResource;
    typedef gdcm::Attribute<0x0008, 0x0106> ContextGroupVersion;
    typedef gdcm::Attribute<0x0008, 0x0107> ContextGroupLocalVersion;
    typedef gdcm::Attribute<0x0008, 0x010b> ContextGroupExtensionFlag;
    typedef gdcm::Attribute<0x0008, 0x010c> CodingSchemeUID;
    typedef gdcm::Attribute<0x0008, 0x010d> ContextGroupExtensionCreatorUID;
    typedef gdcm::Attribute<0x0008, 0x010f> ContextIdentifier;
    typedef gdcm::Attribute<0x0008, 0x0110> CodingSchemeIdentificationSequence;
    typedef gdcm::Attribute<0x0008, 0x0112> CodingSchemeRegistry;
    typedef gdcm::Attribute<0x0008, 0x0114> CodingSchemeExternalID;
    typedef gdcm::Attribute<0x0008, 0x0115> CodingSchemeName;
    typedef gdcm::Attribute<0x0008, 0x0116> CodingSchemeResponsibleOrganization;
    typedef gdcm::Attribute<0x0008, 0x0117> ContextUID;
    typedef gdcm::Attribute<0x0008, 0x0201> TimezoneOffsetFromUTC;
    typedef gdcm::Attribute<0x0008, 0x1000> NetworkID;
    typedef gdcm::Attribute<0x0008, 0x1010> StationName;
    typedef gdcm::Attribute<0x0008, 0x1030> StudyDescription;
    typedef gdcm::Attribute<0x0008, 0x1032> ProcedureCodeSequence;
    typedef gdcm::Attribute<0x0008, 0x103e> SeriesDescription;
    typedef gdcm::Attribute<0x0008, 0x103f> SeriesDescriptionCodeSequence;
    typedef gdcm::Attribute<0x0008, 0x1040> InstitutionalDepartmentName;
    typedef gdcm::Attribute<0x0008, 0x1048> PhysiciansOfRecord;
    typedef gdcm::Attribute<0x0008, 0x1049> PhysiciansOfRecordIdentificationSequence;
    typedef gdcm::Attribute<0x0008, 0x1050> PerformingPhysicianName;
    typedef gdcm::Attribute<0x0008, 0x1052> PerformingPhysicianIdentificationSequence;
    typedef gdcm::Attribute<0x0008, 0x1060> NameOfPhysiciansReadingStudy;
    typedef gdcm::Attribute<0x0008, 0x1062> PhysiciansReadingStudyIdentificationSequence;
    typedef gdcm::Attribute<0x0008, 0x1070> OperatorsName;
    typedef gdcm::Attribute<0x0008, 0x1072> OperatorIdentificationSequence;
    typedef gdcm::Attribute<0x0008, 0x1080> AdmittingDiagnosesDescription;
    typedef gdcm::Attribute<0x0008, 0x1084> AdmittingDiagnosesCodeSequence;
    typedef gdcm::Attribute<0x0008, 0x1090> ManufacturerModelName;
    typedef gdcm::Attribute<0x0008, 0x1100> ReferencedResultsSequence;
    typedef gdcm::Attribute<0x0008, 0x1110> ReferencedStudySequence;
    typedef gdcm::Attribute<0x0008, 0x1111> ReferencedPerformedProcedureStepSequence;
    typedef gdcm::Attribute<0x0008, 0x1115> ReferencedSeriesSequence;
    typedef gdcm::Attribute<0x0008, 0x1120> ReferencedPatientSequence;
    typedef gdcm::Attribute<0x0008, 0x1125> ReferencedVisitSequence;
    typedef gdcm::Attribute<0x0008, 0x1130> ReferencedOverlaySequence;
    typedef gdcm::Attribute<0x0008, 0x1134> ReferencedStereometricInstanceSequence;
    typedef gdcm::Attribute<0x0008, 0x113a> ReferencedWaveformSequence;
    typedef gdcm::Attribute<0x0008, 0x1140> ReferencedImageSequence;
    typedef gdcm::Attribute<0x0008, 0x1145> ReferencedCurveSequence;
    typedef gdcm::Attribute<0x0008, 0x114a> ReferencedInstanceSequence;
    typedef gdcm::Attribute<0x0008, 0x114b> ReferencedRealWorldValueMappingInstanceSequence;
    typedef gdcm::Attribute<0x0008, 0x1150> ReferencedSOPClassUID;
    typedef gdcm::Attribute<0x0008, 0x1155> ReferencedSOPInstanceUID;
    typedef gdcm::Attribute<0x0008, 0x115a> SOPClassesSupported;
    typedef gdcm::Attribute<0x0008, 0x1160> ReferencedFrameNumber;
    typedef gdcm::Attribute<0x0008, 0x1161> SimpleFrameList;
    typedef gdcm::Attribute<0x0008, 0x1162> CalculatedFrameList;
    typedef gdcm::Attribute<0x0008, 0x1163> TimeRange;
    typedef gdcm::Attribute<0x0008, 0x1164> FrameExtractionSequence;
    typedef gdcm::Attribute<0x0008, 0x1167> MultiFrameSourceSOPInstanceUID;
    typedef gdcm::Attribute<0x0008, 0x1195> TransactionUID;
    typedef gdcm::Attribute<0x0008, 0x1197> FailureReason;
    typedef gdcm::Attribute<0x0008, 0x1198> FailedSOPSequence;
    typedef gdcm::Attribute<0x0008, 0x1199> ReferencedSOPSequence;
    typedef gdcm::Attribute<0x0008, 0x1200> StudiesContainingOtherReferencedInstancesSequence;
    typedef gdcm::Attribute<0x0008, 0x1250> RelatedSeriesSequence;
    typedef gdcm::Attribute<0x0008, 0x2110> LossyImageCompressionRetired;
    typedef gdcm::Attribute<0x0008, 0x2111> DerivationDescription;
    typedef gdcm::Attribute<0x0008, 0x2112> SourceImageSequence;
    typedef gdcm::Attribute<0x0008, 0x2120> StageName;
    typedef gdcm::Attribute<0x0008, 0x2122> StageNumber;
    typedef gdcm::Attribute<0x0008, 0x2124> NumberOfStages;
    typedef gdcm::Attribute<0x0008, 0x2127> ViewName;
    typedef gdcm::Attribute<0x0008, 0x2128> ViewNumber;
    typedef gdcm::Attribute<0x0008, 0x2129> NumberOfEventTimers;
    typedef gdcm::Attribute<0x0008, 0x212a> NumberOfViewsInStage;
    typedef gdcm::Attribute<0x0008, 0x2130> EventElapsedTimes;
    typedef gdcm::Attribute<0x0008, 0x2132> EventTimerNames;
    typedef gdcm::Attribute<0x0008, 0x2133> EventTimerSequence;
    typedef gdcm::Attribute<0x0008, 0x2134> EventTimeOffset;
    typedef gdcm::Attribute<0x0008, 0x2135> EventCodeSequence;
    typedef gdcm::Attribute<0x0008, 0x2142> StartTrim;
    typedef gdcm::Attribute<0x0008, 0x2143> StopTrim;
    typedef gdcm::Attribute<0x0008, 0x2144> RecommendedDisplayFrameRate;
    typedef gdcm::Attribute<0x0008, 0x2200> TransducerPosition;
    typedef gdcm::Attribute<0x0008, 0x2204> TransducerOrientation;
    typedef gdcm::Attribute<0x0008, 0x2208> AnatomicStructure;
    typedef gdcm::Attribute<0x0008, 0x2218> AnatomicRegionSequence;
    typedef gdcm::Attribute<0x0008, 0x2220> AnatomicRegionModifierSequence;
    typedef gdcm::Attribute<0x0008, 0x2228> PrimaryAnatomicStructureSequence;
    typedef gdcm::Attribute<0x0008, 0x2229> AnatomicStructureSpaceOrRegionSequence;
    typedef gdcm::Attribute<0x0008, 0x2230> PrimaryAnatomicStructureModifierSequence;
    typedef gdcm::Attribute<0x0008, 0x2240> TransducerPositionSequence;
    typedef gdcm::Attribute<0x0008, 0x2242> TransducerPositionModifierSequence;
    typedef gdcm::Attribute<0x0008, 0x2244> TransducerOrientationSequence;
    typedef gdcm::Attribute<0x0008, 0x2246> TransducerOrientationModifierSequence;
    typedef gdcm::Attribute<0x0008, 0x2251> AnatomicStructureSpaceOrRegionCodeSequenceTrial;
    typedef gdcm::Attribute<0x0008, 0x2253> AnatomicPortalOfEntranceCodeSequenceTrial;
    typedef gdcm::Attribute<0x0008, 0x2255> AnatomicApproachDirectionCodeSequenceTrial;
    typedef gdcm::Attribute<0x0008, 0x2256> AnatomicPerspectiveDescriptionTrial;
    typedef gdcm::Attribute<0x0008, 0x2257> AnatomicPerspectiveCodeSequenceTrial;
    typedef gdcm::Attribute<0x0008, 0x2258> AnatomicLocationOfExaminingInstrumentDescriptionTrial;
    typedef gdcm::Attribute<0x0008, 0x2259> AnatomicLocationOfExaminingInstrumentCodeSequenceTrial;
    typedef gdcm::Attribute<0x0008, 0x225a> AnatomicStructureSpaceOrRegionModifierCodeSequenceTrial;
    typedef gdcm::Attribute<0x0008, 0x225c> OnAxisBackgroundAnatomicStructureCodeSequenceTrial;
    typedef gdcm::Attribute<0x0008, 0x3001> AlternateRepresentationSequence;
    typedef gdcm::Attribute<0x0008, 0x3010> IrradiationEventUID;
    typedef gdcm::Attribute<0x0008, 0x4000> IdentifyingComments;
    typedef gdcm::Attribute<0x0008, 0x9007> FrameType;
    typedef gdcm::Attribute<0x0008, 0x9092> ReferencedImageEvidenceSequence;
    typedef gdcm::Attribute<0x0008, 0x9121> ReferencedRawDataSequence;
    typedef gdcm::Attribute<0x0008, 0x9123> CreatorVersionUID;
    typedef gdcm::Attribute<0x0008, 0x9124> DerivationImageSequence;
    typedef gdcm::Attribute<0x0008, 0x9154> SourceImageEvidenceSequence;
    typedef gdcm::Attribute<0x0008, 0x9205> PixelPresentation;
    typedef gdcm::Attribute<0x0008, 0x9206> VolumetricProperties;
    typedef gdcm::Attribute<0x0008, 0x9207> VolumeBasedCalculationTechnique;
    typedef gdcm::Attribute<0x0008, 0x9208> ComplexImageComponent;
    typedef gdcm::Attribute<0x0008, 0x9209> AcquisitionContrast;
    typedef gdcm::Attribute<0x0008, 0x9215> DerivationCodeSequence;
    typedef gdcm::Attribute<0x0008, 0x9237> ReferencedPresentationStateSequence;
    typedef gdcm::Attribute<0x0008, 0x9410> ReferencedOtherPlaneSequence;
    typedef gdcm::Attribute<0x0008, 0x9458> FrameDisplaySequence;
    typedef gdcm::Attribute<0x0008, 0x9459> RecommendedDisplayFrameRateInFloat;
    typedef gdcm::Attribute<0x0008, 0x9460> SkipFrameRangeFlag;
    typedef gdcm::Attribute<0x0010, 0x0010> PatientName;
    typedef gdcm::Attribute<0x0010, 0x0020> PatientID;
    typedef gdcm::Attribute<0x0010, 0x0021> IssuerOfPatientID;
    typedef gdcm::Attribute<0x0010, 0x0022> TypeOfPatientID;
    typedef gdcm::Attribute<0x0010, 0x0024> IssuerOfPatientIDQualifiersSequence;
    typedef gdcm::Attribute<0x0010, 0x0030> PatientBirthDate;
    typedef gdcm::Attribute<0x0010, 0x0032> PatientBirthTime;
    typedef gdcm::Attribute<0x0010, 0x0040> PatientSex;
    typedef gdcm::Attribute<0x0010, 0x0050> PatientInsurancePlanCodeSequence;
    typedef gdcm::Attribute<0x0010, 0x0101> PatientPrimaryLanguageCodeSequence;
    typedef gdcm::Attribute<0x0010, 0x0102> PatientPrimaryLanguageModifierCodeSequence;
    typedef gdcm::Attribute<0x0010, 0x1000> OtherPatientIDs;
    typedef gdcm::Attribute<0x0010, 0x1001> OtherPatientNames;
    typedef gdcm::Attribute<0x0010, 0x1002> OtherPatientIDsSequence;
    typedef gdcm::Attribute<0x0010, 0x1005> PatientBirthName;
    typedef gdcm::Attribute<0x0010, 0x1010> PatientAge;
    typedef gdcm::Attribute<0x0010, 0x1020> PatientSize;
    typedef gdcm::Attribute<0x0010, 0x1021> PatientSizeCodeSequence;
    typedef gdcm::Attribute<0x0010, 0x1030> PatientWeight;
    typedef gdcm::Attribute<0x0010, 0x1040> PatientAddress;
    typedef gdcm::Attribute<0x0010, 0x1050> InsurancePlanIdentification;
    typedef gdcm::Attribute<0x0010, 0x1060> PatientMotherBirthName;
    typedef gdcm::Attribute<0x0010, 0x1080> MilitaryRank;
    typedef gdcm::Attribute<0x0010, 0x1081> BranchOfService;
    typedef gdcm::Attribute<0x0010, 0x1090> MedicalRecordLocator;
    typedef gdcm::Attribute<0x0010, 0x2000> MedicalAlerts;
    typedef gdcm::Attribute<0x0010, 0x2110> Allergies;
    typedef gdcm::Attribute<0x0010, 0x2150> CountryOfResidence;
    typedef gdcm::Attribute<0x0010, 0x2152> RegionOfResidence;
    typedef gdcm::Attribute<0x0010, 0x2154> PatientTelephoneNumbers;
    typedef gdcm::Attribute<0x0010, 0x2160> EthnicGroup;
    typedef gdcm::Attribute<0x0010, 0x2180> Occupation;
    typedef gdcm::Attribute<0x0010, 0x21a0> SmokingStatus;
    typedef gdcm::Attribute<0x0010, 0x21b0> AdditionalPatientHistory;
    typedef gdcm::Attribute<0x0010, 0x21c0> PregnancyStatus;
    typedef gdcm::Attribute<0x0010, 0x21d0> LastMenstrualDate;
    typedef gdcm::Attribute<0x0010, 0x21f0> PatientReligiousPreference;
    typedef gdcm::Attribute<0x0010, 0x2201> PatientSpeciesDescription;
    typedef gdcm::Attribute<0x0010, 0x2202> PatientSpeciesCodeSequence;
    typedef gdcm::Attribute<0x0010, 0x2203> PatientSexNeutered;
    typedef gdcm::Attribute<0x0010, 0x2210> AnatomicalOrientationType;
    typedef gdcm::Attribute<0x0010, 0x2292> PatientBreedDescription;
    typedef gdcm::Attribute<0x0010, 0x2293> PatientBreedCodeSequence;
    typedef gdcm::Attribute<0x0010, 0x2294> BreedRegistrationSequence;
    typedef gdcm::Attribute<0x0010, 0x2295> BreedRegistrationNumber;
    typedef gdcm::Attribute<0x0010, 0x2296> BreedRegistryCodeSequence;
    typedef gdcm::Attribute<0x0010, 0x2297> ResponsiblePerson;
    typedef gdcm::Attribute<0x0010, 0x2298> ResponsiblePersonRole;
    typedef gdcm::Attribute<0x0010, 0x2299> ResponsibleOrganization;
    typedef gdcm::Attribute<0x0010, 0x4000> PatientComments;
    typedef gdcm::Attribute<0x0010, 0x9431> ExaminedBodyThickness;
    typedef gdcm::Attribute<0x0012, 0x0010> ClinicalTrialSponsorName;
    typedef gdcm::Attribute<0x0012, 0x0020> ClinicalTrialProtocolID;
    typedef gdcm::Attribute<0x0012, 0x0021> ClinicalTrialProtocolName;
    typedef gdcm::Attribute<0x0012, 0x0030> ClinicalTrialSiteID;
    typedef gdcm::Attribute<0x0012, 0x0031> ClinicalTrialSiteName;
    typedef gdcm::Attribute<0x0012, 0x0040> ClinicalTrialSubjectID;
    typedef gdcm::Attribute<0x0012, 0x0042> ClinicalTrialSubjectReadingID;
    typedef gdcm::Attribute<0x0012, 0x0050> ClinicalTrialTimePointID;
    typedef gdcm::Attribute<0x0012, 0x0051> ClinicalTrialTimePointDescription;
    typedef gdcm::Attribute<0x0012, 0x0060> ClinicalTrialCoordinatingCenterName;
    typedef gdcm::Attribute<0x0012, 0x0062> PatientIdentityRemoved;
    typedef gdcm::Attribute<0x0012, 0x0063> DeidentificationMethod;
    typedef gdcm::Attribute<0x0012, 0x0064> DeidentificationMethodCodeSequence;
    typedef gdcm::Attribute<0x0012, 0x0071> ClinicalTrialSeriesID;
    typedef gdcm::Attribute<0x0012, 0x0072> ClinicalTrialSeriesDescription;
    typedef gdcm::Attribute<0x0012, 0x0081> ClinicalTrialProtocolEthicsCommitteeName;
    typedef gdcm::Attribute<0x0012, 0x0082> ClinicalTrialProtocolEthicsCommitteeApprovalNumber;
    typedef gdcm::Attribute<0x0012, 0x0083> ConsentForClinicalTrialUseSequence;
    typedef gdcm::Attribute<0x0012, 0x0084> DistributionType;
    typedef gdcm::Attribute<0x0012, 0x0085> ConsentForDistributionFlag;
    typedef gdcm::Attribute<0x0014, 0x0023> CADFileFormat;
    typedef gdcm::Attribute<0x0014, 0x0024> ComponentReferenceSystem;
    typedef gdcm::Attribute<0x0014, 0x0025> ComponentManufacturingProcedure;
    typedef gdcm::Attribute<0x0014, 0x0028> ComponentManufacturer;
    typedef gdcm::Attribute<0x0014, 0x0030> MaterialThickness;
    typedef gdcm::Attribute<0x0014, 0x0032> MaterialPipeDiameter;
    typedef gdcm::Attribute<0x0014, 0x0034> MaterialIsolationDiameter;
    typedef gdcm::Attribute<0x0014, 0x0042> MaterialGrade;
    typedef gdcm::Attribute<0x0014, 0x0044> MaterialPropertiesFileID;
    typedef gdcm::Attribute<0x0014, 0x0045> MaterialPropertiesFileFormat;
    typedef gdcm::Attribute<0x0014, 0x0046> MaterialNotes;
    typedef gdcm::Attribute<0x0014, 0x0050> ComponentShape;
    typedef gdcm::Attribute<0x0014, 0x0052> CurvatureType;
    typedef gdcm::Attribute<0x0014, 0x0054> OuterDiameter;
    typedef gdcm::Attribute<0x0014, 0x0056> InnerDiameter;
    typedef gdcm::Attribute<0x0014, 0x1010> ActualEnvironmentalConditions;
    typedef gdcm::Attribute<0x0014, 0x1020> ExpiryDate;
    typedef gdcm::Attribute<0x0014, 0x1040> EnvironmentalConditions;
    typedef gdcm::Attribute<0x0014, 0x2002> EvaluatorSequence;
    typedef gdcm::Attribute<0x0014, 0x2004> EvaluatorNumber;
    typedef gdcm::Attribute<0x0014, 0x2006> EvaluatorName;
    typedef gdcm::Attribute<0x0014, 0x2008> EvaluationAttempt;
    typedef gdcm::Attribute<0x0014, 0x2012> IndicationSequence;
    typedef gdcm::Attribute<0x0014, 0x2014> IndicationNumber;
    typedef gdcm::Attribute<0x0014, 0x2016> IndicationLabel;
    typedef gdcm::Attribute<0x0014, 0x2018> IndicationDescription;
    typedef gdcm::Attribute<0x0014, 0x201a> IndicationType;
    typedef gdcm::Attribute<0x0014, 0x201c> IndicationDisposition;
    typedef gdcm::Attribute<0x0014, 0x201e> IndicationROISequence;
    typedef gdcm::Attribute<0x0014, 0x2030> IndicationPhysicalPropertySequence;
    typedef gdcm::Attribute<0x0014, 0x2032> PropertyLabel;
    typedef gdcm::Attribute<0x0014, 0x2202> CoordinateSystemNumberOfAxes;
    typedef gdcm::Attribute<0x0014, 0x2204> CoordinateSystemAxesSequence;
    typedef gdcm::Attribute<0x0014, 0x2206> CoordinateSystemAxisDescription;
    typedef gdcm::Attribute<0x0014, 0x2208> CoordinateSystemDataSetMapping;
    typedef gdcm::Attribute<0x0014, 0x220a> CoordinateSystemAxisNumber;
    typedef gdcm::Attribute<0x0014, 0x220c> CoordinateSystemAxisType;
    typedef gdcm::Attribute<0x0014, 0x220e> CoordinateSystemAxisUnits;
    typedef gdcm::Attribute<0x0014, 0x2210> CoordinateSystemAxisValues;
    typedef gdcm::Attribute<0x0014, 0x2220> CoordinateSystemTransformSequence;
    typedef gdcm::Attribute<0x0014, 0x2222> TransformDescription;
    typedef gdcm::Attribute<0x0014, 0x2224> TransformNumberOfAxes;
    typedef gdcm::Attribute<0x0014, 0x2226> TransformOrderOfAxes;
    typedef gdcm::Attribute<0x0014, 0x2228> TransformedAxisUnits;
    typedef gdcm::Attribute<0x0014, 0x222a> CoordinateSystemTransformRotationAndScaleMatrix;
    typedef gdcm::Attribute<0x0014, 0x222c> CoordinateSystemTransformTranslationMatrix;
    typedef gdcm::Attribute<0x0014, 0x3011> InternalDetectorFrameTime;
    typedef gdcm::Attribute<0x0014, 0x3012> NumberOfFramesIntegrated;
    typedef gdcm::Attribute<0x0014, 0x3020> DetectorTemperatureSequence;
    typedef gdcm::Attribute<0x0014, 0x3022> SensorName;
    typedef gdcm::Attribute<0x0014, 0x3024> HorizontalOffsetOfSensor;
    typedef gdcm::Attribute<0x0014, 0x3026> VerticalOffsetOfSensor;
    typedef gdcm::Attribute<0x0014, 0x3028> SensorTemperature;
    typedef gdcm::Attribute<0x0014, 0x3040> DarkCurrentSequence;
    typedef gdcm::Attribute<0x0014, 0x3050> DarkCurrentCounts;
    typedef gdcm::Attribute<0x0014, 0x3060> GainCorrectionReferenceSequence;
    typedef gdcm::Attribute<0x0014, 0x3070> AirCounts;
    typedef gdcm::Attribute<0x0014, 0x3071> KVUsedInGainCalibration;
    typedef gdcm::Attribute<0x0014, 0x3072> MAUsedInGainCalibration;
    typedef gdcm::Attribute<0x0014, 0x3073> NumberOfFramesUsedForIntegration;
    typedef gdcm::Attribute<0x0014, 0x3074> FilterMaterialUsedInGainCalibration;
    typedef gdcm::Attribute<0x0014, 0x3075> FilterThicknessUsedInGainCalibration;
    typedef gdcm::Attribute<0x0014, 0x3076> DateOfGainCalibration;
    typedef gdcm::Attribute<0x0014, 0x3077> TimeOfGainCalibration;
    typedef gdcm::Attribute<0x0014, 0x3080> BadPixelImage;
    typedef gdcm::Attribute<0x0014, 0x3099> CalibrationNotes;
    typedef gdcm::Attribute<0x0014, 0x4002> PulserEquipmentSequence;
    typedef gdcm::Attribute<0x0014, 0x4004> PulserType;
    typedef gdcm::Attribute<0x0014, 0x4006> PulserNotes;
    typedef gdcm::Attribute<0x0014, 0x4008> ReceiverEquipmentSequence;
    typedef gdcm::Attribute<0x0014, 0x400a> AmplifierType;
    typedef gdcm::Attribute<0x0014, 0x400c> ReceiverNotes;
    typedef gdcm::Attribute<0x0014, 0x400e> PreAmplifierEquipmentSequence;
    typedef gdcm::Attribute<0x0014, 0x400f> PreAmplifierNotes;
    typedef gdcm::Attribute<0x0014, 0x4010> TransmitTransducerSequence;
    typedef gdcm::Attribute<0x0014, 0x4011> ReceiveTransducerSequence;
    typedef gdcm::Attribute<0x0014, 0x4012> NumberOfElements;
    typedef gdcm::Attribute<0x0014, 0x4013> ElementShape;
    typedef gdcm::Attribute<0x0014, 0x4014> ElementDimensionA;
    typedef gdcm::Attribute<0x0014, 0x4015> ElementDimensionB;
    typedef gdcm::Attribute<0x0014, 0x4016> ElementPitch;
    typedef gdcm::Attribute<0x0014, 0x4017> MeasuredBeamDimensionA;
    typedef gdcm::Attribute<0x0014, 0x4018> MeasuredBeamDimensionB;
    typedef gdcm::Attribute<0x0014, 0x4019> LocationOfMeasuredBeamDiameter;
    typedef gdcm::Attribute<0x0014, 0x401a> NominalFrequency;
    typedef gdcm::Attribute<0x0014, 0x401b> MeasuredCenterFrequency;
    typedef gdcm::Attribute<0x0014, 0x401c> MeasuredBandwidth;
    typedef gdcm::Attribute<0x0014, 0x4020> PulserSettingsSequence;
    typedef gdcm::Attribute<0x0014, 0x4022> PulseWidth;
    typedef gdcm::Attribute<0x0014, 0x4024> ExcitationFrequency;
    typedef gdcm::Attribute<0x0014, 0x4026> ModulationType;
    typedef gdcm::Attribute<0x0014, 0x4028> Damping;
    typedef gdcm::Attribute<0x0014, 0x4030> ReceiverSettingsSequence;
    typedef gdcm::Attribute<0x0014, 0x4031> AcquiredSoundpathLength;
    typedef gdcm::Attribute<0x0014, 0x4032> AcquisitionCompressionType;
    typedef gdcm::Attribute<0x0014, 0x4033> AcquisitionSampleSize;
    typedef gdcm::Attribute<0x0014, 0x4034> RectifierSmoothing;
    typedef gdcm::Attribute<0x0014, 0x4035> DACSequence;
    typedef gdcm::Attribute<0x0014, 0x4036> DACType;
    typedef gdcm::Attribute<0x0014, 0x4038> DACGainPoints;
    typedef gdcm::Attribute<0x0014, 0x403a> DACTimePoints;
    typedef gdcm::Attribute<0x0014, 0x403c> DACAmplitude;
    typedef gdcm::Attribute<0x0014, 0x4040> PreAmplifierSettingsSequence;
    typedef gdcm::Attribute<0x0014, 0x4050> TransmitTransducerSettingsSequence;
    typedef gdcm::Attribute<0x0014, 0x4051> ReceiveTransducerSettingsSequence;
    typedef gdcm::Attribute<0x0014, 0x4052> IncidentAngle;
    typedef gdcm::Attribute<0x0014, 0x4054> CouplingTechnique;
    typedef gdcm::Attribute<0x0014, 0x4056> CouplingMedium;
    typedef gdcm::Attribute<0x0014, 0x4057> CouplingVelocity;
    typedef gdcm::Attribute<0x0014, 0x4058> CrystalCenterLocationX;
    typedef gdcm::Attribute<0x0014, 0x4059> CrystalCenterLocationZ;
    typedef gdcm::Attribute<0x0014, 0x405a> SoundPathLength;
    typedef gdcm::Attribute<0x0014, 0x405c> DelayLawIdentifier;
    typedef gdcm::Attribute<0x0014, 0x4060> GateSettingsSequence;
    typedef gdcm::Attribute<0x0014, 0x4062> GateThreshold;
    typedef gdcm::Attribute<0x0014, 0x4064> VelocityOfSound;
    typedef gdcm::Attribute<0x0014, 0x4070> CalibrationSettingsSequence;
    typedef gdcm::Attribute<0x0014, 0x4072> CalibrationProcedure;
    typedef gdcm::Attribute<0x0014, 0x4074> ProcedureVersion;
    typedef gdcm::Attribute<0x0014, 0x4076> ProcedureCreationDate;
    typedef gdcm::Attribute<0x0014, 0x4078> ProcedureExpirationDate;
    typedef gdcm::Attribute<0x0014, 0x407a> ProcedureLastModifiedDate;
    typedef gdcm::Attribute<0x0014, 0x407c> CalibrationTime;
    typedef gdcm::Attribute<0x0014, 0x407e> CalibrationDate;
    typedef gdcm::Attribute<0x0014, 0x5002> LINACEnergy;
    typedef gdcm::Attribute<0x0014, 0x5004> LINACOutput;
    typedef gdcm::Attribute<0x0018, 0x0010> ContrastBolusAgent;
    typedef gdcm::Attribute<0x0018, 0x0012> ContrastBolusAgentSequence;
    typedef gdcm::Attribute<0x0018, 0x0014> ContrastBolusAdministrationRouteSequence;
    typedef gdcm::Attribute<0x0018, 0x0015> BodyPartExamined;
    typedef gdcm::Attribute<0x0018, 0x0020> ScanningSequence;
    typedef gdcm::Attribute<0x0018, 0x0021> SequenceVariant;
    typedef gdcm::Attribute<0x0018, 0x0022> ScanOptions;
    typedef gdcm::Attribute<0x0018, 0x0023> MRAcquisitionType;
    typedef gdcm::Attribute<0x0018, 0x0024> SequenceName;
    typedef gdcm::Attribute<0x0018, 0x0025> AngioFlag;
    typedef gdcm::Attribute<0x0018, 0x0026> InterventionDrugInformationSequence;
    typedef gdcm::Attribute<0x0018, 0x0027> InterventionDrugStopTime;
    typedef gdcm::Attribute<0x0018, 0x0028> InterventionDrugDose;
    typedef gdcm::Attribute<0x0018, 0x0029> InterventionDrugCodeSequence;
    typedef gdcm::Attribute<0x0018, 0x002a> AdditionalDrugSequence;
    typedef gdcm::Attribute<0x0018, 0x0030> Radionuclide;
    typedef gdcm::Attribute<0x0018, 0x0031> Radiopharmaceutical;
    typedef gdcm::Attribute<0x0018, 0x0032> EnergyWindowCenterline;
    typedef gdcm::Attribute<0x0018, 0x0033> EnergyWindowTotalWidth;
    typedef gdcm::Attribute<0x0018, 0x0034> InterventionDrugName;
    typedef gdcm::Attribute<0x0018, 0x0035> InterventionDrugStartTime;
    typedef gdcm::Attribute<0x0018, 0x0036> InterventionSequence;
    typedef gdcm::Attribute<0x0018, 0x0037> TherapyType;
    typedef gdcm::Attribute<0x0018, 0x0038> InterventionStatus;
    typedef gdcm::Attribute<0x0018, 0x0039> TherapyDescription;
    typedef gdcm::Attribute<0x0018, 0x003a> InterventionDescription;
    typedef gdcm::Attribute<0x0018, 0x0040> CineRate;
    typedef gdcm::Attribute<0x0018, 0x0042> InitialCineRunState;
    typedef gdcm::Attribute<0x0018, 0x0050> SliceThickness;
    typedef gdcm::Attribute<0x0018, 0x0060> KVP;
    typedef gdcm::Attribute<0x0018, 0x0070> CountsAccumulated;
    typedef gdcm::Attribute<0x0018, 0x0071> AcquisitionTerminationCondition;
    typedef gdcm::Attribute<0x0018, 0x0072> EffectiveDuration;
    typedef gdcm::Attribute<0x0018, 0x0073> AcquisitionStartCondition;
    typedef gdcm::Attribute<0x0018, 0x0074> AcquisitionStartConditionData;
    typedef gdcm::Attribute<0x0018, 0x0075> AcquisitionTerminationConditionData;
    typedef gdcm::Attribute<0x0018, 0x0080> RepetitionTime;
    typedef gdcm::Attribute<0x0018, 0x0081> EchoTime;
    typedef gdcm::Attribute<0x0018, 0x0082> InversionTime;
    typedef gdcm::Attribute<0x0018, 0x0083> NumberOfAverages;
    typedef gdcm::Attribute<0x0018, 0x0084> ImagingFrequency;
    typedef gdcm::Attribute<0x0018, 0x0085> ImagedNucleus;
    typedef gdcm::Attribute<0x0018, 0x0086> EchoNumbers;
    typedef gdcm::Attribute<0x0018, 0x0087> MagneticFieldStrength;
    typedef gdcm::Attribute<0x0018, 0x0088> SpacingBetweenSlices;
    typedef gdcm::Attribute<0x0018, 0x0089> NumberOfPhaseEncodingSteps;
    typedef gdcm::Attribute<0x0018, 0x0090> DataCollectionDiameter;
    typedef gdcm::Attribute<0x0018, 0x0091> EchoTrainLength;
    typedef gdcm::Attribute<0x0018, 0x0093> PercentSampling;
    typedef gdcm::Attribute<0x0018, 0x0094> PercentPhaseFieldOfView;
    typedef gdcm::Attribute<0x0018, 0x0095> PixelBandwidth;
    typedef gdcm::Attribute<0x0018, 0x1000> DeviceSerialNumber;
    typedef gdcm::Attribute<0x0018, 0x1002> DeviceUID;
    typedef gdcm::Attribute<0x0018, 0x1003> DeviceID;
    typedef gdcm::Attribute<0x0018, 0x1004> PlateID;
    typedef gdcm::Attribute<0x0018, 0x1005> GeneratorID;
    typedef gdcm::Attribute<0x0018, 0x1006> GridID;
    typedef gdcm::Attribute<0x0018, 0x1007> CassetteID;
    typedef gdcm::Attribute<0x0018, 0x1008> GantryID;
    typedef gdcm::Attribute<0x0018, 0x1010> SecondaryCaptureDeviceID;
    typedef gdcm::Attribute<0x0018, 0x1011> HardcopyCreationDeviceID;
    typedef gdcm::Attribute<0x0018, 0x1012> DateOfSecondaryCapture;
    typedef gdcm::Attribute<0x0018, 0x1014> TimeOfSecondaryCapture;
    typedef gdcm::Attribute<0x0018, 0x1016> SecondaryCaptureDeviceManufacturer;
    typedef gdcm::Attribute<0x0018, 0x1017> HardcopyDeviceManufacturer;
    typedef gdcm::Attribute<0x0018, 0x1018> SecondaryCaptureDeviceManufacturerModelName;
    typedef gdcm::Attribute<0x0018, 0x1019> SecondaryCaptureDeviceSoftwareVersions;
    typedef gdcm::Attribute<0x0018, 0x101a> HardcopyDeviceSoftwareVersion;
    typedef gdcm::Attribute<0x0018, 0x101b> HardcopyDeviceManufacturerModelName;
    typedef gdcm::Attribute<0x0018, 0x1020> SoftwareVersions;
    typedef gdcm::Attribute<0x0018, 0x1022> VideoImageFormatAcquired;
    typedef gdcm::Attribute<0x0018, 0x1023> DigitalImageFormatAcquired;
    typedef gdcm::Attribute<0x0018, 0x1030> ProtocolName;
    typedef gdcm::Attribute<0x0018, 0x1040> ContrastBolusRoute;
    typedef gdcm::Attribute<0x0018, 0x1041> ContrastBolusVolume;
    typedef gdcm::Attribute<0x0018, 0x1042> ContrastBolusStartTime;
    typedef gdcm::Attribute<0x0018, 0x1043> ContrastBolusStopTime;
    typedef gdcm::Attribute<0x0018, 0x1044> ContrastBolusTotalDose;
    typedef gdcm::Attribute<0x0018, 0x1045> SyringeCounts;
    typedef gdcm::Attribute<0x0018, 0x1046> ContrastFlowRate;
    typedef gdcm::Attribute<0x0018, 0x1047> ContrastFlowDuration;
    typedef gdcm::Attribute<0x0018, 0x1048> ContrastBolusIngredient;
    typedef gdcm::Attribute<0x0018, 0x1049> ContrastBolusIngredientConcentration;
    typedef gdcm::Attribute<0x0018, 0x1050> SpatialResolution;
    typedef gdcm::Attribute<0x0018, 0x1060> TriggerTime;
    typedef gdcm::Attribute<0x0018, 0x1061> TriggerSourceOrType;
    typedef gdcm::Attribute<0x0018, 0x1062> NominalInterval;
    typedef gdcm::Attribute<0x0018, 0x1063> FrameTime;
    typedef gdcm::Attribute<0x0018, 0x1064> CardiacFramingType;
    typedef gdcm::Attribute<0x0018, 0x1065> FrameTimeVector;
    typedef gdcm::Attribute<0x0018, 0x1066> FrameDelay;
    typedef gdcm::Attribute<0x0018, 0x1067> ImageTriggerDelay;
    typedef gdcm::Attribute<0x0018, 0x1068> MultiplexGroupTimeOffset;
    typedef gdcm::Attribute<0x0018, 0x1069> TriggerTimeOffset;
    typedef gdcm::Attribute<0x0018, 0x106a> SynchronizationTrigger;
    typedef gdcm::Attribute<0x0018, 0x106c> SynchronizationChannel;
    typedef gdcm::Attribute<0x0018, 0x106e> TriggerSamplePosition;
    typedef gdcm::Attribute<0x0018, 0x1070> RadiopharmaceuticalRoute;
    typedef gdcm::Attribute<0x0018, 0x1071> RadiopharmaceuticalVolume;
    typedef gdcm::Attribute<0x0018, 0x1072> RadiopharmaceuticalStartTime;
    typedef gdcm::Attribute<0x0018, 0x1073> RadiopharmaceuticalStopTime;
    typedef gdcm::Attribute<0x0018, 0x1074> RadionuclideTotalDose;
    typedef gdcm::Attribute<0x0018, 0x1075> RadionuclideHalfLife;
    typedef gdcm::Attribute<0x0018, 0x1076> RadionuclidePositronFraction;
    typedef gdcm::Attribute<0x0018, 0x1077> RadiopharmaceuticalSpecificActivity;
    typedef gdcm::Attribute<0x0018, 0x1078> RadiopharmaceuticalStartDateTime;
    typedef gdcm::Attribute<0x0018, 0x1079> RadiopharmaceuticalStopDateTime;
    typedef gdcm::Attribute<0x0018, 0x1080> BeatRejectionFlag;
    typedef gdcm::Attribute<0x0018, 0x1081> LowRRValue;
    typedef gdcm::Attribute<0x0018, 0x1082> HighRRValue;
    typedef gdcm::Attribute<0x0018, 0x1083> IntervalsAcquired;
    typedef gdcm::Attribute<0x0018, 0x1084> IntervalsRejected;
    typedef gdcm::Attribute<0x0018, 0x1085> PVCRejection;
    typedef gdcm::Attribute<0x0018, 0x1086> SkipBeats;
    typedef gdcm::Attribute<0x0018, 0x1088> HeartRate;
    typedef gdcm::Attribute<0x0018, 0x1090> CardiacNumberOfImages;
    typedef gdcm::Attribute<0x0018, 0x1094> TriggerWindow;
    typedef gdcm::Attribute<0x0018, 0x1100> ReconstructionDiameter;
    typedef gdcm::Attribute<0x0018, 0x1110> DistanceSourceToDetector;
    typedef gdcm::Attribute<0x0018, 0x1111> DistanceSourceToPatient;
    typedef gdcm::Attribute<0x0018, 0x1114> EstimatedRadiographicMagnificationFactor;
    typedef gdcm::Attribute<0x0018, 0x1120> GantryDetectorTilt;
    typedef gdcm::Attribute<0x0018, 0x1121> GantryDetectorSlew;
    typedef gdcm::Attribute<0x0018, 0x1130> TableHeight;
    typedef gdcm::Attribute<0x0018, 0x1131> TableTraverse;
    typedef gdcm::Attribute<0x0018, 0x1134> TableMotion;
    typedef gdcm::Attribute<0x0018, 0x1135> TableVerticalIncrement;
    typedef gdcm::Attribute<0x0018, 0x1136> TableLateralIncrement;
    typedef gdcm::Attribute<0x0018, 0x1137> TableLongitudinalIncrement;
    typedef gdcm::Attribute<0x0018, 0x1138> TableAngle;
    typedef gdcm::Attribute<0x0018, 0x113a> TableType;
    typedef gdcm::Attribute<0x0018, 0x1140> RotationDirection;
    typedef gdcm::Attribute<0x0018, 0x1141> AngularPosition;
    typedef gdcm::Attribute<0x0018, 0x1142> RadialPosition;
    typedef gdcm::Attribute<0x0018, 0x1143> ScanArc;
    typedef gdcm::Attribute<0x0018, 0x1144> AngularStep;
    typedef gdcm::Attribute<0x0018, 0x1145> CenterOfRotationOffset;
    typedef gdcm::Attribute<0x0018, 0x1146> RotationOffset;
    typedef gdcm::Attribute<0x0018, 0x1147> FieldOfViewShape;
    typedef gdcm::Attribute<0x0018, 0x1149> FieldOfViewDimensions;
    typedef gdcm::Attribute<0x0018, 0x1150> ExposureTime;
    typedef gdcm::Attribute<0x0018, 0x1151> XRayTubeCurrent;
    typedef gdcm::Attribute<0x0018, 0x1152> Exposure;
    typedef gdcm::Attribute<0x0018, 0x1153> ExposureInuAs;
    typedef gdcm::Attribute<0x0018, 0x1154> AveragePulseWidth;
    typedef gdcm::Attribute<0x0018, 0x1155> RadiationSetting;
    typedef gdcm::Attribute<0x0018, 0x1156> RectificationType;
    typedef gdcm::Attribute<0x0018, 0x115a> RadiationMode;
    typedef gdcm::Attribute<0x0018, 0x115e> ImageAndFluoroscopyAreaDoseProduct;
    typedef gdcm::Attribute<0x0018, 0x1160> FilterType;
    typedef gdcm::Attribute<0x0018, 0x1161> TypeOfFilters;
    typedef gdcm::Attribute<0x0018, 0x1162> IntensifierSize;
    typedef gdcm::Attribute<0x0018, 0x1164> ImagerPixelSpacing;
    typedef gdcm::Attribute<0x0018, 0x1166> Grid;
    typedef gdcm::Attribute<0x0018, 0x1170> GeneratorPower;
    typedef gdcm::Attribute<0x0018, 0x1180> CollimatorGridName;
    typedef gdcm::Attribute<0x0018, 0x1181> CollimatorType;
    typedef gdcm::Attribute<0x0018, 0x1182> FocalDistance;
    typedef gdcm::Attribute<0x0018, 0x1183> XFocusCenter;
    typedef gdcm::Attribute<0x0018, 0x1184> YFocusCenter;
    typedef gdcm::Attribute<0x0018, 0x1190> FocalSpots;
    typedef gdcm::Attribute<0x0018, 0x1191> AnodeTargetMaterial;
    typedef gdcm::Attribute<0x0018, 0x11a0> BodyPartThickness;
    typedef gdcm::Attribute<0x0018, 0x11a2> CompressionForce;
    typedef gdcm::Attribute<0x0018, 0x1200> DateOfLastCalibration;
    typedef gdcm::Attribute<0x0018, 0x1201> TimeOfLastCalibration;
    typedef gdcm::Attribute<0x0018, 0x1210> ConvolutionKernel;
    typedef gdcm::Attribute<0x0018, 0x1240> UpperLowerPixelValues;
    typedef gdcm::Attribute<0x0018, 0x1242> ActualFrameDuration;
    typedef gdcm::Attribute<0x0018, 0x1243> CountRate;
    typedef gdcm::Attribute<0x0018, 0x1244> PreferredPlaybackSequencing;
    typedef gdcm::Attribute<0x0018, 0x1250> ReceiveCoilName;
    typedef gdcm::Attribute<0x0018, 0x1251> TransmitCoilName;
    typedef gdcm::Attribute<0x0018, 0x1260> PlateType;
    typedef gdcm::Attribute<0x0018, 0x1261> PhosphorType;
    typedef gdcm::Attribute<0x0018, 0x1300> ScanVelocity;
    typedef gdcm::Attribute<0x0018, 0x1301> WholeBodyTechnique;
    typedef gdcm::Attribute<0x0018, 0x1302> ScanLength;
    typedef gdcm::Attribute<0x0018, 0x1310> AcquisitionMatrix;
    typedef gdcm::Attribute<0x0018, 0x1312> InPlanePhaseEncodingDirection;
    typedef gdcm::Attribute<0x0018, 0x1314> FlipAngle;
    typedef gdcm::Attribute<0x0018, 0x1315> VariableFlipAngleFlag;
    typedef gdcm::Attribute<0x0018, 0x1316> SAR;
    typedef gdcm::Attribute<0x0018, 0x1318> dBdt;
    typedef gdcm::Attribute<0x0018, 0x1400> AcquisitionDeviceProcessingDescription;
    typedef gdcm::Attribute<0x0018, 0x1401> AcquisitionDeviceProcessingCode;
    typedef gdcm::Attribute<0x0018, 0x1402> CassetteOrientation;
    typedef gdcm::Attribute<0x0018, 0x1403> CassetteSize;
    typedef gdcm::Attribute<0x0018, 0x1404> ExposuresOnPlate;
    typedef gdcm::Attribute<0x0018, 0x1405> RelativeXRayExposure;
    typedef gdcm::Attribute<0x0018, 0x1411> ExposureIndex;
    typedef gdcm::Attribute<0x0018, 0x1412> TargetExposureIndex;
    typedef gdcm::Attribute<0x0018, 0x1413> DeviationIndex;
    typedef gdcm::Attribute<0x0018, 0x1450> ColumnAngulation;
    typedef gdcm::Attribute<0x0018, 0x1460> TomoLayerHeight;
    typedef gdcm::Attribute<0x0018, 0x1470> TomoAngle;
    typedef gdcm::Attribute<0x0018, 0x1480> TomoTime;
    typedef gdcm::Attribute<0x0018, 0x1490> TomoType;
    typedef gdcm::Attribute<0x0018, 0x1491> TomoClass;
    typedef gdcm::Attribute<0x0018, 0x1495> NumberOfTomosynthesisSourceImages;
    typedef gdcm::Attribute<0x0018, 0x1500> PositionerMotion;
    typedef gdcm::Attribute<0x0018, 0x1508> PositionerType;
    typedef gdcm::Attribute<0x0018, 0x1510> PositionerPrimaryAngle;
    typedef gdcm::Attribute<0x0018, 0x1511> PositionerSecondaryAngle;
    typedef gdcm::Attribute<0x0018, 0x1520> PositionerPrimaryAngleIncrement;
    typedef gdcm::Attribute<0x0018, 0x1521> PositionerSecondaryAngleIncrement;
    typedef gdcm::Attribute<0x0018, 0x1530> DetectorPrimaryAngle;
    typedef gdcm::Attribute<0x0018, 0x1531> DetectorSecondaryAngle;
    typedef gdcm::Attribute<0x0018, 0x1600> ShutterShape;
    typedef gdcm::Attribute<0x0018, 0x1602> ShutterLeftVerticalEdge;
    typedef gdcm::Attribute<0x0018, 0x1604> ShutterRightVerticalEdge;
    typedef gdcm::Attribute<0x0018, 0x1606> ShutterUpperHorizontalEdge;
    typedef gdcm::Attribute<0x0018, 0x1608> ShutterLowerHorizontalEdge;
    typedef gdcm::Attribute<0x0018, 0x1610> CenterOfCircularShutter;
    typedef gdcm::Attribute<0x0018, 0x1612> RadiusOfCircularShutter;
    typedef gdcm::Attribute<0x0018, 0x1620> VerticesOfThePolygonalShutter;
    typedef gdcm::Attribute<0x0018, 0x1622> ShutterPresentationValue;
    typedef gdcm::Attribute<0x0018, 0x1623> ShutterOverlayGroup;
    typedef gdcm::Attribute<0x0018, 0x1624> ShutterPresentationColorCIELabValue;
    typedef gdcm::Attribute<0x0018, 0x1700> CollimatorShape;
    typedef gdcm::Attribute<0x0018, 0x1702> CollimatorLeftVerticalEdge;
    typedef gdcm::Attribute<0x0018, 0x1704> CollimatorRightVerticalEdge;
    typedef gdcm::Attribute<0x0018, 0x1706> CollimatorUpperHorizontalEdge;
    typedef gdcm::Attribute<0x0018, 0x1708> CollimatorLowerHorizontalEdge;
    typedef gdcm::Attribute<0x0018, 0x1710> CenterOfCircularCollimator;
    typedef gdcm::Attribute<0x0018, 0x1712> RadiusOfCircularCollimator;
    typedef gdcm::Attribute<0x0018, 0x1720> VerticesOfThePolygonalCollimator;
    typedef gdcm::Attribute<0x0018, 0x1800> AcquisitionTimeSynchronized;
    typedef gdcm::Attribute<0x0018, 0x1801> TimeSource;
    typedef gdcm::Attribute<0x0018, 0x1802> TimeDistributionProtocol;
    typedef gdcm::Attribute<0x0018, 0x1803> NTPSourceAddress;
    typedef gdcm::Attribute<0x0018, 0x2001> PageNumberVector;
    typedef gdcm::Attribute<0x0018, 0x2002> FrameLabelVector;
    typedef gdcm::Attribute<0x0018, 0x2003> FramePrimaryAngleVector;
    typedef gdcm::Attribute<0x0018, 0x2004> FrameSecondaryAngleVector;
    typedef gdcm::Attribute<0x0018, 0x2005> SliceLocationVector;
    typedef gdcm::Attribute<0x0018, 0x2006> DisplayWindowLabelVector;
    typedef gdcm::Attribute<0x0018, 0x2010> NominalScannedPixelSpacing;
    typedef gdcm::Attribute<0x0018, 0x2020> DigitizingDeviceTransportDirection;
    typedef gdcm::Attribute<0x0018, 0x2030> RotationOfScannedFilm;
    typedef gdcm::Attribute<0x0018, 0x3100> IVUSAcquisition;
    typedef gdcm::Attribute<0x0018, 0x3101> IVUSPullbackRate;
    typedef gdcm::Attribute<0x0018, 0x3102> IVUSGatedRate;
    typedef gdcm::Attribute<0x0018, 0x3103> IVUSPullbackStartFrameNumber;
    typedef gdcm::Attribute<0x0018, 0x3104> IVUSPullbackStopFrameNumber;
    typedef gdcm::Attribute<0x0018, 0x3105> LesionNumber;
    typedef gdcm::Attribute<0x0018, 0x4000> AcquisitionComments;
    typedef gdcm::Attribute<0x0018, 0x5000> OutputPower;
    typedef gdcm::Attribute<0x0018, 0x5010> TransducerData;
    typedef gdcm::Attribute<0x0018, 0x5012> FocusDepth;
    typedef gdcm::Attribute<0x0018, 0x5020> ProcessingFunction;
    typedef gdcm::Attribute<0x0018, 0x5021> PostprocessingFunction;
    typedef gdcm::Attribute<0x0018, 0x5022> MechanicalIndex;
    typedef gdcm::Attribute<0x0018, 0x5024> BoneThermalIndex;
    typedef gdcm::Attribute<0x0018, 0x5026> CranialThermalIndex;
    typedef gdcm::Attribute<0x0018, 0x5027> SoftTissueThermalIndex;
    typedef gdcm::Attribute<0x0018, 0x5028> SoftTissueFocusThermalIndex;
    typedef gdcm::Attribute<0x0018, 0x5029> SoftTissueSurfaceThermalIndex;
    typedef gdcm::Attribute<0x0018, 0x5030> DynamicRange;
    typedef gdcm::Attribute<0x0018, 0x5040> TotalGain;
    typedef gdcm::Attribute<0x0018, 0x5050> DepthOfScanField;
    typedef gdcm::Attribute<0x0018, 0x5100> PatientPosition;
    typedef gdcm::Attribute<0x0018, 0x5101> ViewPosition;
    typedef gdcm::Attribute<0x0018, 0x5104> ProjectionEponymousNameCodeSequence;
    typedef gdcm::Attribute<0x0018, 0x5210> ImageTransformationMatrix;
    typedef gdcm::Attribute<0x0018, 0x5212> ImageTranslationVector;
    typedef gdcm::Attribute<0x0018, 0x6000> Sensitivity;
    typedef gdcm::Attribute<0x0018, 0x6011> SequenceOfUltrasoundRegions;
    typedef gdcm::Attribute<0x0018, 0x6012> RegionSpatialFormat;
    typedef gdcm::Attribute<0x0018, 0x6014> RegionDataType;
    typedef gdcm::Attribute<0x0018, 0x6016> RegionFlags;
    typedef gdcm::Attribute<0x0018, 0x6018> RegionLocationMinX0;
    typedef gdcm::Attribute<0x0018, 0x601a> RegionLocationMinY0;
    typedef gdcm::Attribute<0x0018, 0x601c> RegionLocationMaxX1;
    typedef gdcm::Attribute<0x0018, 0x601e> RegionLocationMaxY1;
    typedef gdcm::Attribute<0x0018, 0x6020> ReferencePixelX0;
    typedef gdcm::Attribute<0x0018, 0x6022> ReferencePixelY0;
    typedef gdcm::Attribute<0x0018, 0x6024> PhysicalUnitsXDirection;
    typedef gdcm::Attribute<0x0018, 0x6026> PhysicalUnitsYDirection;
    typedef gdcm::Attribute<0x0018, 0x6028> ReferencePixelPhysicalValueX;
    typedef gdcm::Attribute<0x0018, 0x602a> ReferencePixelPhysicalValueY;
    typedef gdcm::Attribute<0x0018, 0x602c> PhysicalDeltaX;
    typedef gdcm::Attribute<0x0018, 0x602e> PhysicalDeltaY;
    typedef gdcm::Attribute<0x0018, 0x6030> TransducerFrequency;
    typedef gdcm::Attribute<0x0018, 0x6031> TransducerType;
    typedef gdcm::Attribute<0x0018, 0x6032> PulseRepetitionFrequency;
    typedef gdcm::Attribute<0x0018, 0x6034> DopplerCorrectionAngle;
    typedef gdcm::Attribute<0x0018, 0x6036> SteeringAngle;
    typedef gdcm::Attribute<0x0018, 0x6038> DopplerSampleVolumeXPositionRetired;
    typedef gdcm::Attribute<0x0018, 0x6039> DopplerSampleVolumeXPosition;
    typedef gdcm::Attribute<0x0018, 0x603a> DopplerSampleVolumeYPositionRetired;
    typedef gdcm::Attribute<0x0018, 0x603b> DopplerSampleVolumeYPosition;
    typedef gdcm::Attribute<0x0018, 0x603c> TMLinePositionX0Retired;
    typedef gdcm::Attribute<0x0018, 0x603d> TMLinePositionX0;
    typedef gdcm::Attribute<0x0018, 0x603e> TMLinePositionY0Retired;
    typedef gdcm::Attribute<0x0018, 0x603f> TMLinePositionY0;
    typedef gdcm::Attribute<0x0018, 0x6040> TMLinePositionX1Retired;
    typedef gdcm::Attribute<0x0018, 0x6041> TMLinePositionX1;
    typedef gdcm::Attribute<0x0018, 0x6042> TMLinePositionY1Retired;
    typedef gdcm::Attribute<0x0018, 0x6043> TMLinePositionY1;
    typedef gdcm::Attribute<0x0018, 0x6044> PixelComponentOrganization;
    typedef gdcm::Attribute<0x0018, 0x6046> PixelComponentMask;
    typedef gdcm::Attribute<0x0018, 0x6048> PixelComponentRangeStart;
    typedef gdcm::Attribute<0x0018, 0x604a> PixelComponentRangeStop;
    typedef gdcm::Attribute<0x0018, 0x604c> PixelComponentPhysicalUnits;
    typedef gdcm::Attribute<0x0018, 0x604e> PixelComponentDataType;
    typedef gdcm::Attribute<0x0018, 0x6050> NumberOfTableBreakPoints;
    typedef gdcm::Attribute<0x0018, 0x6052> TableOfXBreakPoints;
    typedef gdcm::Attribute<0x0018, 0x6054> TableOfYBreakPoints;
    typedef gdcm::Attribute<0x0018, 0x6056> NumberOfTableEntries;
    typedef gdcm::Attribute<0x0018, 0x6058> TableOfPixelValues;
    typedef gdcm::Attribute<0x0018, 0x605a> TableOfParameterValues;
    typedef gdcm::Attribute<0x0018, 0x6060> RWaveTimeVector;
    typedef gdcm::Attribute<0x0018, 0x7000> DetectorConditionsNominalFlag;
    typedef gdcm::Attribute<0x0018, 0x7001> DetectorTemperature;
    typedef gdcm::Attribute<0x0018, 0x7004> DetectorType;
    typedef gdcm::Attribute<0x0018, 0x7005> DetectorConfiguration;
    typedef gdcm::Attribute<0x0018, 0x7006> DetectorDescription;
    typedef gdcm::Attribute<0x0018, 0x7008> DetectorMode;
    typedef gdcm::Attribute<0x0018, 0x700a> DetectorID;
    typedef gdcm::Attribute<0x0018, 0x700c> DateOfLastDetectorCalibration;
    typedef gdcm::Attribute<0x0018, 0x700e> TimeOfLastDetectorCalibration;
    typedef gdcm::Attribute<0x0018, 0x7010> ExposuresOnDetectorSinceLastCalibration;
    typedef gdcm::Attribute<0x0018, 0x7011> ExposuresOnDetectorSinceManufactured;
    typedef gdcm::Attribute<0x0018, 0x7012> DetectorTimeSinceLastExposure;
    typedef gdcm::Attribute<0x0018, 0x7014> DetectorActiveTime;
    typedef gdcm::Attribute<0x0018, 0x7016> DetectorActivationOffsetFromExposure;
    typedef gdcm::Attribute<0x0018, 0x701a> DetectorBinning;
    typedef gdcm::Attribute<0x0018, 0x7020> DetectorElementPhysicalSize;
    typedef gdcm::Attribute<0x0018, 0x7022> DetectorElementSpacing;
    typedef gdcm::Attribute<0x0018, 0x7024> DetectorActiveShape;
    typedef gdcm::Attribute<0x0018, 0x7026> DetectorActiveDimensions;
    typedef gdcm::Attribute<0x0018, 0x7028> DetectorActiveOrigin;
    typedef gdcm::Attribute<0x0018, 0x702a> DetectorManufacturerName;
    typedef gdcm::Attribute<0x0018, 0x702b> DetectorManufacturerModelName;
    typedef gdcm::Attribute<0x0018, 0x7030> FieldOfViewOrigin;
    typedef gdcm::Attribute<0x0018, 0x7032> FieldOfViewRotation;
    typedef gdcm::Attribute<0x0018, 0x7034> FieldOfViewHorizontalFlip;
    typedef gdcm::Attribute<0x0018, 0x7036> PixelDataAreaOriginRelativeToFOV;
    typedef gdcm::Attribute<0x0018, 0x7038> PixelDataAreaRotationAngleRelativeToFOV;
    typedef gdcm::Attribute<0x0018, 0x7040> GridAbsorbingMaterial;
    typedef gdcm::Attribute<0x0018, 0x7041> GridSpacingMaterial;
    typedef gdcm::Attribute<0x0018, 0x7042> GridThickness;
    typedef gdcm::Attribute<0x0018, 0x7044> GridPitch;
    typedef gdcm::Attribute<0x0018, 0x7046> GridAspectRatio;
    typedef gdcm::Attribute<0x0018, 0x7048> GridPeriod;
    typedef gdcm::Attribute<0x0018, 0x704c> GridFocalDistance;
    typedef gdcm::Attribute<0x0018, 0x7050> FilterMaterial;
    typedef gdcm::Attribute<0x0018, 0x7052> FilterThicknessMinimum;
    typedef gdcm::Attribute<0x0018, 0x7054> FilterThicknessMaximum;
    typedef gdcm::Attribute<0x0018, 0x7056> FilterBeamPathLengthMinimum;
    typedef gdcm::Attribute<0x0018, 0x7058> FilterBeamPathLengthMaximum;
    typedef gdcm::Attribute<0x0018, 0x7060> ExposureControlMode;
    typedef gdcm::Attribute<0x0018, 0x7062> ExposureControlModeDescription;
    typedef gdcm::Attribute<0x0018, 0x7064> ExposureStatus;
    typedef gdcm::Attribute<0x0018, 0x7065> PhototimerSetting;
    typedef gdcm::Attribute<0x0018, 0x8150> ExposureTimeInuS;
    typedef gdcm::Attribute<0x0018, 0x8151> XRayTubeCurrentInuA;
    typedef gdcm::Attribute<0x0018, 0x9004> ContentQualification;
    typedef gdcm::Attribute<0x0018, 0x9005> PulseSequenceName;
    typedef gdcm::Attribute<0x0018, 0x9006> MRImagingModifierSequence;
    typedef gdcm::Attribute<0x0018, 0x9008> EchoPulseSequence;
    typedef gdcm::Attribute<0x0018, 0x9009> InversionRecovery;
    typedef gdcm::Attribute<0x0018, 0x9010> FlowCompensation;
    typedef gdcm::Attribute<0x0018, 0x9011> MultipleSpinEcho;
    typedef gdcm::Attribute<0x0018, 0x9012> MultiPlanarExcitation;
    typedef gdcm::Attribute<0x0018, 0x9014> PhaseContrast;
    typedef gdcm::Attribute<0x0018, 0x9015> TimeOfFlightContrast;
    typedef gdcm::Attribute<0x0018, 0x9016> Spoiling;
    typedef gdcm::Attribute<0x0018, 0x9017> SteadyStatePulseSequence;
    typedef gdcm::Attribute<0x0018, 0x9018> EchoPlanarPulseSequence;
    typedef gdcm::Attribute<0x0018, 0x9019> TagAngleFirstAxis;
    typedef gdcm::Attribute<0x0018, 0x9020> MagnetizationTransfer;
    typedef gdcm::Attribute<0x0018, 0x9021> T2Preparation;
    typedef gdcm::Attribute<0x0018, 0x9022> BloodSignalNulling;
    typedef gdcm::Attribute<0x0018, 0x9024> SaturationRecovery;
    typedef gdcm::Attribute<0x0018, 0x9025> SpectrallySelectedSuppression;
    typedef gdcm::Attribute<0x0018, 0x9026> SpectrallySelectedExcitation;
    typedef gdcm::Attribute<0x0018, 0x9027> SpatialPresaturation;
    typedef gdcm::Attribute<0x0018, 0x9028> Tagging;
    typedef gdcm::Attribute<0x0018, 0x9029> OversamplingPhase;
    typedef gdcm::Attribute<0x0018, 0x9030> TagSpacingFirstDimension;
    typedef gdcm::Attribute<0x0018, 0x9032> GeometryOfKSpaceTraversal;
    typedef gdcm::Attribute<0x0018, 0x9033> SegmentedKSpaceTraversal;
    typedef gdcm::Attribute<0x0018, 0x9034> RectilinearPhaseEncodeReordering;
    typedef gdcm::Attribute<0x0018, 0x9035> TagThickness;
    typedef gdcm::Attribute<0x0018, 0x9036> PartialFourierDirection;
    typedef gdcm::Attribute<0x0018, 0x9037> CardiacSynchronizationTechnique;
    typedef gdcm::Attribute<0x0018, 0x9041> ReceiveCoilManufacturerName;
    typedef gdcm::Attribute<0x0018, 0x9042> MRReceiveCoilSequence;
    typedef gdcm::Attribute<0x0018, 0x9043> ReceiveCoilType;
    typedef gdcm::Attribute<0x0018, 0x9044> QuadratureReceiveCoil;
    typedef gdcm::Attribute<0x0018, 0x9045> MultiCoilDefinitionSequence;
    typedef gdcm::Attribute<0x0018, 0x9046> MultiCoilConfiguration;
    typedef gdcm::Attribute<0x0018, 0x9047> MultiCoilElementName;
    typedef gdcm::Attribute<0x0018, 0x9048> MultiCoilElementUsed;
    typedef gdcm::Attribute<0x0018, 0x9049> MRTransmitCoilSequence;
    typedef gdcm::Attribute<0x0018, 0x9050> TransmitCoilManufacturerName;
    typedef gdcm::Attribute<0x0018, 0x9051> TransmitCoilType;
    typedef gdcm::Attribute<0x0018, 0x9052> SpectralWidth;
    typedef gdcm::Attribute<0x0018, 0x9053> ChemicalShiftReference;
    typedef gdcm::Attribute<0x0018, 0x9054> VolumeLocalizationTechnique;
    typedef gdcm::Attribute<0x0018, 0x9058> MRAcquisitionFrequencyEncodingSteps;
    typedef gdcm::Attribute<0x0018, 0x9059> Decoupling;
    typedef gdcm::Attribute<0x0018, 0x9060> DecoupledNucleus;
    typedef gdcm::Attribute<0x0018, 0x9061> DecouplingFrequency;
    typedef gdcm::Attribute<0x0018, 0x9062> DecouplingMethod;
    typedef gdcm::Attribute<0x0018, 0x9063> DecouplingChemicalShiftReference;
    typedef gdcm::Attribute<0x0018, 0x9064> KSpaceFiltering;
    typedef gdcm::Attribute<0x0018, 0x9065> TimeDomainFiltering;
    typedef gdcm::Attribute<0x0018, 0x9066> NumberOfZeroFills;
    typedef gdcm::Attribute<0x0018, 0x9067> BaselineCorrection;
    typedef gdcm::Attribute<0x0018, 0x9069> ParallelReductionFactorInPlane;
    typedef gdcm::Attribute<0x0018, 0x9070> CardiacRRIntervalSpecified;
    typedef gdcm::Attribute<0x0018, 0x9073> AcquisitionDuration;
    typedef gdcm::Attribute<0x0018, 0x9074> FrameAcquisitionDateTime;
    typedef gdcm::Attribute<0x0018, 0x9075> DiffusionDirectionality;
    typedef gdcm::Attribute<0x0018, 0x9076> DiffusionGradientDirectionSequence;
    typedef gdcm::Attribute<0x0018, 0x9077> ParallelAcquisition;
    typedef gdcm::Attribute<0x0018, 0x9078> ParallelAcquisitionTechnique;
    typedef gdcm::Attribute<0x0018, 0x9079> InversionTimes;
    typedef gdcm::Attribute<0x0018, 0x9080> MetaboliteMapDescription;
    typedef gdcm::Attribute<0x0018, 0x9081> PartialFourier;
    typedef gdcm::Attribute<0x0018, 0x9082> EffectiveEchoTime;
    typedef gdcm::Attribute<0x0018, 0x9083> MetaboliteMapCodeSequence;
    typedef gdcm::Attribute<0x0018, 0x9084> ChemicalShiftSequence;
    typedef gdcm::Attribute<0x0018, 0x9085> CardiacSignalSource;
    typedef gdcm::Attribute<0x0018, 0x9087> DiffusionBValue;
    typedef gdcm::Attribute<0x0018, 0x9089> DiffusionGradientOrientation;
    typedef gdcm::Attribute<0x0018, 0x9090> VelocityEncodingDirection;
    typedef gdcm::Attribute<0x0018, 0x9091> VelocityEncodingMinimumValue;
    typedef gdcm::Attribute<0x0018, 0x9092> VelocityEncodingAcquisitionSequence;
    typedef gdcm::Attribute<0x0018, 0x9093> NumberOfKSpaceTrajectories;
    typedef gdcm::Attribute<0x0018, 0x9094> CoverageOfKSpace;
    typedef gdcm::Attribute<0x0018, 0x9095> SpectroscopyAcquisitionPhaseRows;
    typedef gdcm::Attribute<0x0018, 0x9096> ParallelReductionFactorInPlaneRetired;
    typedef gdcm::Attribute<0x0018, 0x9098> TransmitterFrequency;
    typedef gdcm::Attribute<0x0018, 0x9100> ResonantNucleus;
    typedef gdcm::Attribute<0x0018, 0x9101> FrequencyCorrection;
    typedef gdcm::Attribute<0x0018, 0x9103> MRSpectroscopyFOVGeometrySequence;
    typedef gdcm::Attribute<0x0018, 0x9104> SlabThickness;
    typedef gdcm::Attribute<0x0018, 0x9105> SlabOrientation;
    typedef gdcm::Attribute<0x0018, 0x9106> MidSlabPosition;
    typedef gdcm::Attribute<0x0018, 0x9107> MRSpatialSaturationSequence;
    typedef gdcm::Attribute<0x0018, 0x9112> MRTimingAndRelatedParametersSequence;
    typedef gdcm::Attribute<0x0018, 0x9114> MREchoSequence;
    typedef gdcm::Attribute<0x0018, 0x9115> MRModifierSequence;
    typedef gdcm::Attribute<0x0018, 0x9117> MRDiffusionSequence;
    typedef gdcm::Attribute<0x0018, 0x9118> CardiacSynchronizationSequence;
    typedef gdcm::Attribute<0x0018, 0x9119> MRAveragesSequence;
    typedef gdcm::Attribute<0x0018, 0x9125> MRFOVGeometrySequence;
    typedef gdcm::Attribute<0x0018, 0x9126> VolumeLocalizationSequence;
    typedef gdcm::Attribute<0x0018, 0x9127> SpectroscopyAcquisitionDataColumns;
    typedef gdcm::Attribute<0x0018, 0x9147> DiffusionAnisotropyType;
    typedef gdcm::Attribute<0x0018, 0x9151> FrameReferenceDateTime;
    typedef gdcm::Attribute<0x0018, 0x9152> MRMetaboliteMapSequence;
    typedef gdcm::Attribute<0x0018, 0x9155> ParallelReductionFactorOutOfPlane;
    typedef gdcm::Attribute<0x0018, 0x9159> SpectroscopyAcquisitionOutOfPlanePhaseSteps;
    typedef gdcm::Attribute<0x0018, 0x9166> BulkMotionStatus;
    typedef gdcm::Attribute<0x0018, 0x9168> ParallelReductionFactorSecondInPlane;
    typedef gdcm::Attribute<0x0018, 0x9169> CardiacBeatRejectionTechnique;
    typedef gdcm::Attribute<0x0018, 0x9170> RespiratoryMotionCompensationTechnique;
    typedef gdcm::Attribute<0x0018, 0x9171> RespiratorySignalSource;
    typedef gdcm::Attribute<0x0018, 0x9172> BulkMotionCompensationTechnique;
    typedef gdcm::Attribute<0x0018, 0x9173> BulkMotionSignalSource;
    typedef gdcm::Attribute<0x0018, 0x9174> ApplicableSafetyStandardAgency;
    typedef gdcm::Attribute<0x0018, 0x9175> ApplicableSafetyStandardDescription;
    typedef gdcm::Attribute<0x0018, 0x9176> OperatingModeSequence;
    typedef gdcm::Attribute<0x0018, 0x9177> OperatingModeType;
    typedef gdcm::Attribute<0x0018, 0x9178> OperatingMode;
    typedef gdcm::Attribute<0x0018, 0x9179> SpecificAbsorptionRateDefinition;
    typedef gdcm::Attribute<0x0018, 0x9180> GradientOutputType;
    typedef gdcm::Attribute<0x0018, 0x9181> SpecificAbsorptionRateValue;
    typedef gdcm::Attribute<0x0018, 0x9182> GradientOutput;
    typedef gdcm::Attribute<0x0018, 0x9183> FlowCompensationDirection;
    typedef gdcm::Attribute<0x0018, 0x9184> TaggingDelay;
    typedef gdcm::Attribute<0x0018, 0x9185> RespiratoryMotionCompensationTechniqueDescription;
    typedef gdcm::Attribute<0x0018, 0x9186> RespiratorySignalSourceID;
    typedef gdcm::Attribute<0x0018, 0x9195> ChemicalShiftMinimumIntegrationLimitInHz;
    typedef gdcm::Attribute<0x0018, 0x9196> ChemicalShiftMaximumIntegrationLimitInHz;
    typedef gdcm::Attribute<0x0018, 0x9197> MRVelocityEncodingSequence;
    typedef gdcm::Attribute<0x0018, 0x9198> FirstOrderPhaseCorrection;
    typedef gdcm::Attribute<0x0018, 0x9199> WaterReferencedPhaseCorrection;
    typedef gdcm::Attribute<0x0018, 0x9200> MRSpectroscopyAcquisitionType;
    typedef gdcm::Attribute<0x0018, 0x9214> RespiratoryCyclePosition;
    typedef gdcm::Attribute<0x0018, 0x9217> VelocityEncodingMaximumValue;
    typedef gdcm::Attribute<0x0018, 0x9218> TagSpacingSecondDimension;
    typedef gdcm::Attribute<0x0018, 0x9219> TagAngleSecondAxis;
    typedef gdcm::Attribute<0x0018, 0x9220> FrameAcquisitionDuration;
    typedef gdcm::Attribute<0x0018, 0x9226> MRImageFrameTypeSequence;
    typedef gdcm::Attribute<0x0018, 0x9227> MRSpectroscopyFrameTypeSequence;
    typedef gdcm::Attribute<0x0018, 0x9231> MRAcquisitionPhaseEncodingStepsInPlane;
    typedef gdcm::Attribute<0x0018, 0x9232> MRAcquisitionPhaseEncodingStepsOutOfPlane;
    typedef gdcm::Attribute<0x0018, 0x9234> SpectroscopyAcquisitionPhaseColumns;
    typedef gdcm::Attribute<0x0018, 0x9236> CardiacCyclePosition;
    typedef gdcm::Attribute<0x0018, 0x9239> SpecificAbsorptionRateSequence;
    typedef gdcm::Attribute<0x0018, 0x9240> RFEchoTrainLength;
    typedef gdcm::Attribute<0x0018, 0x9241> GradientEchoTrainLength;
    typedef gdcm::Attribute<0x0018, 0x9250> ArterialSpinLabelingContrast;
    typedef gdcm::Attribute<0x0018, 0x9251> MRArterialSpinLabelingSequence;
    typedef gdcm::Attribute<0x0018, 0x9252> ASLTechniqueDescription;
    typedef gdcm::Attribute<0x0018, 0x9253> ASLSlabNumber;
    typedef gdcm::Attribute<0x0018, 0x9254> ASLSlabThickness;
    typedef gdcm::Attribute<0x0018, 0x9255> ASLSlabOrientation;
    typedef gdcm::Attribute<0x0018, 0x9256> ASLMidSlabPosition;
    typedef gdcm::Attribute<0x0018, 0x9257> ASLContext;
    typedef gdcm::Attribute<0x0018, 0x9258> ASLPulseTrainDuration;
    typedef gdcm::Attribute<0x0018, 0x9259> ASLCrusherFlag;
    typedef gdcm::Attribute<0x0018, 0x925a> ASLCrusherFlow;
    typedef gdcm::Attribute<0x0018, 0x925b> ASLCrusherDescription;
    typedef gdcm::Attribute<0x0018, 0x925c> ASLBolusCutoffFlag;
    typedef gdcm::Attribute<0x0018, 0x925d> ASLBolusCutoffTimingSequence;
    typedef gdcm::Attribute<0x0018, 0x925e> ASLBolusCutoffTechnique;
    typedef gdcm::Attribute<0x0018, 0x925f> ASLBolusCutoffDelayTime;
    typedef gdcm::Attribute<0x0018, 0x9260> ASLSlabSequence;
    typedef gdcm::Attribute<0x0018, 0x9295> ChemicalShiftMinimumIntegrationLimitInppm;
    typedef gdcm::Attribute<0x0018, 0x9296> ChemicalShiftMaximumIntegrationLimitInppm;
    typedef gdcm::Attribute<0x0018, 0x9301> CTAcquisitionTypeSequence;
    typedef gdcm::Attribute<0x0018, 0x9302> AcquisitionType;
    typedef gdcm::Attribute<0x0018, 0x9303> TubeAngle;
    typedef gdcm::Attribute<0x0018, 0x9304> CTAcquisitionDetailsSequence;
    typedef gdcm::Attribute<0x0018, 0x9305> RevolutionTime;
    typedef gdcm::Attribute<0x0018, 0x9306> SingleCollimationWidth;
    typedef gdcm::Attribute<0x0018, 0x9307> TotalCollimationWidth;
    typedef gdcm::Attribute<0x0018, 0x9308> CTTableDynamicsSequence;
    typedef gdcm::Attribute<0x0018, 0x9309> TableSpeed;
    typedef gdcm::Attribute<0x0018, 0x9310> TableFeedPerRotation;
    typedef gdcm::Attribute<0x0018, 0x9311> SpiralPitchFactor;
    typedef gdcm::Attribute<0x0018, 0x9312> CTGeometrySequence;
    typedef gdcm::Attribute<0x0018, 0x9313> DataCollectionCenterPatient;
    typedef gdcm::Attribute<0x0018, 0x9314> CTReconstructionSequence;
    typedef gdcm::Attribute<0x0018, 0x9315> ReconstructionAlgorithm;
    typedef gdcm::Attribute<0x0018, 0x9316> ConvolutionKernelGroup;
    typedef gdcm::Attribute<0x0018, 0x9317> ReconstructionFieldOfView;
    typedef gdcm::Attribute<0x0018, 0x9318> ReconstructionTargetCenterPatient;
    typedef gdcm::Attribute<0x0018, 0x9319> ReconstructionAngle;
    typedef gdcm::Attribute<0x0018, 0x9320> ImageFilter;
    typedef gdcm::Attribute<0x0018, 0x9321> CTExposureSequence;
    typedef gdcm::Attribute<0x0018, 0x9322> ReconstructionPixelSpacing;
    typedef gdcm::Attribute<0x0018, 0x9323> ExposureModulationType;
    typedef gdcm::Attribute<0x0018, 0x9324> EstimatedDoseSaving;
    typedef gdcm::Attribute<0x0018, 0x9325> CTXRayDetailsSequence;
    typedef gdcm::Attribute<0x0018, 0x9326> CTPositionSequence;
    typedef gdcm::Attribute<0x0018, 0x9327> TablePosition;
    typedef gdcm::Attribute<0x0018, 0x9328> ExposureTimeInms;
    typedef gdcm::Attribute<0x0018, 0x9329> CTImageFrameTypeSequence;
    typedef gdcm::Attribute<0x0018, 0x9330> XRayTubeCurrentInmA;
    typedef gdcm::Attribute<0x0018, 0x9332> ExposureInmAs;
    typedef gdcm::Attribute<0x0018, 0x9333> ConstantVolumeFlag;
    typedef gdcm::Attribute<0x0018, 0x9334> FluoroscopyFlag;
    typedef gdcm::Attribute<0x0018, 0x9335> DistanceSourceToDataCollectionCenter;
    typedef gdcm::Attribute<0x0018, 0x9337> ContrastBolusAgentNumber;
    typedef gdcm::Attribute<0x0018, 0x9338> ContrastBolusIngredientCodeSequence;
    typedef gdcm::Attribute<0x0018, 0x9340> ContrastAdministrationProfileSequence;
    typedef gdcm::Attribute<0x0018, 0x9341> ContrastBolusUsageSequence;
    typedef gdcm::Attribute<0x0018, 0x9342> ContrastBolusAgentAdministered;
    typedef gdcm::Attribute<0x0018, 0x9343> ContrastBolusAgentDetected;
    typedef gdcm::Attribute<0x0018, 0x9344> ContrastBolusAgentPhase;
    typedef gdcm::Attribute<0x0018, 0x9345> CTDIvol;
    typedef gdcm::Attribute<0x0018, 0x9346> CTDIPhantomTypeCodeSequence;
    typedef gdcm::Attribute<0x0018, 0x9351> CalciumScoringMassFactorPatient;
    typedef gdcm::Attribute<0x0018, 0x9352> CalciumScoringMassFactorDevice;
    typedef gdcm::Attribute<0x0018, 0x9353> EnergyWeightingFactor;
    typedef gdcm::Attribute<0x0018, 0x9360> CTAdditionalXRaySourceSequence;
    typedef gdcm::Attribute<0x0018, 0x9401> ProjectionPixelCalibrationSequence;
    typedef gdcm::Attribute<0x0018, 0x9402> DistanceSourceToIsocenter;
    typedef gdcm::Attribute<0x0018, 0x9403> DistanceObjectToTableTop;
    typedef gdcm::Attribute<0x0018, 0x9404> ObjectPixelSpacingInCenterOfBeam;
    typedef gdcm::Attribute<0x0018, 0x9405> PositionerPositionSequence;
    typedef gdcm::Attribute<0x0018, 0x9406> TablePositionSequence;
    typedef gdcm::Attribute<0x0018, 0x9407> CollimatorShapeSequence;
    typedef gdcm::Attribute<0x0018, 0x9410> PlanesInAcquisition;
    typedef gdcm::Attribute<0x0018, 0x9412> XAXRFFrameCharacteristicsSequence;
    typedef gdcm::Attribute<0x0018, 0x9417> FrameAcquisitionSequence;
    typedef gdcm::Attribute<0x0018, 0x9420> XRayReceptorType;
    typedef gdcm::Attribute<0x0018, 0x9423> AcquisitionProtocolName;
    typedef gdcm::Attribute<0x0018, 0x9424> AcquisitionProtocolDescription;
    typedef gdcm::Attribute<0x0018, 0x9425> ContrastBolusIngredientOpaque;
    typedef gdcm::Attribute<0x0018, 0x9426> DistanceReceptorPlaneToDetectorHousing;
    typedef gdcm::Attribute<0x0018, 0x9427> IntensifierActiveShape;
    typedef gdcm::Attribute<0x0018, 0x9428> IntensifierActiveDimensions;
    typedef gdcm::Attribute<0x0018, 0x9429> PhysicalDetectorSize;
    typedef gdcm::Attribute<0x0018, 0x9430> PositionOfIsocenterProjection;
    typedef gdcm::Attribute<0x0018, 0x9432> FieldOfViewSequence;
    typedef gdcm::Attribute<0x0018, 0x9433> FieldOfViewDescription;
    typedef gdcm::Attribute<0x0018, 0x9434> ExposureControlSensingRegionsSequence;
    typedef gdcm::Attribute<0x0018, 0x9435> ExposureControlSensingRegionShape;
    typedef gdcm::Attribute<0x0018, 0x9436> ExposureControlSensingRegionLeftVerticalEdge;
    typedef gdcm::Attribute<0x0018, 0x9437> ExposureControlSensingRegionRightVerticalEdge;
    typedef gdcm::Attribute<0x0018, 0x9438> ExposureControlSensingRegionUpperHorizontalEdge;
    typedef gdcm::Attribute<0x0018, 0x9439> ExposureControlSensingRegionLowerHorizontalEdge;
    typedef gdcm::Attribute<0x0018, 0x9440> CenterOfCircularExposureControlSensingRegion;
    typedef gdcm::Attribute<0x0018, 0x9441> RadiusOfCircularExposureControlSensingRegion;
    typedef gdcm::Attribute<0x0018, 0x9442> VerticesOfThePolygonalExposureControlSensingRegion;
    // 0x0018, 0x9445 SHALL NOT BE USED
    typedef gdcm::Attribute<0x0018, 0x9447> ColumnAngulationPatient;
    typedef gdcm::Attribute<0x0018, 0x9449> BeamAngle;
    typedef gdcm::Attribute<0x0018, 0x9451> FrameDetectorParametersSequence;
    typedef gdcm::Attribute<0x0018, 0x9452> CalculatedAnatomyThickness;
    typedef gdcm::Attribute<0x0018, 0x9455> CalibrationSequence;
    typedef gdcm::Attribute<0x0018, 0x9456> ObjectThicknessSequence;
    typedef gdcm::Attribute<0x0018, 0x9457> PlaneIdentification;
    typedef gdcm::Attribute<0x0018, 0x9461> FieldOfViewDimensionsInFloat;
    typedef gdcm::Attribute<0x0018, 0x9462> IsocenterReferenceSystemSequence;
    typedef gdcm::Attribute<0x0018, 0x9463> PositionerIsocenterPrimaryAngle;
    typedef gdcm::Attribute<0x0018, 0x9464> PositionerIsocenterSecondaryAngle;
    typedef gdcm::Attribute<0x0018, 0x9465> PositionerIsocenterDetectorRotationAngle;
    typedef gdcm::Attribute<0x0018, 0x9466> TableXPositionToIsocenter;
    typedef gdcm::Attribute<0x0018, 0x9467> TableYPositionToIsocenter;
    typedef gdcm::Attribute<0x0018, 0x9468> TableZPositionToIsocenter;
    typedef gdcm::Attribute<0x0018, 0x9469> TableHorizontalRotationAngle;
    typedef gdcm::Attribute<0x0018, 0x9470> TableHeadTiltAngle;
    typedef gdcm::Attribute<0x0018, 0x9471> TableCradleTiltAngle;
    typedef gdcm::Attribute<0x0018, 0x9472> FrameDisplayShutterSequence;
    typedef gdcm::Attribute<0x0018, 0x9473> AcquiredImageAreaDoseProduct;
    typedef gdcm::Attribute<0x0018, 0x9474> CArmPositionerTabletopRelationship;
    typedef gdcm::Attribute<0x0018, 0x9476> XRayGeometrySequence;
    typedef gdcm::Attribute<0x0018, 0x9477> IrradiationEventIdentificationSequence;
    typedef gdcm::Attribute<0x0018, 0x9504> XRay3DFrameTypeSequence;
    typedef gdcm::Attribute<0x0018, 0x9506> ContributingSourcesSequence;
    typedef gdcm::Attribute<0x0018, 0x9507> XRay3DAcquisitionSequence;
    typedef gdcm::Attribute<0x0018, 0x9508> PrimaryPositionerScanArc;
    typedef gdcm::Attribute<0x0018, 0x9509> SecondaryPositionerScanArc;
    typedef gdcm::Attribute<0x0018, 0x9510> PrimaryPositionerScanStartAngle;
    typedef gdcm::Attribute<0x0018, 0x9511> SecondaryPositionerScanStartAngle;
    typedef gdcm::Attribute<0x0018, 0x9514> PrimaryPositionerIncrement;
    typedef gdcm::Attribute<0x0018, 0x9515> SecondaryPositionerIncrement;
    typedef gdcm::Attribute<0x0018, 0x9516> StartAcquisitionDateTime;
    typedef gdcm::Attribute<0x0018, 0x9517> EndAcquisitionDateTime;
    typedef gdcm::Attribute<0x0018, 0x9524> ApplicationName;
    typedef gdcm::Attribute<0x0018, 0x9525> ApplicationVersion;
    typedef gdcm::Attribute<0x0018, 0x9526> ApplicationManufacturer;
    typedef gdcm::Attribute<0x0018, 0x9527> AlgorithmType;
    typedef gdcm::Attribute<0x0018, 0x9528> AlgorithmDescription;
    typedef gdcm::Attribute<0x0018, 0x9530> XRay3DReconstructionSequence;
    typedef gdcm::Attribute<0x0018, 0x9531> ReconstructionDescription;
    typedef gdcm::Attribute<0x0018, 0x9538> PerProjectionAcquisitionSequence;
    typedef gdcm::Attribute<0x0018, 0x9601> DiffusionBMatrixSequence;
    typedef gdcm::Attribute<0x0018, 0x9602> DiffusionBValueXX;
    typedef gdcm::Attribute<0x0018, 0x9603> DiffusionBValueXY;
    typedef gdcm::Attribute<0x0018, 0x9604> DiffusionBValueXZ;
    typedef gdcm::Attribute<0x0018, 0x9605> DiffusionBValueYY;
    typedef gdcm::Attribute<0x0018, 0x9606> DiffusionBValueYZ;
    typedef gdcm::Attribute<0x0018, 0x9607> DiffusionBValueZZ;
    typedef gdcm::Attribute<0x0018, 0x9701> DecayCorrectionDateTime;
    typedef gdcm::Attribute<0x0018, 0x9715> StartDensityThreshold;
    typedef gdcm::Attribute<0x0018, 0x9716> StartRelativeDensityDifferenceThreshold;
    typedef gdcm::Attribute<0x0018, 0x9717> StartCardiacTriggerCountThreshold;
    typedef gdcm::Attribute<0x0018, 0x9718> StartRespiratoryTriggerCountThreshold;
    typedef gdcm::Attribute<0x0018, 0x9719> TerminationCountsThreshold;
    typedef gdcm::Attribute<0x0018, 0x9720> TerminationDensityThreshold;
    typedef gdcm::Attribute<0x0018, 0x9721> TerminationRelativeDensityThreshold;
    typedef gdcm::Attribute<0x0018, 0x9722> TerminationTimeThreshold;
    typedef gdcm::Attribute<0x0018, 0x9723> TerminationCardiacTriggerCountThreshold;
    typedef gdcm::Attribute<0x0018, 0x9724> TerminationRespiratoryTriggerCountThreshold;
    typedef gdcm::Attribute<0x0018, 0x9725> DetectorGeometry;
    typedef gdcm::Attribute<0x0018, 0x9726> TransverseDetectorSeparation;
    typedef gdcm::Attribute<0x0018, 0x9727> AxialDetectorDimension;
    typedef gdcm::Attribute<0x0018, 0x9729> RadiopharmaceuticalAgentNumber;
    typedef gdcm::Attribute<0x0018, 0x9732> PETFrameAcquisitionSequence;
    typedef gdcm::Attribute<0x0018, 0x9733> PETDetectorMotionDetailsSequence;
    typedef gdcm::Attribute<0x0018, 0x9734> PETTableDynamicsSequence;
    typedef gdcm::Attribute<0x0018, 0x9735> PETPositionSequence;
    typedef gdcm::Attribute<0x0018, 0x9736> PETFrameCorrectionFactorsSequence;
    typedef gdcm::Attribute<0x0018, 0x9737> RadiopharmaceuticalUsageSequence;
    typedef gdcm::Attribute<0x0018, 0x9738> AttenuationCorrectionSource;
    typedef gdcm::Attribute<0x0018, 0x9739> NumberOfIterations;
    typedef gdcm::Attribute<0x0018, 0x9740> NumberOfSubsets;
    typedef gdcm::Attribute<0x0018, 0x9749> PETReconstructionSequence;
    typedef gdcm::Attribute<0x0018, 0x9751> PETFrameTypeSequence;
    typedef gdcm::Attribute<0x0018, 0x9755> TimeOfFlightInformationUsed;
    typedef gdcm::Attribute<0x0018, 0x9756> ReconstructionType;
    typedef gdcm::Attribute<0x0018, 0x9758> DecayCorrected;
    typedef gdcm::Attribute<0x0018, 0x9759> AttenuationCorrected;
    typedef gdcm::Attribute<0x0018, 0x9760> ScatterCorrected;
    typedef gdcm::Attribute<0x0018, 0x9761> DeadTimeCorrected;
    typedef gdcm::Attribute<0x0018, 0x9762> GantryMotionCorrected;
    typedef gdcm::Attribute<0x0018, 0x9763> PatientMotionCorrected;
    typedef gdcm::Attribute<0x0018, 0x9764> CountLossNormalizationCorrected;
    typedef gdcm::Attribute<0x0018, 0x9765> RandomsCorrected;
    typedef gdcm::Attribute<0x0018, 0x9766> NonUniformRadialSamplingCorrected;
    typedef gdcm::Attribute<0x0018, 0x9767> SensitivityCalibrated;
    typedef gdcm::Attribute<0x0018, 0x9768> DetectorNormalizationCorrection;
    typedef gdcm::Attribute<0x0018, 0x9769> IterativeReconstructionMethod;
    typedef gdcm::Attribute<0x0018, 0x9770> AttenuationCorrectionTemporalRelationship;
    typedef gdcm::Attribute<0x0018, 0x9771> PatientPhysiologicalStateSequence;
    typedef gdcm::Attribute<0x0018, 0x9772> PatientPhysiologicalStateCodeSequence;
    typedef gdcm::Attribute<0x0018, 0x9801> DepthsOfFocus;
    typedef gdcm::Attribute<0x0018, 0x9803> ExcludedIntervalsSequence;
    typedef gdcm::Attribute<0x0018, 0x9804> ExclusionStartDatetime;
    typedef gdcm::Attribute<0x0018, 0x9805> ExclusionDuration;
    typedef gdcm::Attribute<0x0018, 0x9806> USImageDescriptionSequence;
    typedef gdcm::Attribute<0x0018, 0x9807> ImageDataTypeSequence;
    typedef gdcm::Attribute<0x0018, 0x9808> DataType;
    typedef gdcm::Attribute<0x0018, 0x9809> TransducerScanPatternCodeSequence;
    typedef gdcm::Attribute<0x0018, 0x980b> AliasedDataType;
    typedef gdcm::Attribute<0x0018, 0x980c> PositionMeasuringDeviceUsed;
    typedef gdcm::Attribute<0x0018, 0x980d> TransducerGeometryCodeSequence;
    typedef gdcm::Attribute<0x0018, 0x980e> TransducerBeamSteeringCodeSequence;
    typedef gdcm::Attribute<0x0018, 0x980f> TransducerApplicationCodeSequence;
    typedef gdcm::Attribute<0x0018, 0xa001> ContributingEquipmentSequence;
    typedef gdcm::Attribute<0x0018, 0xa002> ContributionDateTime;
    typedef gdcm::Attribute<0x0018, 0xa003> ContributionDescription;
    typedef gdcm::Attribute<0x0020, 0x000d> StudyInstanceUID;
    typedef gdcm::Attribute<0x0020, 0x000e> SeriesInstanceUID;
    typedef gdcm::Attribute<0x0020, 0x0010> StudyID;
    typedef gdcm::Attribute<0x0020, 0x0011> SeriesNumber;
    typedef gdcm::Attribute<0x0020, 0x0012> AcquisitionNumber;
    typedef gdcm::Attribute<0x0020, 0x0013> InstanceNumber;
    typedef gdcm::Attribute<0x0020, 0x0014> IsotopeNumber;
    typedef gdcm::Attribute<0x0020, 0x0015> PhaseNumber;
    typedef gdcm::Attribute<0x0020, 0x0016> IntervalNumber;
    typedef gdcm::Attribute<0x0020, 0x0017> TimeSlotNumber;
    typedef gdcm::Attribute<0x0020, 0x0018> AngleNumber;
    typedef gdcm::Attribute<0x0020, 0x0019> ItemNumber;
    typedef gdcm::Attribute<0x0020, 0x0020> PatientOrientation;
    typedef gdcm::Attribute<0x0020, 0x0022> OverlayNumber;
    typedef gdcm::Attribute<0x0020, 0x0024> CurveNumber;
    typedef gdcm::Attribute<0x0020, 0x0026> LUTNumber;
    typedef gdcm::Attribute<0x0020, 0x0030> ImagePosition;
    typedef gdcm::Attribute<0x0020, 0x0032> ImagePositionPatient;
    typedef gdcm::Attribute<0x0020, 0x0035> ImageOrientation;
    typedef gdcm::Attribute<0x0020, 0x0037> ImageOrientationPatient;
    typedef gdcm::Attribute<0x0020, 0x0050> Location;
    typedef gdcm::Attribute<0x0020, 0x0052> FrameOfReferenceUID;
    typedef gdcm::Attribute<0x0020, 0x0060> Laterality;
    typedef gdcm::Attribute<0x0020, 0x0062> ImageLaterality;
    typedef gdcm::Attribute<0x0020, 0x0070> ImageGeometryType;
    typedef gdcm::Attribute<0x0020, 0x0080> MaskingImage;
    typedef gdcm::Attribute<0x0020, 0x00aa> ReportNumber;
    typedef gdcm::Attribute<0x0020, 0x0100> TemporalPositionIdentifier;
    typedef gdcm::Attribute<0x0020, 0x0105> NumberOfTemporalPositions;
    typedef gdcm::Attribute<0x0020, 0x0110> TemporalResolution;
    typedef gdcm::Attribute<0x0020, 0x0200> SynchronizationFrameOfReferenceUID;
    typedef gdcm::Attribute<0x0020, 0x0242> SOPInstanceUIDOfConcatenationSource;
    typedef gdcm::Attribute<0x0020, 0x1000> SeriesInStudy;
    typedef gdcm::Attribute<0x0020, 0x1001> AcquisitionsInSeries;
    typedef gdcm::Attribute<0x0020, 0x1002> ImagesInAcquisition;
    typedef gdcm::Attribute<0x0020, 0x1003> ImagesInSeries;
    typedef gdcm::Attribute<0x0020, 0x1004> AcquisitionsInStudy;
    typedef gdcm::Attribute<0x0020, 0x1005> ImagesInStudy;
    typedef gdcm::Attribute<0x0020, 0x1020> Reference;
    typedef gdcm::Attribute<0x0020, 0x1040> PositionReferenceIndicator;
    typedef gdcm::Attribute<0x0020, 0x1041> SliceLocation;
    typedef gdcm::Attribute<0x0020, 0x1070> OtherStudyNumbers;
    typedef gdcm::Attribute<0x0020, 0x1200> NumberOfPatientRelatedStudies;
    typedef gdcm::Attribute<0x0020, 0x1202> NumberOfPatientRelatedSeries;
    typedef gdcm::Attribute<0x0020, 0x1204> NumberOfPatientRelatedInstances;
    typedef gdcm::Attribute<0x0020, 0x1206> NumberOfStudyRelatedSeries;
    typedef gdcm::Attribute<0x0020, 0x1208> NumberOfStudyRelatedInstances;
    typedef gdcm::Attribute<0x0020, 0x1209> NumberOfSeriesRelatedInstances;
    typedef gdcm::Attribute<0x0020, 0x3100> SourceImageIDs;
    typedef gdcm::Attribute<0x0020, 0x3401> ModifyingDeviceID;
    typedef gdcm::Attribute<0x0020, 0x3402> ModifiedImageID;
    typedef gdcm::Attribute<0x0020, 0x3403> ModifiedImageDate;
    typedef gdcm::Attribute<0x0020, 0x3404> ModifyingDeviceManufacturer;
    typedef gdcm::Attribute<0x0020, 0x3405> ModifiedImageTime;
    typedef gdcm::Attribute<0x0020, 0x3406> ModifiedImageDescription;
    typedef gdcm::Attribute<0x0020, 0x4000> ImageComments;
    typedef gdcm::Attribute<0x0020, 0x5000> OriginalImageIdentification;
    typedef gdcm::Attribute<0x0020, 0x5002> OriginalImageIdentificationNomenclature;
    typedef gdcm::Attribute<0x0020, 0x9056> StackID;
    typedef gdcm::Attribute<0x0020, 0x9057> InStackPositionNumber;
    typedef gdcm::Attribute<0x0020, 0x9071> FrameAnatomySequence;
    typedef gdcm::Attribute<0x0020, 0x9072> FrameLaterality;
    typedef gdcm::Attribute<0x0020, 0x9111> FrameContentSequence;
    typedef gdcm::Attribute<0x0020, 0x9113> PlanePositionSequence;
    typedef gdcm::Attribute<0x0020, 0x9116> PlaneOrientationSequence;
    typedef gdcm::Attribute<0x0020, 0x9128> TemporalPositionIndex;
    typedef gdcm::Attribute<0x0020, 0x9153> NominalCardiacTriggerDelayTime;
    typedef gdcm::Attribute<0x0020, 0x9154> NominalCardiacTriggerTimePriorToRPeak;
    typedef gdcm::Attribute<0x0020, 0x9155> ActualCardiacTriggerTimePriorToRPeak;
    typedef gdcm::Attribute<0x0020, 0x9156> FrameAcquisitionNumber;
    typedef gdcm::Attribute<0x0020, 0x9157> DimensionIndexValues;
    typedef gdcm::Attribute<0x0020, 0x9158> FrameComments;
    typedef gdcm::Attribute<0x0020, 0x9161> ConcatenationUID;
    typedef gdcm::Attribute<0x0020, 0x9162> InConcatenationNumber;
    typedef gdcm::Attribute<0x0020, 0x9163> InConcatenationTotalNumber;
    typedef gdcm::Attribute<0x0020, 0x9164> DimensionOrganizationUID;
    typedef gdcm::Attribute<0x0020, 0x9165> DimensionIndexPointer;
    typedef gdcm::Attribute<0x0020, 0x9167> FunctionalGroupPointer;
    typedef gdcm::Attribute<0x0020, 0x9213> DimensionIndexPrivateCreator;
    typedef gdcm::Attribute<0x0020, 0x9221> DimensionOrganizationSequence;
    typedef gdcm::Attribute<0x0020, 0x9222> DimensionIndexSequence;
    typedef gdcm::Attribute<0x0020, 0x9228> ConcatenationFrameOffsetNumber;
    typedef gdcm::Attribute<0x0020, 0x9238> FunctionalGroupPrivateCreator;
    typedef gdcm::Attribute<0x0020, 0x9241> NominalPercentageOfCardiacPhase;
    typedef gdcm::Attribute<0x0020, 0x9245> NominalPercentageOfRespiratoryPhase;
    typedef gdcm::Attribute<0x0020, 0x9246> StartingRespiratoryAmplitude;
    typedef gdcm::Attribute<0x0020, 0x9247> StartingRespiratoryPhase;
    typedef gdcm::Attribute<0x0020, 0x9248> EndingRespiratoryAmplitude;
    typedef gdcm::Attribute<0x0020, 0x9249> EndingRespiratoryPhase;
    typedef gdcm::Attribute<0x0020, 0x9250> RespiratoryTriggerType;
    typedef gdcm::Attribute<0x0020, 0x9251> RRIntervalTimeNominal;
    typedef gdcm::Attribute<0x0020, 0x9252> ActualCardiacTriggerDelayTime;
    typedef gdcm::Attribute<0x0020, 0x9253> RespiratorySynchronizationSequence;
    typedef gdcm::Attribute<0x0020, 0x9254> RespiratoryIntervalTime;
    typedef gdcm::Attribute<0x0020, 0x9255> NominalRespiratoryTriggerDelayTime;
    typedef gdcm::Attribute<0x0020, 0x9256> RespiratoryTriggerDelayThreshold;
    typedef gdcm::Attribute<0x0020, 0x9257> ActualRespiratoryTriggerDelayTime;
    typedef gdcm::Attribute<0x0020, 0x9301> ImagePositionVolume;
    typedef gdcm::Attribute<0x0020, 0x9302> ImageOrientationVolume;
    typedef gdcm::Attribute<0x0020, 0x9307> UltrasoundAcquisitionGeometry;
    typedef gdcm::Attribute<0x0020, 0x9308> ApexPosition;
    typedef gdcm::Attribute<0x0020, 0x9309> VolumeToTransducerMappingMatrix;
    typedef gdcm::Attribute<0x0020, 0x930a> VolumeToTableMappingMatrix;
    typedef gdcm::Attribute<0x0020, 0x930c> PatientFrameOfReferenceSource;
    typedef gdcm::Attribute<0x0020, 0x930d> TemporalPositionTimeOffset;
    typedef gdcm::Attribute<0x0020, 0x930e> PlanePositionVolumeSequence;
    typedef gdcm::Attribute<0x0020, 0x930f> PlaneOrientationVolumeSequence;
    typedef gdcm::Attribute<0x0020, 0x9310> TemporalPositionSequence;
    typedef gdcm::Attribute<0x0020, 0x9311> DimensionOrganizationType;
    typedef gdcm::Attribute<0x0020, 0x9312> VolumeFrameOfReferenceUID;
    typedef gdcm::Attribute<0x0020, 0x9313> TableFrameOfReferenceUID;
    typedef gdcm::Attribute<0x0020, 0x9421> DimensionDescriptionLabel;
    typedef gdcm::Attribute<0x0020, 0x9450> PatientOrientationInFrameSequence;
    typedef gdcm::Attribute<0x0020, 0x9453> FrameLabel;
    typedef gdcm::Attribute<0x0020, 0x9518> AcquisitionIndex;
    typedef gdcm::Attribute<0x0020, 0x9529> ContributingSOPInstancesReferenceSequence;
    typedef gdcm::Attribute<0x0020, 0x9536> ReconstructionIndex;
    typedef gdcm::Attribute<0x0022, 0x0001> LightPathFilterPassThroughWavelength;
    typedef gdcm::Attribute<0x0022, 0x0002> LightPathFilterPassBand;
    typedef gdcm::Attribute<0x0022, 0x0003> ImagePathFilterPassThroughWavelength;
    typedef gdcm::Attribute<0x0022, 0x0004> ImagePathFilterPassBand;
    typedef gdcm::Attribute<0x0022, 0x0005> PatientEyeMovementCommanded;
    typedef gdcm::Attribute<0x0022, 0x0006> PatientEyeMovementCommandCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x0007> SphericalLensPower;
    typedef gdcm::Attribute<0x0022, 0x0008> CylinderLensPower;
    typedef gdcm::Attribute<0x0022, 0x0009> CylinderAxis;
    typedef gdcm::Attribute<0x0022, 0x000a> EmmetropicMagnification;
    typedef gdcm::Attribute<0x0022, 0x000b> IntraOcularPressure;
    typedef gdcm::Attribute<0x0022, 0x000c> HorizontalFieldOfView;
    typedef gdcm::Attribute<0x0022, 0x000d> PupilDilated;
    typedef gdcm::Attribute<0x0022, 0x000e> DegreeOfDilation;
    typedef gdcm::Attribute<0x0022, 0x0010> StereoBaselineAngle;
    typedef gdcm::Attribute<0x0022, 0x0011> StereoBaselineDisplacement;
    typedef gdcm::Attribute<0x0022, 0x0012> StereoHorizontalPixelOffset;
    typedef gdcm::Attribute<0x0022, 0x0013> StereoVerticalPixelOffset;
    typedef gdcm::Attribute<0x0022, 0x0014> StereoRotation;
    typedef gdcm::Attribute<0x0022, 0x0015> AcquisitionDeviceTypeCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x0016> IlluminationTypeCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x0017> LightPathFilterTypeStackCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x0018> ImagePathFilterTypeStackCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x0019> LensesCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x001a> ChannelDescriptionCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x001b> RefractiveStateSequence;
    typedef gdcm::Attribute<0x0022, 0x001c> MydriaticAgentCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x001d> RelativeImagePositionCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x001e> CameraAngleOfView;
    typedef gdcm::Attribute<0x0022, 0x0020> StereoPairsSequence;
    typedef gdcm::Attribute<0x0022, 0x0021> LeftImageSequence;
    typedef gdcm::Attribute<0x0022, 0x0022> RightImageSequence;
    typedef gdcm::Attribute<0x0022, 0x0030> AxialLengthOfTheEye;
    typedef gdcm::Attribute<0x0022, 0x0031> OphthalmicFrameLocationSequence;
    typedef gdcm::Attribute<0x0022, 0x0032> ReferenceCoordinates;
    typedef gdcm::Attribute<0x0022, 0x0035> DepthSpatialResolution;
    typedef gdcm::Attribute<0x0022, 0x0036> MaximumDepthDistortion;
    typedef gdcm::Attribute<0x0022, 0x0037> AlongScanSpatialResolution;
    typedef gdcm::Attribute<0x0022, 0x0038> MaximumAlongScanDistortion;
    typedef gdcm::Attribute<0x0022, 0x0039> OphthalmicImageOrientation;
    typedef gdcm::Attribute<0x0022, 0x0041> DepthOfTransverseImage;
    typedef gdcm::Attribute<0x0022, 0x0042> MydriaticAgentConcentrationUnitsSequence;
    typedef gdcm::Attribute<0x0022, 0x0048> AcrossScanSpatialResolution;
    typedef gdcm::Attribute<0x0022, 0x0049> MaximumAcrossScanDistortion;
    typedef gdcm::Attribute<0x0022, 0x004e> MydriaticAgentConcentration;
    typedef gdcm::Attribute<0x0022, 0x0055> IlluminationWaveLength;
    typedef gdcm::Attribute<0x0022, 0x0056> IlluminationPower;
    typedef gdcm::Attribute<0x0022, 0x0057> IlluminationBandwidth;
    typedef gdcm::Attribute<0x0022, 0x0058> MydriaticAgentSequence;
    typedef gdcm::Attribute<0x0022, 0x1007> OphthalmicAxialMeasurementsRightEyeSequence;
    typedef gdcm::Attribute<0x0022, 0x1008> OphthalmicAxialMeasurementsLeftEyeSequence;
    typedef gdcm::Attribute<0x0022, 0x1009> OphthalmicAxialMeasurementsDeviceType;
    typedef gdcm::Attribute<0x0022, 0x1010> OphthalmicAxialLengthMeasurementsType;
    typedef gdcm::Attribute<0x0022, 0x1012> OphthalmicAxialLengthSequence;
    typedef gdcm::Attribute<0x0022, 0x1019> OphthalmicAxialLength;
    typedef gdcm::Attribute<0x0022, 0x1024> LensStatusCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x1025> VitreousStatusCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x1028> IOLFormulaCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x1029> IOLFormulaDetail;
    typedef gdcm::Attribute<0x0022, 0x1033> KeratometerIndex;
    typedef gdcm::Attribute<0x0022, 0x1035> SourceOfOphthalmicAxialLengthCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x1037> TargetRefraction;
    typedef gdcm::Attribute<0x0022, 0x1039> RefractiveProcedureOccurred;
    typedef gdcm::Attribute<0x0022, 0x1040> RefractiveSurgeryTypeCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x1044> OphthalmicUltrasoundMethodCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x1050> OphthalmicAxialLengthMeasurementsSequence;
    typedef gdcm::Attribute<0x0022, 0x1053> IOLPower;
    typedef gdcm::Attribute<0x0022, 0x1054> PredictedRefractiveError;
    typedef gdcm::Attribute<0x0022, 0x1059> OphthalmicAxialLengthVelocity;
    typedef gdcm::Attribute<0x0022, 0x1065> LensStatusDescription;
    typedef gdcm::Attribute<0x0022, 0x1066> VitreousStatusDescription;
    typedef gdcm::Attribute<0x0022, 0x1090> IOLPowerSequence;
    typedef gdcm::Attribute<0x0022, 0x1092> LensConstantSequence;
    typedef gdcm::Attribute<0x0022, 0x1093> IOLManufacturer;
    typedef gdcm::Attribute<0x0022, 0x1094> LensConstantDescription;
    typedef gdcm::Attribute<0x0022, 0x1095> ImplantName;
    typedef gdcm::Attribute<0x0022, 0x1096> KeratometryMeasurementTypeCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x1097> ImplantPartNumber;
    typedef gdcm::Attribute<0x0022, 0x1100> ReferencedOphthalmicAxialMeasurementsSequence;
    typedef gdcm::Attribute<0x0022, 0x1101> OphthalmicAxialLengthMeasurementsSegmentNameCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x1103> RefractiveErrorBeforeRefractiveSurgeryCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x1121> IOLPowerForExactEmmetropia;
    typedef gdcm::Attribute<0x0022, 0x1122> IOLPowerForExactTargetRefraction;
    typedef gdcm::Attribute<0x0022, 0x1125> AnteriorChamberDepthDefinitionCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x1127> LensThicknessSequence;
    typedef gdcm::Attribute<0x0022, 0x1128> AnteriorChamberDepthSequence;
    typedef gdcm::Attribute<0x0022, 0x1130> LensThickness;
    typedef gdcm::Attribute<0x0022, 0x1131> AnteriorChamberDepth;
    typedef gdcm::Attribute<0x0022, 0x1132> SourceOfLensThicknessDataCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x1133> SourceOfAnteriorChamberDepthDataCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x1134> SourceOfRefractiveMeasurementsSequence;
    typedef gdcm::Attribute<0x0022, 0x1135> SourceOfRefractiveMeasurementsCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x1140> OphthalmicAxialLengthMeasurementModified;
    typedef gdcm::Attribute<0x0022, 0x1150> OphthalmicAxialLengthDataSourceCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x1153> OphthalmicAxialLengthAcquisitionMethodCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x1155> SignalToNoiseRatio;
    typedef gdcm::Attribute<0x0022, 0x1159> OphthalmicAxialLengthDataSourceDescription;
    typedef gdcm::Attribute<0x0022, 0x1210> OphthalmicAxialLengthMeasurementsTotalLengthSequence;
    typedef gdcm::Attribute<0x0022, 0x1211> OphthalmicAxialLengthMeasurementsSegmentalLengthSequence;
    typedef gdcm::Attribute<0x0022, 0x1212> OphthalmicAxialLengthMeasurementsLengthSummationSequence;
    typedef gdcm::Attribute<0x0022, 0x1220> UltrasoundOphthalmicAxialLengthMeasurementsSequence;
    typedef gdcm::Attribute<0x0022, 0x1225> OpticalOphthalmicAxialLengthMeasurementsSequence;
    typedef gdcm::Attribute<0x0022, 0x1230> UltrasoundSelectedOphthalmicAxialLengthSequence;
    typedef gdcm::Attribute<0x0022, 0x1250> OphthalmicAxialLengthSelectionMethodCodeSequence;
    typedef gdcm::Attribute<0x0022, 0x1255> OpticalSelectedOphthalmicAxialLengthSequence;
    typedef gdcm::Attribute<0x0022, 0x1257> SelectedSegmentalOphthalmicAxialLengthSequence;
    typedef gdcm::Attribute<0x0022, 0x1260> SelectedTotalOphthalmicAxialLengthSequence;
    typedef gdcm::Attribute<0x0022, 0x1262> OphthalmicAxialLengthQualityMetricSequence;
    typedef gdcm::Attribute<0x0022, 0x1273> OphthalmicAxialLengthQualityMetricTypeDescription;
    typedef gdcm::Attribute<0x0022, 0x1300> IntraocularLensCalculationsRightEyeSequence;
    typedef gdcm::Attribute<0x0022, 0x1310> IntraocularLensCalculationsLeftEyeSequence;
    typedef gdcm::Attribute<0x0022, 0x1330> ReferencedOphthalmicAxialLengthMeasurementQCImageSequence;
    typedef gdcm::Attribute<0x0024, 0x0010> VisualFieldHorizontalExtent;
    typedef gdcm::Attribute<0x0024, 0x0011> VisualFieldVerticalExtent;
    typedef gdcm::Attribute<0x0024, 0x0012> VisualFieldShape;
    typedef gdcm::Attribute<0x0024, 0x0016> ScreeningTestModeCodeSequence;
    typedef gdcm::Attribute<0x0024, 0x0018> MaximumStimulusLuminance;
    typedef gdcm::Attribute<0x0024, 0x0020> BackgroundLuminance;
    typedef gdcm::Attribute<0x0024, 0x0021> StimulusColorCodeSequence;
    typedef gdcm::Attribute<0x0024, 0x0024> BackgroundIlluminationColorCodeSequence;
    typedef gdcm::Attribute<0x0024, 0x0025> StimulusArea;
    typedef gdcm::Attribute<0x0024, 0x0028> StimulusPresentationTime;
    typedef gdcm::Attribute<0x0024, 0x0032> FixationSequence;
    typedef gdcm::Attribute<0x0024, 0x0033> FixationMonitoringCodeSequence;
    typedef gdcm::Attribute<0x0024, 0x0034> VisualFieldCatchTrialSequence;
    typedef gdcm::Attribute<0x0024, 0x0035> FixationCheckedQuantity;
    typedef gdcm::Attribute<0x0024, 0x0036> PatientNotProperlyFixatedQuantity;
    typedef gdcm::Attribute<0x0024, 0x0037> PresentedVisualStimuliDataFlag;
    typedef gdcm::Attribute<0x0024, 0x0038> NumberOfVisualStimuli;
    typedef gdcm::Attribute<0x0024, 0x0039> ExcessiveFixationLossesDataFlag;
    typedef gdcm::Attribute<0x0024, 0x0040> ExcessiveFixationLosses;
    typedef gdcm::Attribute<0x0024, 0x0042> StimuliRetestingQuantity;
    typedef gdcm::Attribute<0x0024, 0x0044> CommentsOnPatientPerformanceOfVisualField;
    typedef gdcm::Attribute<0x0024, 0x0045> FalseNegativesEstimateFlag;
    typedef gdcm::Attribute<0x0024, 0x0046> FalseNegativesEstimate;
    typedef gdcm::Attribute<0x0024, 0x0048> NegativeCatchTrialsQuantity;
    typedef gdcm::Attribute<0x0024, 0x0050> FalseNegativesQuantity;
    typedef gdcm::Attribute<0x0024, 0x0051> ExcessiveFalseNegativesDataFlag;
    typedef gdcm::Attribute<0x0024, 0x0052> ExcessiveFalseNegatives;
    typedef gdcm::Attribute<0x0024, 0x0053> FalsePositivesEstimateFlag;
    typedef gdcm::Attribute<0x0024, 0x0054> FalsePositivesEstimate;
    typedef gdcm::Attribute<0x0024, 0x0055> CatchTrialsDataFlag;
    typedef gdcm::Attribute<0x0024, 0x0056> PositiveCatchTrialsQuantity;
    typedef gdcm::Attribute<0x0024, 0x0057> TestPointNormalsDataFlag;
    typedef gdcm::Attribute<0x0024, 0x0058> TestPointNormalsSequence;
    typedef gdcm::Attribute<0x0024, 0x0059> GlobalDeviationProbabilityNormalsFlag;
    typedef gdcm::Attribute<0x0024, 0x0060> FalsePositivesQuantity;
    typedef gdcm::Attribute<0x0024, 0x0061> ExcessiveFalsePositivesDataFlag;
    typedef gdcm::Attribute<0x0024, 0x0062> ExcessiveFalsePositives;
    typedef gdcm::Attribute<0x0024, 0x0063> VisualFieldTestNormalsFlag;
    typedef gdcm::Attribute<0x0024, 0x0064> ResultsNormalsSequence;
    typedef gdcm::Attribute<0x0024, 0x0065> AgeCorrectedSensitivityDeviationAlgorithmSequence;
    typedef gdcm::Attribute<0x0024, 0x0066> GlobalDeviationFromNormal;
    typedef gdcm::Attribute<0x0024, 0x0067> GeneralizedDefectSensitivityDeviationAlgorithmSequence;
    typedef gdcm::Attribute<0x0024, 0x0068> LocalizedDeviationfromNormal;
    typedef gdcm::Attribute<0x0024, 0x0069> PatientReliabilityIndicator;
    typedef gdcm::Attribute<0x0024, 0x0070> VisualFieldMeanSensitivity;
    typedef gdcm::Attribute<0x0024, 0x0071> GlobalDeviationProbability;
    typedef gdcm::Attribute<0x0024, 0x0072> LocalDeviationProbabilityNormalsFlag;
    typedef gdcm::Attribute<0x0024, 0x0073> LocalizedDeviationProbability;
    typedef gdcm::Attribute<0x0024, 0x0074> ShortTermFluctuationCalculated;
    typedef gdcm::Attribute<0x0024, 0x0075> ShortTermFluctuation;
    typedef gdcm::Attribute<0x0024, 0x0076> ShortTermFluctuationProbabilityCalculated;
    typedef gdcm::Attribute<0x0024, 0x0077> ShortTermFluctuationProbability;
    typedef gdcm::Attribute<0x0024, 0x0078> CorrectedLocalizedDeviationFromNormalCalculated;
    typedef gdcm::Attribute<0x0024, 0x0079> CorrectedLocalizedDeviationFromNormal;
    typedef gdcm::Attribute<0x0024, 0x0080> CorrectedLocalizedDeviationFromNormalProbabilityCalculated;
    typedef gdcm::Attribute<0x0024, 0x0081> CorrectedLocalizedDeviationFromNormalProbability;
    typedef gdcm::Attribute<0x0024, 0x0083> GlobalDeviationProbabilitySequence;
    typedef gdcm::Attribute<0x0024, 0x0085> LocalizedDeviationProbabilitySequence;
    typedef gdcm::Attribute<0x0024, 0x0086> FovealSensitivityMeasured;
    typedef gdcm::Attribute<0x0024, 0x0087> FovealSensitivity;
    typedef gdcm::Attribute<0x0024, 0x0088> VisualFieldTestDuration;
    typedef gdcm::Attribute<0x0024, 0x0089> VisualFieldTestPointSequence;
    typedef gdcm::Attribute<0x0024, 0x0090> VisualFieldTestPointXCoordinate;
    typedef gdcm::Attribute<0x0024, 0x0091> VisualFieldTestPointYCoordinate;
    typedef gdcm::Attribute<0x0024, 0x0092> AgeCorrectedSensitivityDeviationValue;
    typedef gdcm::Attribute<0x0024, 0x0093> StimulusResults;
    typedef gdcm::Attribute<0x0024, 0x0094> SensitivityValue;
    typedef gdcm::Attribute<0x0024, 0x0095> RetestStimulusSeen;
    typedef gdcm::Attribute<0x0024, 0x0096> RetestSensitivityValue;
    typedef gdcm::Attribute<0x0024, 0x0097> VisualFieldTestPointNormalsSequence;
    typedef gdcm::Attribute<0x0024, 0x0098> QuantifiedDefect;
    typedef gdcm::Attribute<0x0024, 0x0100> AgeCorrectedSensitivityDeviationProbabilityValue;
    typedef gdcm::Attribute<0x0024, 0x0102> GeneralizedDefectCorrectedSensitivityDeviationFlag;
    typedef gdcm::Attribute<0x0024, 0x0103> GeneralizedDefectCorrectedSensitivityDeviationValue;
    typedef gdcm::Attribute<0x0024, 0x0104> GeneralizedDefectCorrectedSensitivityDeviationProbabilityValue;
    typedef gdcm::Attribute<0x0024, 0x0105> MinimumSensitivityValue;
    typedef gdcm::Attribute<0x0024, 0x0106> BlindSpotLocalized;
    typedef gdcm::Attribute<0x0024, 0x0107> BlindSpotXCoordinate;
    typedef gdcm::Attribute<0x0024, 0x0108> BlindSpotYCoordinate;
    typedef gdcm::Attribute<0x0024, 0x0110> VisualAcuityMeasurementSequence;
    typedef gdcm::Attribute<0x0024, 0x0112> RefractiveParametersUsedOnPatientSequence;
    typedef gdcm::Attribute<0x0024, 0x0113> MeasurementLaterality;
    typedef gdcm::Attribute<0x0024, 0x0114> OphthalmicPatientClinicalInformationLeftEyeSequence;
    typedef gdcm::Attribute<0x0024, 0x0115> OphthalmicPatientClinicalInformationRightEyeSequence;
    typedef gdcm::Attribute<0x0024, 0x0117> FovealPointNormativeDataFlag;
    typedef gdcm::Attribute<0x0024, 0x0118> FovealPointProbabilityValue;
    typedef gdcm::Attribute<0x0024, 0x0120> ScreeningBaselineMeasured;
    typedef gdcm::Attribute<0x0024, 0x0122> ScreeningBaselineMeasuredSequence;
    typedef gdcm::Attribute<0x0024, 0x0124> ScreeningBaselineType;
    typedef gdcm::Attribute<0x0024, 0x0126> ScreeningBaselineValue;
    typedef gdcm::Attribute<0x0024, 0x0202> AlgorithmSource;
    typedef gdcm::Attribute<0x0024, 0x0306> DataSetName;
    typedef gdcm::Attribute<0x0024, 0x0307> DataSetVersion;
    typedef gdcm::Attribute<0x0024, 0x0308> DataSetSource;
    typedef gdcm::Attribute<0x0024, 0x0309> DataSetDescription;
    typedef gdcm::Attribute<0x0024, 0x0317> VisualFieldTestReliabilityGlobalIndexSequence;
    typedef gdcm::Attribute<0x0024, 0x0320> VisualFieldGlobalResultsIndexSequence;
    typedef gdcm::Attribute<0x0024, 0x0325> DataObservationSequence;
    typedef gdcm::Attribute<0x0024, 0x0338> IndexNormalsFlag;
    typedef gdcm::Attribute<0x0024, 0x0341> IndexProbability;
    typedef gdcm::Attribute<0x0024, 0x0344> IndexProbabilitySequence;
    typedef gdcm::Attribute<0x0028, 0x0002> SamplesPerPixel;
    typedef gdcm::Attribute<0x0028, 0x0003> SamplesPerPixelUsed;
    typedef gdcm::Attribute<0x0028, 0x0004> PhotometricInterpretation;
    typedef gdcm::Attribute<0x0028, 0x0005> ImageDimensions;
    typedef gdcm::Attribute<0x0028, 0x0006> PlanarConfiguration;
    typedef gdcm::Attribute<0x0028, 0x0008> NumberOfFrames;
    typedef gdcm::Attribute<0x0028, 0x0009> FrameIncrementPointer;
    typedef gdcm::Attribute<0x0028, 0x000a> FrameDimensionPointer;
    typedef gdcm::Attribute<0x0028, 0x0010> Rows;
    typedef gdcm::Attribute<0x0028, 0x0011> Columns;
    typedef gdcm::Attribute<0x0028, 0x0012> Planes;
    typedef gdcm::Attribute<0x0028, 0x0014> UltrasoundColorDataPresent;
    // 0x0028, 0x0020 SHALL NOT BE USED
    typedef gdcm::Attribute<0x0028, 0x0030> PixelSpacing;
    typedef gdcm::Attribute<0x0028, 0x0031> ZoomFactor;
    typedef gdcm::Attribute<0x0028, 0x0032> ZoomCenter;
    typedef gdcm::Attribute<0x0028, 0x0034> PixelAspectRatio;
    typedef gdcm::Attribute<0x0028, 0x0040> ImageFormat;
    typedef gdcm::Attribute<0x0028, 0x0050> ManipulatedImage;
    typedef gdcm::Attribute<0x0028, 0x0051> CorrectedImage;
    typedef gdcm::Attribute<0x0028, 0x005f> CompressionRecognitionCode;
    typedef gdcm::Attribute<0x0028, 0x0060> CompressionCode;
    typedef gdcm::Attribute<0x0028, 0x0061> CompressionOriginator;
    typedef gdcm::Attribute<0x0028, 0x0062> CompressionLabel;
    typedef gdcm::Attribute<0x0028, 0x0063> CompressionDescription;
    typedef gdcm::Attribute<0x0028, 0x0065> CompressionSequence;
    typedef gdcm::Attribute<0x0028, 0x0066> CompressionStepPointers;
    typedef gdcm::Attribute<0x0028, 0x0068> RepeatInterval;
    typedef gdcm::Attribute<0x0028, 0x0069> BitsGrouped;
    typedef gdcm::Attribute<0x0028, 0x0070> PerimeterTable;
    typedef gdcm::Attribute<0x0028, 0x0071> PerimeterValue;
    typedef gdcm::Attribute<0x0028, 0x0080> PredictorRows;
    typedef gdcm::Attribute<0x0028, 0x0081> PredictorColumns;
    typedef gdcm::Attribute<0x0028, 0x0082> PredictorConstants;
    typedef gdcm::Attribute<0x0028, 0x0090> BlockedPixels;
    typedef gdcm::Attribute<0x0028, 0x0091> BlockRows;
    typedef gdcm::Attribute<0x0028, 0x0092> BlockColumns;
    typedef gdcm::Attribute<0x0028, 0x0093> RowOverlap;
    typedef gdcm::Attribute<0x0028, 0x0094> ColumnOverlap;
    typedef gdcm::Attribute<0x0028, 0x0100> BitsAllocated;
    typedef gdcm::Attribute<0x0028, 0x0101> BitsStored;
    typedef gdcm::Attribute<0x0028, 0x0102> HighBit;
    typedef gdcm::Attribute<0x0028, 0x0103> PixelRepresentation;
    typedef gdcm::Attribute<0x0028, 0x0104> SmallestValidPixelValue;
    typedef gdcm::Attribute<0x0028, 0x0105> LargestValidPixelValue;
    typedef gdcm::Attribute<0x0028, 0x0106> SmallestImagePixelValue;
    typedef gdcm::Attribute<0x0028, 0x0107> LargestImagePixelValue;
    typedef gdcm::Attribute<0x0028, 0x0108> SmallestPixelValueInSeries;
    typedef gdcm::Attribute<0x0028, 0x0109> LargestPixelValueInSeries;
    typedef gdcm::Attribute<0x0028, 0x0110> SmallestImagePixelValueInPlane;
    typedef gdcm::Attribute<0x0028, 0x0111> LargestImagePixelValueInPlane;
    typedef gdcm::Attribute<0x0028, 0x0120> PixelPaddingValue;
    typedef gdcm::Attribute<0x0028, 0x0121> PixelPaddingRangeLimit;
    typedef gdcm::Attribute<0x0028, 0x0200> ImageLocation;
    typedef gdcm::Attribute<0x0028, 0x0300> QualityControlImage;
    typedef gdcm::Attribute<0x0028, 0x0301> BurnedInAnnotation;
    typedef gdcm::Attribute<0x0028, 0x0302> RecognizableVisualFeatures;
    typedef gdcm::Attribute<0x0028, 0x0303> LongitudinalTemporalInformationModified;
    typedef gdcm::Attribute<0x0028, 0x0304> ReferencedColorPaletteInstanceUID;
    typedef gdcm::Attribute<0x0028, 0x0400> TransformLabel;
    typedef gdcm::Attribute<0x0028, 0x0401> TransformVersionNumber;
    typedef gdcm::Attribute<0x0028, 0x0402> NumberOfTransformSteps;
    typedef gdcm::Attribute<0x0028, 0x0403> SequenceOfCompressedData;
    typedef gdcm::Attribute<0x0028, 0x0404> DetailsOfCoefficients;
    typedef gdcm::Attribute<0x0028, 0x0400> RowsForNthOrderCoefficients;
    typedef gdcm::Attribute<0x0028, 0x0401> ColumnsForNthOrderCoefficients;
    typedef gdcm::Attribute<0x0028, 0x0402> CoefficientCoding;
    typedef gdcm::Attribute<0x0028, 0x0403> CoefficientCodingPointers;
    typedef gdcm::Attribute<0x0028, 0x0700> DCTLabel;
    typedef gdcm::Attribute<0x0028, 0x0701> DataBlockDescription;
    typedef gdcm::Attribute<0x0028, 0x0702> DataBlock;
    typedef gdcm::Attribute<0x0028, 0x0710> NormalizationFactorFormat;
    typedef gdcm::Attribute<0x0028, 0x0720> ZonalMapNumberFormat;
    typedef gdcm::Attribute<0x0028, 0x0721> ZonalMapLocation;
    typedef gdcm::Attribute<0x0028, 0x0722> ZonalMapFormat;
    typedef gdcm::Attribute<0x0028, 0x0730> AdaptiveMapFormat;
    typedef gdcm::Attribute<0x0028, 0x0740> CodeNumberFormat;
    typedef gdcm::Attribute<0x0028, 0x0800> CodeLabel;
    typedef gdcm::Attribute<0x0028, 0x0802> NumberOfTables;
    typedef gdcm::Attribute<0x0028, 0x0803> CodeTableLocation;
    typedef gdcm::Attribute<0x0028, 0x0804> BitsForCodeWord;
    typedef gdcm::Attribute<0x0028, 0x0808> ImageDataLocation;
    typedef gdcm::Attribute<0x0028, 0x0a02> PixelSpacingCalibrationType;
    typedef gdcm::Attribute<0x0028, 0x0a04> PixelSpacingCalibrationDescription;
    typedef gdcm::Attribute<0x0028, 0x1040> PixelIntensityRelationship;
    typedef gdcm::Attribute<0x0028, 0x1041> PixelIntensityRelationshipSign;
    typedef gdcm::Attribute<0x0028, 0x1050> WindowCenter;
    typedef gdcm::Attribute<0x0028, 0x1051> WindowWidth;
    typedef gdcm::Attribute<0x0028, 0x1052> RescaleIntercept;
    typedef gdcm::Attribute<0x0028, 0x1053> RescaleSlope;
    typedef gdcm::Attribute<0x0028, 0x1054> RescaleType;
    typedef gdcm::Attribute<0x0028, 0x1055> WindowCenterWidthExplanation;
    typedef gdcm::Attribute<0x0028, 0x1056> VOILUTFunction;
    typedef gdcm::Attribute<0x0028, 0x1080> GrayScale;
    typedef gdcm::Attribute<0x0028, 0x1090> RecommendedViewingMode;
    typedef gdcm::Attribute<0x0028, 0x1100> GrayLookupTableDescriptor;
    typedef gdcm::Attribute<0x0028, 0x1101> RedPaletteColorLookupTableDescriptor;
    typedef gdcm::Attribute<0x0028, 0x1102> GreenPaletteColorLookupTableDescriptor;
    typedef gdcm::Attribute<0x0028, 0x1103> BluePaletteColorLookupTableDescriptor;
    typedef gdcm::Attribute<0x0028, 0x1104> AlphaPaletteColorLookupTableDescriptor;
    typedef gdcm::Attribute<0x0028, 0x1111> LargeRedPaletteColorLookupTableDescriptor;
    typedef gdcm::Attribute<0x0028, 0x1112> LargeGreenPaletteColorLookupTableDescriptor;
    typedef gdcm::Attribute<0x0028, 0x1113> LargeBluePaletteColorLookupTableDescriptor;
    typedef gdcm::Attribute<0x0028, 0x1199> PaletteColorLookupTableUID;
    typedef gdcm::Attribute<0x0028, 0x1200> GrayLookupTableData;
    typedef gdcm::Attribute<0x0028, 0x1201> RedPaletteColorLookupTableData;
    typedef gdcm::Attribute<0x0028, 0x1202> GreenPaletteColorLookupTableData;
    typedef gdcm::Attribute<0x0028, 0x1203> BluePaletteColorLookupTableData;
    typedef gdcm::Attribute<0x0028, 0x1204> AlphaPaletteColorLookupTableData;
    typedef gdcm::Attribute<0x0028, 0x1211> LargeRedPaletteColorLookupTableData;
    typedef gdcm::Attribute<0x0028, 0x1212> LargeGreenPaletteColorLookupTableData;
    typedef gdcm::Attribute<0x0028, 0x1213> LargeBluePaletteColorLookupTableData;
    typedef gdcm::Attribute<0x0028, 0x1214> LargePaletteColorLookupTableUID;
    typedef gdcm::Attribute<0x0028, 0x1221> SegmentedRedPaletteColorLookupTableData;
    typedef gdcm::Attribute<0x0028, 0x1222> SegmentedGreenPaletteColorLookupTableData;
    typedef gdcm::Attribute<0x0028, 0x1223> SegmentedBluePaletteColorLookupTableData;
    typedef gdcm::Attribute<0x0028, 0x1300> BreastImplantPresent;
    typedef gdcm::Attribute<0x0028, 0x1350> PartialView;
    typedef gdcm::Attribute<0x0028, 0x1351> PartialViewDescription;
    typedef gdcm::Attribute<0x0028, 0x1352> PartialViewCodeSequence;
    typedef gdcm::Attribute<0x0028, 0x135a> SpatialLocationsPreserved;
    typedef gdcm::Attribute<0x0028, 0x1401> DataFrameAssignmentSequence;
    typedef gdcm::Attribute<0x0028, 0x1402> DataPathAssignment;
    typedef gdcm::Attribute<0x0028, 0x1403> BitsMappedToColorLookupTable;
    typedef gdcm::Attribute<0x0028, 0x1404> BlendingLUT1Sequence;
    typedef gdcm::Attribute<0x0028, 0x1405> BlendingLUT1TransferFunction;
    typedef gdcm::Attribute<0x0028, 0x1406> BlendingWeightConstant;
    typedef gdcm::Attribute<0x0028, 0x1407> BlendingLookupTableDescriptor;
    typedef gdcm::Attribute<0x0028, 0x1408> BlendingLookupTableData;
    typedef gdcm::Attribute<0x0028, 0x140b> EnhancedPaletteColorLookupTableSequence;
    typedef gdcm::Attribute<0x0028, 0x140c> BlendingLUT2Sequence;
    typedef gdcm::Attribute<0x0028, 0x140d> BlendingLUT2TransferFunction;
    typedef gdcm::Attribute<0x0028, 0x140e> DataPathID;
    typedef gdcm::Attribute<0x0028, 0x140f> RGBLUTTransferFunction;
    typedef gdcm::Attribute<0x0028, 0x1410> AlphaLUTTransferFunction;
    typedef gdcm::Attribute<0x0028, 0x2000> ICCProfile;
    typedef gdcm::Attribute<0x0028, 0x2110> LossyImageCompression;
    typedef gdcm::Attribute<0x0028, 0x2112> LossyImageCompressionRatio;
    typedef gdcm::Attribute<0x0028, 0x2114> LossyImageCompressionMethod;
    typedef gdcm::Attribute<0x0028, 0x3000> ModalityLUTSequence;
    typedef gdcm::Attribute<0x0028, 0x3002> LUTDescriptor;
    typedef gdcm::Attribute<0x0028, 0x3003> LUTExplanation;
    typedef gdcm::Attribute<0x0028, 0x3004> ModalityLUTType;
    typedef gdcm::Attribute<0x0028, 0x3006> LUTData;
    typedef gdcm::Attribute<0x0028, 0x3010> VOILUTSequence;
    typedef gdcm::Attribute<0x0028, 0x3110> SoftcopyVOILUTSequence;
    typedef gdcm::Attribute<0x0028, 0x4000> ImagePresentationComments;
    typedef gdcm::Attribute<0x0028, 0x5000> BiPlaneAcquisitionSequence;
    typedef gdcm::Attribute<0x0028, 0x6010> RepresentativeFrameNumber;
    typedef gdcm::Attribute<0x0028, 0x6020> FrameNumbersOfInterest;
    typedef gdcm::Attribute<0x0028, 0x6022> FrameOfInterestDescription;
    typedef gdcm::Attribute<0x0028, 0x6023> FrameOfInterestType;
    typedef gdcm::Attribute<0x0028, 0x6030> MaskPointers;
    typedef gdcm::Attribute<0x0028, 0x6040> RWavePointer;
    typedef gdcm::Attribute<0x0028, 0x6100> MaskSubtractionSequence;
    typedef gdcm::Attribute<0x0028, 0x6101> MaskOperation;
    typedef gdcm::Attribute<0x0028, 0x6102> ApplicableFrameRange;
    typedef gdcm::Attribute<0x0028, 0x6110> MaskFrameNumbers;
    typedef gdcm::Attribute<0x0028, 0x6112> ContrastFrameAveraging;
    typedef gdcm::Attribute<0x0028, 0x6114> MaskSubPixelShift;
    typedef gdcm::Attribute<0x0028, 0x6120> TIDOffset;
    typedef gdcm::Attribute<0x0028, 0x6190> MaskOperationExplanation;
    typedef gdcm::Attribute<0x0028, 0x7fe0> PixelDataProviderURL;
    typedef gdcm::Attribute<0x0028, 0x9001> DataPointRows;
    typedef gdcm::Attribute<0x0028, 0x9002> DataPointColumns;
    typedef gdcm::Attribute<0x0028, 0x9003> SignalDomainColumns;
    typedef gdcm::Attribute<0x0028, 0x9099> LargestMonochromePixelValue;
    typedef gdcm::Attribute<0x0028, 0x9108> DataRepresentation;
    typedef gdcm::Attribute<0x0028, 0x9110> PixelMeasuresSequence;
    typedef gdcm::Attribute<0x0028, 0x9132> FrameVOILUTSequence;
    typedef gdcm::Attribute<0x0028, 0x9145> PixelValueTransformationSequence;
    typedef gdcm::Attribute<0x0028, 0x9235> SignalDomainRows;
    typedef gdcm::Attribute<0x0028, 0x9411> DisplayFilterPercentage;
    typedef gdcm::Attribute<0x0028, 0x9415> FramePixelShiftSequence;
    typedef gdcm::Attribute<0x0028, 0x9416> SubtractionItemID;
    typedef gdcm::Attribute<0x0028, 0x9422> PixelIntensityRelationshipLUTSequence;
    typedef gdcm::Attribute<0x0028, 0x9443> FramePixelDataPropertiesSequence;
    typedef gdcm::Attribute<0x0028, 0x9444> GeometricalProperties;
    typedef gdcm::Attribute<0x0028, 0x9445> GeometricMaximumDistortion;
    typedef gdcm::Attribute<0x0028, 0x9446> ImageProcessingApplied;
    typedef gdcm::Attribute<0x0028, 0x9454> MaskSelectionMode;
    typedef gdcm::Attribute<0x0028, 0x9474> LUTFunction;
    typedef gdcm::Attribute<0x0028, 0x9478> MaskVisibilityPercentage;
    typedef gdcm::Attribute<0x0028, 0x9501> PixelShiftSequence;
    typedef gdcm::Attribute<0x0028, 0x9502> RegionPixelShiftSequence;
    typedef gdcm::Attribute<0x0028, 0x9503> VerticesOfTheRegion;
    typedef gdcm::Attribute<0x0028, 0x9505> MultiFramePresentationSequence;
    typedef gdcm::Attribute<0x0028, 0x9506> PixelShiftFrameRange;
    typedef gdcm::Attribute<0x0028, 0x9507> LUTFrameRange;
    typedef gdcm::Attribute<0x0028, 0x9520> ImageToEquipmentMappingMatrix;
    typedef gdcm::Attribute<0x0028, 0x9537> EquipmentCoordinateSystemIdentification;
    typedef gdcm::Attribute<0x0032, 0x000a> StudyStatusID;
    typedef gdcm::Attribute<0x0032, 0x000c> StudyPriorityID;
    typedef gdcm::Attribute<0x0032, 0x0012> StudyIDIssuer;
    typedef gdcm::Attribute<0x0032, 0x0032> StudyVerifiedDate;
    typedef gdcm::Attribute<0x0032, 0x0033> StudyVerifiedTime;
    typedef gdcm::Attribute<0x0032, 0x0034> StudyReadDate;
    typedef gdcm::Attribute<0x0032, 0x0035> StudyReadTime;
    typedef gdcm::Attribute<0x0032, 0x1000> ScheduledStudyStartDate;
    typedef gdcm::Attribute<0x0032, 0x1001> ScheduledStudyStartTime;
    typedef gdcm::Attribute<0x0032, 0x1010> ScheduledStudyStopDate;
    typedef gdcm::Attribute<0x0032, 0x1011> ScheduledStudyStopTime;
    typedef gdcm::Attribute<0x0032, 0x1020> ScheduledStudyLocation;
    typedef gdcm::Attribute<0x0032, 0x1021> ScheduledStudyLocationAETitle;
    typedef gdcm::Attribute<0x0032, 0x1030> ReasonForStudy;
    typedef gdcm::Attribute<0x0032, 0x1031> RequestingPhysicianIdentificationSequence;
    typedef gdcm::Attribute<0x0032, 0x1032> RequestingPhysician;
    typedef gdcm::Attribute<0x0032, 0x1033> RequestingService;
    typedef gdcm::Attribute<0x0032, 0x1034> RequestingServiceCodeSequence;
    typedef gdcm::Attribute<0x0032, 0x1040> StudyArrivalDate;
    typedef gdcm::Attribute<0x0032, 0x1041> StudyArrivalTime;
    typedef gdcm::Attribute<0x0032, 0x1050> StudyCompletionDate;
    typedef gdcm::Attribute<0x0032, 0x1051> StudyCompletionTime;
    typedef gdcm::Attribute<0x0032, 0x1055> StudyComponentStatusID;
    typedef gdcm::Attribute<0x0032, 0x1060> RequestedProcedureDescription;
    typedef gdcm::Attribute<0x0032, 0x1064> RequestedProcedureCodeSequence;
    typedef gdcm::Attribute<0x0032, 0x1070> RequestedContrastAgent;
    typedef gdcm::Attribute<0x0032, 0x4000> StudyComments;
    typedef gdcm::Attribute<0x0038, 0x0004> ReferencedPatientAliasSequence;
    typedef gdcm::Attribute<0x0038, 0x0008> VisitStatusID;
    typedef gdcm::Attribute<0x0038, 0x0010> AdmissionID;
    typedef gdcm::Attribute<0x0038, 0x0011> IssuerOfAdmissionID;
    typedef gdcm::Attribute<0x0038, 0x0014> IssuerOfAdmissionIDSequence;
    typedef gdcm::Attribute<0x0038, 0x0016> RouteOfAdmissions;
    typedef gdcm::Attribute<0x0038, 0x001a> ScheduledAdmissionDate;
    typedef gdcm::Attribute<0x0038, 0x001b> ScheduledAdmissionTime;
    typedef gdcm::Attribute<0x0038, 0x001c> ScheduledDischargeDate;
    typedef gdcm::Attribute<0x0038, 0x001d> ScheduledDischargeTime;
    typedef gdcm::Attribute<0x0038, 0x001e> ScheduledPatientInstitutionResidence;
    typedef gdcm::Attribute<0x0038, 0x0020> AdmittingDate;
    typedef gdcm::Attribute<0x0038, 0x0021> AdmittingTime;
    typedef gdcm::Attribute<0x0038, 0x0030> DischargeDate;
    typedef gdcm::Attribute<0x0038, 0x0032> DischargeTime;
    typedef gdcm::Attribute<0x0038, 0x0040> DischargeDiagnosisDescription;
    typedef gdcm::Attribute<0x0038, 0x0044> DischargeDiagnosisCodeSequence;
    typedef gdcm::Attribute<0x0038, 0x0050> SpecialNeeds;
    typedef gdcm::Attribute<0x0038, 0x0060> ServiceEpisodeID;
    typedef gdcm::Attribute<0x0038, 0x0061> IssuerOfServiceEpisodeID;
    typedef gdcm::Attribute<0x0038, 0x0062> ServiceEpisodeDescription;
    typedef gdcm::Attribute<0x0038, 0x0064> IssuerOfServiceEpisodeIDSequence;
    typedef gdcm::Attribute<0x0038, 0x0100> PertinentDocumentsSequence;
    typedef gdcm::Attribute<0x0038, 0x0300> CurrentPatientLocation;
    typedef gdcm::Attribute<0x0038, 0x0400> PatientInstitutionResidence;
    typedef gdcm::Attribute<0x0038, 0x0500> PatientState;
    typedef gdcm::Attribute<0x0038, 0x0502> PatientClinicalTrialParticipationSequence;
    typedef gdcm::Attribute<0x0038, 0x4000> VisitComments;
    typedef gdcm::Attribute<0x003a, 0x0004> WaveformOriginality;
    typedef gdcm::Attribute<0x003a, 0x0005> NumberOfWaveformChannels;
    typedef gdcm::Attribute<0x003a, 0x0010> NumberOfWaveformSamples;
    typedef gdcm::Attribute<0x003a, 0x001a> SamplingFrequency;
    typedef gdcm::Attribute<0x003a, 0x0020> MultiplexGroupLabel;
    typedef gdcm::Attribute<0x003a, 0x0200> ChannelDefinitionSequence;
    typedef gdcm::Attribute<0x003a, 0x0202> WaveformChannelNumber;
    typedef gdcm::Attribute<0x003a, 0x0203> ChannelLabel;
    typedef gdcm::Attribute<0x003a, 0x0205> ChannelStatus;
    typedef gdcm::Attribute<0x003a, 0x0208> ChannelSourceSequence;
    typedef gdcm::Attribute<0x003a, 0x0209> ChannelSourceModifiersSequence;
    typedef gdcm::Attribute<0x003a, 0x020a> SourceWaveformSequence;
    typedef gdcm::Attribute<0x003a, 0x020c> ChannelDerivationDescription;
    typedef gdcm::Attribute<0x003a, 0x0210> ChannelSensitivity;
    typedef gdcm::Attribute<0x003a, 0x0211> ChannelSensitivityUnitsSequence;
    typedef gdcm::Attribute<0x003a, 0x0212> ChannelSensitivityCorrectionFactor;
    typedef gdcm::Attribute<0x003a, 0x0213> ChannelBaseline;
    typedef gdcm::Attribute<0x003a, 0x0214> ChannelTimeSkew;
    typedef gdcm::Attribute<0x003a, 0x0215> ChannelSampleSkew;
    typedef gdcm::Attribute<0x003a, 0x0218> ChannelOffset;
    typedef gdcm::Attribute<0x003a, 0x021a> WaveformBitsStored;
    typedef gdcm::Attribute<0x003a, 0x0220> FilterLowFrequency;
    typedef gdcm::Attribute<0x003a, 0x0221> FilterHighFrequency;
    typedef gdcm::Attribute<0x003a, 0x0222> NotchFilterFrequency;
    typedef gdcm::Attribute<0x003a, 0x0223> NotchFilterBandwidth;
    typedef gdcm::Attribute<0x003a, 0x0230> WaveformDataDisplayScale;
    typedef gdcm::Attribute<0x003a, 0x0231> WaveformDisplayBackgroundCIELabValue;
    typedef gdcm::Attribute<0x003a, 0x0240> WaveformPresentationGroupSequence;
    typedef gdcm::Attribute<0x003a, 0x0241> PresentationGroupNumber;
    typedef gdcm::Attribute<0x003a, 0x0242> ChannelDisplaySequence;
    typedef gdcm::Attribute<0x003a, 0x0244> ChannelRecommendedDisplayCIELabValue;
    typedef gdcm::Attribute<0x003a, 0x0245> ChannelPosition;
    typedef gdcm::Attribute<0x003a, 0x0246> DisplayShadingFlag;
    typedef gdcm::Attribute<0x003a, 0x0247> FractionalChannelDisplayScale;
    typedef gdcm::Attribute<0x003a, 0x0248> AbsoluteChannelDisplayScale;
    typedef gdcm::Attribute<0x003a, 0x0300> MultiplexedAudioChannelsDescriptionCodeSequence;
    typedef gdcm::Attribute<0x003a, 0x0301> ChannelIdentificationCode;
    typedef gdcm::Attribute<0x003a, 0x0302> ChannelMode;
    typedef gdcm::Attribute<0x0040, 0x0001> ScheduledStationAETitle;
    typedef gdcm::Attribute<0x0040, 0x0002> ScheduledProcedureStepStartDate;
    typedef gdcm::Attribute<0x0040, 0x0003> ScheduledProcedureStepStartTime;
    typedef gdcm::Attribute<0x0040, 0x0004> ScheduledProcedureStepEndDate;
    typedef gdcm::Attribute<0x0040, 0x0005> ScheduledProcedureStepEndTime;
    typedef gdcm::Attribute<0x0040, 0x0006> ScheduledPerformingPhysicianName;
    typedef gdcm::Attribute<0x0040, 0x0007> ScheduledProcedureStepDescription;
    typedef gdcm::Attribute<0x0040, 0x0008> ScheduledProtocolCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x0009> ScheduledProcedureStepID;
    typedef gdcm::Attribute<0x0040, 0x000a> StageCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x000b> ScheduledPerformingPhysicianIdentificationSequence;
    typedef gdcm::Attribute<0x0040, 0x0010> ScheduledStationName;
    typedef gdcm::Attribute<0x0040, 0x0011> ScheduledProcedureStepLocation;
    typedef gdcm::Attribute<0x0040, 0x0012> PreMedication;
    typedef gdcm::Attribute<0x0040, 0x0020> ScheduledProcedureStepStatus;
    typedef gdcm::Attribute<0x0040, 0x0026> OrderPlacerIdentifierSequence;
    typedef gdcm::Attribute<0x0040, 0x0027> OrderFillerIdentifierSequence;
    typedef gdcm::Attribute<0x0040, 0x0031> LocalNamespaceEntityID;
    typedef gdcm::Attribute<0x0040, 0x0032> UniversalEntityID;
    typedef gdcm::Attribute<0x0040, 0x0033> UniversalEntityIDType;
    typedef gdcm::Attribute<0x0040, 0x0035> IdentifierTypeCode;
    typedef gdcm::Attribute<0x0040, 0x0036> AssigningFacilitySequence;
    typedef gdcm::Attribute<0x0040, 0x0039> AssigningJurisdictionCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x003a> AssigningAgencyOrDepartmentCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x0100> ScheduledProcedureStepSequence;
    typedef gdcm::Attribute<0x0040, 0x0220> ReferencedNonImageCompositeSOPInstanceSequence;
    typedef gdcm::Attribute<0x0040, 0x0241> PerformedStationAETitle;
    typedef gdcm::Attribute<0x0040, 0x0242> PerformedStationName;
    typedef gdcm::Attribute<0x0040, 0x0243> PerformedLocation;
    typedef gdcm::Attribute<0x0040, 0x0244> PerformedProcedureStepStartDate;
    typedef gdcm::Attribute<0x0040, 0x0245> PerformedProcedureStepStartTime;
    typedef gdcm::Attribute<0x0040, 0x0250> PerformedProcedureStepEndDate;
    typedef gdcm::Attribute<0x0040, 0x0251> PerformedProcedureStepEndTime;
    typedef gdcm::Attribute<0x0040, 0x0252> PerformedProcedureStepStatus;
    typedef gdcm::Attribute<0x0040, 0x0253> PerformedProcedureStepID;
    typedef gdcm::Attribute<0x0040, 0x0254> PerformedProcedureStepDescription;
    typedef gdcm::Attribute<0x0040, 0x0255> PerformedProcedureTypeDescription;
    typedef gdcm::Attribute<0x0040, 0x0260> PerformedProtocolCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x0261> PerformedProtocolType;
    typedef gdcm::Attribute<0x0040, 0x0270> ScheduledStepAttributesSequence;
    typedef gdcm::Attribute<0x0040, 0x0275> RequestAttributesSequence;
    typedef gdcm::Attribute<0x0040, 0x0280> CommentsOnThePerformedProcedureStep;
    typedef gdcm::Attribute<0x0040, 0x0281> PerformedProcedureStepDiscontinuationReasonCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x0293> QuantitySequence;
    typedef gdcm::Attribute<0x0040, 0x0294> Quantity;
    typedef gdcm::Attribute<0x0040, 0x0295> MeasuringUnitsSequence;
    typedef gdcm::Attribute<0x0040, 0x0296> BillingItemSequence;
    typedef gdcm::Attribute<0x0040, 0x0300> TotalTimeOfFluoroscopy;
    typedef gdcm::Attribute<0x0040, 0x0301> TotalNumberOfExposures;
    typedef gdcm::Attribute<0x0040, 0x0302> EntranceDose;
    typedef gdcm::Attribute<0x0040, 0x0303> ExposedArea;
    typedef gdcm::Attribute<0x0040, 0x0306> DistanceSourceToEntrance;
    typedef gdcm::Attribute<0x0040, 0x0307> DistanceSourceToSupport;
    typedef gdcm::Attribute<0x0040, 0x030e> ExposureDoseSequence;
    typedef gdcm::Attribute<0x0040, 0x0310> CommentsOnRadiationDose;
    typedef gdcm::Attribute<0x0040, 0x0312> XRayOutput;
    typedef gdcm::Attribute<0x0040, 0x0314> HalfValueLayer;
    typedef gdcm::Attribute<0x0040, 0x0316> OrganDose;
    typedef gdcm::Attribute<0x0040, 0x0318> OrganExposed;
    typedef gdcm::Attribute<0x0040, 0x0320> BillingProcedureStepSequence;
    typedef gdcm::Attribute<0x0040, 0x0321> FilmConsumptionSequence;
    typedef gdcm::Attribute<0x0040, 0x0324> BillingSuppliesAndDevicesSequence;
    typedef gdcm::Attribute<0x0040, 0x0330> ReferencedProcedureStepSequence;
    typedef gdcm::Attribute<0x0040, 0x0340> PerformedSeriesSequence;
    typedef gdcm::Attribute<0x0040, 0x0400> CommentsOnTheScheduledProcedureStep;
    typedef gdcm::Attribute<0x0040, 0x0440> ProtocolContextSequence;
    typedef gdcm::Attribute<0x0040, 0x0441> ContentItemModifierSequence;
    typedef gdcm::Attribute<0x0040, 0x0500> ScheduledSpecimenSequence;
    typedef gdcm::Attribute<0x0040, 0x050a> SpecimenAccessionNumber;
    typedef gdcm::Attribute<0x0040, 0x0512> ContainerIdentifier;
    typedef gdcm::Attribute<0x0040, 0x0513> IssuerOfTheContainerIdentifierSequence;
    typedef gdcm::Attribute<0x0040, 0x0515> AlternateContainerIdentifierSequence;
    typedef gdcm::Attribute<0x0040, 0x0518> ContainerTypeCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x051a> ContainerDescription;
    typedef gdcm::Attribute<0x0040, 0x0520> ContainerComponentSequence;
    typedef gdcm::Attribute<0x0040, 0x0550> SpecimenSequence;
    typedef gdcm::Attribute<0x0040, 0x0551> SpecimenIdentifier;
    typedef gdcm::Attribute<0x0040, 0x0552> SpecimenDescriptionSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0x0553> SpecimenDescriptionTrial;
    typedef gdcm::Attribute<0x0040, 0x0554> SpecimenUID;
    typedef gdcm::Attribute<0x0040, 0x0555> AcquisitionContextSequence;
    typedef gdcm::Attribute<0x0040, 0x0556> AcquisitionContextDescription;
    typedef gdcm::Attribute<0x0040, 0x059a> SpecimenTypeCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x0560> SpecimenDescriptionSequence;
    typedef gdcm::Attribute<0x0040, 0x0562> IssuerOfTheSpecimenIdentifierSequence;
    typedef gdcm::Attribute<0x0040, 0x0600> SpecimenShortDescription;
    typedef gdcm::Attribute<0x0040, 0x0602> SpecimenDetailedDescription;
    typedef gdcm::Attribute<0x0040, 0x0610> SpecimenPreparationSequence;
    typedef gdcm::Attribute<0x0040, 0x0612> SpecimenPreparationStepContentItemSequence;
    typedef gdcm::Attribute<0x0040, 0x0620> SpecimenLocalizationContentItemSequence;
    typedef gdcm::Attribute<0x0040, 0x06fa> SlideIdentifier;
    typedef gdcm::Attribute<0x0040, 0x071a> ImageCenterPointCoordinatesSequence;
    typedef gdcm::Attribute<0x0040, 0x072a> XOffsetInSlideCoordinateSystem;
    typedef gdcm::Attribute<0x0040, 0x073a> YOffsetInSlideCoordinateSystem;
    typedef gdcm::Attribute<0x0040, 0x074a> ZOffsetInSlideCoordinateSystem;
    typedef gdcm::Attribute<0x0040, 0x08d8> PixelSpacingSequence;
    typedef gdcm::Attribute<0x0040, 0x08da> CoordinateSystemAxisCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x08ea> MeasurementUnitsCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x09f8> VitalStainCodeSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0x1001> RequestedProcedureID;
    typedef gdcm::Attribute<0x0040, 0x1002> ReasonForTheRequestedProcedure;
    typedef gdcm::Attribute<0x0040, 0x1003> RequestedProcedurePriority;
    typedef gdcm::Attribute<0x0040, 0x1004> PatientTransportArrangements;
    typedef gdcm::Attribute<0x0040, 0x1005> RequestedProcedureLocation;
    typedef gdcm::Attribute<0x0040, 0x1006> PlacerOrderNumberProcedure;
    typedef gdcm::Attribute<0x0040, 0x1007> FillerOrderNumberProcedure;
    typedef gdcm::Attribute<0x0040, 0x1008> ConfidentialityCode;
    typedef gdcm::Attribute<0x0040, 0x1009> ReportingPriority;
    typedef gdcm::Attribute<0x0040, 0x100a> ReasonForRequestedProcedureCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x1010> NamesOfIntendedRecipientsOfResults;
    typedef gdcm::Attribute<0x0040, 0x1011> IntendedRecipientsOfResultsIdentificationSequence;
    typedef gdcm::Attribute<0x0040, 0x1012> ReasonForPerformedProcedureCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x1060> RequestedProcedureDescriptionTrial;
    typedef gdcm::Attribute<0x0040, 0x1101> PersonIdentificationCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x1102> PersonAddress;
    typedef gdcm::Attribute<0x0040, 0x1103> PersonTelephoneNumbers;
    typedef gdcm::Attribute<0x0040, 0x1400> RequestedProcedureComments;
    typedef gdcm::Attribute<0x0040, 0x2001> ReasonForTheImagingServiceRequest;
    typedef gdcm::Attribute<0x0040, 0x2004> IssueDateOfImagingServiceRequest;
    typedef gdcm::Attribute<0x0040, 0x2005> IssueTimeOfImagingServiceRequest;
    typedef gdcm::Attribute<0x0040, 0x2006> PlacerOrderNumberImagingServiceRequestRetired;
    typedef gdcm::Attribute<0x0040, 0x2007> FillerOrderNumberImagingServiceRequestRetired;
    typedef gdcm::Attribute<0x0040, 0x2008> OrderEnteredBy;
    typedef gdcm::Attribute<0x0040, 0x2009> OrderEntererLocation;
    typedef gdcm::Attribute<0x0040, 0x2010> OrderCallbackPhoneNumber;
    typedef gdcm::Attribute<0x0040, 0x2016> PlacerOrderNumberImagingServiceRequest;
    typedef gdcm::Attribute<0x0040, 0x2017> FillerOrderNumberImagingServiceRequest;
    typedef gdcm::Attribute<0x0040, 0x2400> ImagingServiceRequestComments;
    typedef gdcm::Attribute<0x0040, 0x3001> ConfidentialityConstraintOnPatientDataDescription;
    typedef gdcm::Attribute<0x0040, 0x4001> GeneralPurposeScheduledProcedureStepStatus;
    typedef gdcm::Attribute<0x0040, 0x4002> GeneralPurposePerformedProcedureStepStatus;
    typedef gdcm::Attribute<0x0040, 0x4003> GeneralPurposeScheduledProcedureStepPriority;
    typedef gdcm::Attribute<0x0040, 0x4004> ScheduledProcessingApplicationsCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x4005> ScheduledProcedureStepStartDateTime;
    typedef gdcm::Attribute<0x0040, 0x4006> MultipleCopiesFlag;
    typedef gdcm::Attribute<0x0040, 0x4007> PerformedProcessingApplicationsCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x4009> HumanPerformerCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x4010> ScheduledProcedureStepModificationDateTime;
    typedef gdcm::Attribute<0x0040, 0x4011> ExpectedCompletionDateTime;
    typedef gdcm::Attribute<0x0040, 0x4015> ResultingGeneralPurposePerformedProcedureStepsSequence;
    typedef gdcm::Attribute<0x0040, 0x4016> ReferencedGeneralPurposeScheduledProcedureStepSequence;
    typedef gdcm::Attribute<0x0040, 0x4018> ScheduledWorkitemCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x4019> PerformedWorkitemCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x4020> InputAvailabilityFlag;
    typedef gdcm::Attribute<0x0040, 0x4021> InputInformationSequence;
    typedef gdcm::Attribute<0x0040, 0x4022> RelevantInformationSequence;
    typedef gdcm::Attribute<0x0040, 0x4023> ReferencedGeneralPurposeScheduledProcedureStepTransactionUID;
    typedef gdcm::Attribute<0x0040, 0x4025> ScheduledStationNameCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x4026> ScheduledStationClassCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x4027> ScheduledStationGeographicLocationCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x4028> PerformedStationNameCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x4029> PerformedStationClassCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x4030> PerformedStationGeographicLocationCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x4031> RequestedSubsequentWorkitemCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x4032> NonDICOMOutputCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x4033> OutputInformationSequence;
    typedef gdcm::Attribute<0x0040, 0x4034> ScheduledHumanPerformersSequence;
    typedef gdcm::Attribute<0x0040, 0x4035> ActualHumanPerformersSequence;
    typedef gdcm::Attribute<0x0040, 0x4036> HumanPerformerOrganization;
    typedef gdcm::Attribute<0x0040, 0x4037> HumanPerformerName;
    typedef gdcm::Attribute<0x0040, 0x4040> RawDataHandling;
    typedef gdcm::Attribute<0x0040, 0x4041> InputReadinessState;
    typedef gdcm::Attribute<0x0040, 0x4050> PerformedProcedureStepStartDateTime;
    typedef gdcm::Attribute<0x0040, 0x4051> PerformedProcedureStepEndDateTime;
    typedef gdcm::Attribute<0x0040, 0x4052> ProcedureStepCancellationDateTime;
    typedef gdcm::Attribute<0x0040, 0x8302> EntranceDoseInmGy;
    typedef gdcm::Attribute<0x0040, 0x9094> ReferencedImageRealWorldValueMappingSequence;
    typedef gdcm::Attribute<0x0040, 0x9096> RealWorldValueMappingSequence;
    typedef gdcm::Attribute<0x0040, 0x9098> PixelValueMappingCodeSequence;
    typedef gdcm::Attribute<0x0040, 0x9210> LUTLabel;
    typedef gdcm::Attribute<0x0040, 0x9211> RealWorldValueLastValueMapped;
    typedef gdcm::Attribute<0x0040, 0x9212> RealWorldValueLUTData;
    typedef gdcm::Attribute<0x0040, 0x9216> RealWorldValueFirstValueMapped;
    typedef gdcm::Attribute<0x0040, 0x9224> RealWorldValueIntercept;
    typedef gdcm::Attribute<0x0040, 0x9225> RealWorldValueSlope;
    typedef gdcm::Attribute<0x0040, 0xa007> FindingsFlagTrial;
    typedef gdcm::Attribute<0x0040, 0xa010> RelationshipType;
    typedef gdcm::Attribute<0x0040, 0xa020> FindingsSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa021> FindingsGroupUIDTrial;
    typedef gdcm::Attribute<0x0040, 0xa022> ReferencedFindingsGroupUIDTrial;
    typedef gdcm::Attribute<0x0040, 0xa023> FindingsGroupRecordingDateTrial;
    typedef gdcm::Attribute<0x0040, 0xa024> FindingsGroupRecordingTimeTrial;
    typedef gdcm::Attribute<0x0040, 0xa026> FindingsSourceCategoryCodeSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa027> VerifyingOrganization;
    typedef gdcm::Attribute<0x0040, 0xa028> DocumentingOrganizationIdentifierCodeSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa030> VerificationDateTime;
    typedef gdcm::Attribute<0x0040, 0xa032> ObservationDateTime;
    typedef gdcm::Attribute<0x0040, 0xa040> ValueType;
    typedef gdcm::Attribute<0x0040, 0xa043> ConceptNameCodeSequence;
    typedef gdcm::Attribute<0x0040, 0xa047> MeasurementPrecisionDescriptionTrial;
    typedef gdcm::Attribute<0x0040, 0xa050> ContinuityOfContent;
    typedef gdcm::Attribute<0x0040, 0xa057> UrgencyOrPriorityAlertsTrial;
    typedef gdcm::Attribute<0x0040, 0xa060> SequencingIndicatorTrial;
    typedef gdcm::Attribute<0x0040, 0xa066> DocumentIdentifierCodeSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa067> DocumentAuthorTrial;
    typedef gdcm::Attribute<0x0040, 0xa068> DocumentAuthorIdentifierCodeSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa070> IdentifierCodeSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa073> VerifyingObserverSequence;
    typedef gdcm::Attribute<0x0040, 0xa074> ObjectBinaryIdentifierTrial;
    typedef gdcm::Attribute<0x0040, 0xa075> VerifyingObserverName;
    typedef gdcm::Attribute<0x0040, 0xa076> DocumentingObserverIdentifierCodeSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa078> AuthorObserverSequence;
    typedef gdcm::Attribute<0x0040, 0xa07a> ParticipantSequence;
    typedef gdcm::Attribute<0x0040, 0xa07c> CustodialOrganizationSequence;
    typedef gdcm::Attribute<0x0040, 0xa080> ParticipationType;
    typedef gdcm::Attribute<0x0040, 0xa082> ParticipationDateTime;
    typedef gdcm::Attribute<0x0040, 0xa084> ObserverType;
    typedef gdcm::Attribute<0x0040, 0xa085> ProcedureIdentifierCodeSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa088> VerifyingObserverIdentificationCodeSequence;
    typedef gdcm::Attribute<0x0040, 0xa089> ObjectDirectoryBinaryIdentifierTrial;
    typedef gdcm::Attribute<0x0040, 0xa090> EquivalentCDADocumentSequence;
    typedef gdcm::Attribute<0x0040, 0xa0b0> ReferencedWaveformChannels;
    typedef gdcm::Attribute<0x0040, 0xa110> DateOfDocumentOrVerbalTransactionTrial;
    typedef gdcm::Attribute<0x0040, 0xa112> TimeOfDocumentCreationOrVerbalTransactionTrial;
    typedef gdcm::Attribute<0x0040, 0xa120> DateTime;
    typedef gdcm::Attribute<0x0040, 0xa121> Date;
    typedef gdcm::Attribute<0x0040, 0xa122> Time;
    typedef gdcm::Attribute<0x0040, 0xa123> PersonName;
    typedef gdcm::Attribute<0x0040, 0xa124> UID;
    typedef gdcm::Attribute<0x0040, 0xa125> ReportStatusIDTrial;
    typedef gdcm::Attribute<0x0040, 0xa130> TemporalRangeType;
    typedef gdcm::Attribute<0x0040, 0xa132> ReferencedSamplePositions;
    typedef gdcm::Attribute<0x0040, 0xa136> ReferencedFrameNumbers;
    typedef gdcm::Attribute<0x0040, 0xa138> ReferencedTimeOffsets;
    typedef gdcm::Attribute<0x0040, 0xa13a> ReferencedDateTime;
    typedef gdcm::Attribute<0x0040, 0xa160> TextValue;
    typedef gdcm::Attribute<0x0040, 0xa167> ObservationCategoryCodeSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa168> ConceptCodeSequence;
    typedef gdcm::Attribute<0x0040, 0xa16a> BibliographicCitationTrial;
    typedef gdcm::Attribute<0x0040, 0xa170> PurposeOfReferenceCodeSequence;
    typedef gdcm::Attribute<0x0040, 0xa171> ObservationUIDTrial;
    typedef gdcm::Attribute<0x0040, 0xa172> ReferencedObservationUIDTrial;
    typedef gdcm::Attribute<0x0040, 0xa173> ReferencedObservationClassTrial;
    typedef gdcm::Attribute<0x0040, 0xa174> ReferencedObjectObservationClassTrial;
    typedef gdcm::Attribute<0x0040, 0xa180> AnnotationGroupNumber;
    typedef gdcm::Attribute<0x0040, 0xa192> ObservationDateTrial;
    typedef gdcm::Attribute<0x0040, 0xa193> ObservationTimeTrial;
    typedef gdcm::Attribute<0x0040, 0xa194> MeasurementAutomationTrial;
    typedef gdcm::Attribute<0x0040, 0xa195> ModifierCodeSequence;
    typedef gdcm::Attribute<0x0040, 0xa224> IdentificationDescriptionTrial;
    typedef gdcm::Attribute<0x0040, 0xa290> CoordinatesSetGeometricTypeTrial;
    typedef gdcm::Attribute<0x0040, 0xa296> AlgorithmCodeSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa297> AlgorithmDescriptionTrial;
    typedef gdcm::Attribute<0x0040, 0xa29a> PixelCoordinatesSetTrial;
    typedef gdcm::Attribute<0x0040, 0xa300> MeasuredValueSequence;
    typedef gdcm::Attribute<0x0040, 0xa301> NumericValueQualifierCodeSequence;
    typedef gdcm::Attribute<0x0040, 0xa307> CurrentObserverTrial;
    typedef gdcm::Attribute<0x0040, 0xa30a> NumericValue;
    typedef gdcm::Attribute<0x0040, 0xa313> ReferencedAccessionSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa33a> ReportStatusCommentTrial;
    typedef gdcm::Attribute<0x0040, 0xa340> ProcedureContextSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa352> VerbalSourceTrial;
    typedef gdcm::Attribute<0x0040, 0xa353> AddressTrial;
    typedef gdcm::Attribute<0x0040, 0xa354> TelephoneNumberTrial;
    typedef gdcm::Attribute<0x0040, 0xa358> VerbalSourceIdentifierCodeSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa360> PredecessorDocumentsSequence;
    typedef gdcm::Attribute<0x0040, 0xa370> ReferencedRequestSequence;
    typedef gdcm::Attribute<0x0040, 0xa372> PerformedProcedureCodeSequence;
    typedef gdcm::Attribute<0x0040, 0xa375> CurrentRequestedProcedureEvidenceSequence;
    typedef gdcm::Attribute<0x0040, 0xa380> ReportDetailSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa385> PertinentOtherEvidenceSequence;
    typedef gdcm::Attribute<0x0040, 0xa390> HL7StructuredDocumentReferenceSequence;
    typedef gdcm::Attribute<0x0040, 0xa402> ObservationSubjectUIDTrial;
    typedef gdcm::Attribute<0x0040, 0xa403> ObservationSubjectClassTrial;
    typedef gdcm::Attribute<0x0040, 0xa404> ObservationSubjectTypeCodeSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa491> CompletionFlag;
    typedef gdcm::Attribute<0x0040, 0xa492> CompletionFlagDescription;
    typedef gdcm::Attribute<0x0040, 0xa493> VerificationFlag;
    typedef gdcm::Attribute<0x0040, 0xa494> ArchiveRequested;
    typedef gdcm::Attribute<0x0040, 0xa496> PreliminaryFlag;
    typedef gdcm::Attribute<0x0040, 0xa504> ContentTemplateSequence;
    typedef gdcm::Attribute<0x0040, 0xa525> IdenticalDocumentsSequence;
    typedef gdcm::Attribute<0x0040, 0xa600> ObservationSubjectContextFlagTrial;
    typedef gdcm::Attribute<0x0040, 0xa601> ObserverContextFlagTrial;
    typedef gdcm::Attribute<0x0040, 0xa603> ProcedureContextFlagTrial;
    typedef gdcm::Attribute<0x0040, 0xa730> ContentSequence;
    typedef gdcm::Attribute<0x0040, 0xa731> RelationshipSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa732> RelationshipTypeCodeSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa744> LanguageCodeSequenceTrial;
    typedef gdcm::Attribute<0x0040, 0xa992> UniformResourceLocatorTrial;
    typedef gdcm::Attribute<0x0040, 0xb020> WaveformAnnotationSequence;
    typedef gdcm::Attribute<0x0040, 0xdb00> TemplateIdentifier;
    typedef gdcm::Attribute<0x0040, 0xdb06> TemplateVersion;
    typedef gdcm::Attribute<0x0040, 0xdb07> TemplateLocalVersion;
    typedef gdcm::Attribute<0x0040, 0xdb0b> TemplateExtensionFlag;
    typedef gdcm::Attribute<0x0040, 0xdb0c> TemplateExtensionOrganizationUID;
    typedef gdcm::Attribute<0x0040, 0xdb0d> TemplateExtensionCreatorUID;
    typedef gdcm::Attribute<0x0040, 0xdb73> ReferencedContentItemIdentifier;
    typedef gdcm::Attribute<0x0040, 0xe001> HL7InstanceIdentifier;
    typedef gdcm::Attribute<0x0040, 0xe004> HL7DocumentEffectiveTime;
    typedef gdcm::Attribute<0x0040, 0xe006> HL7DocumentTypeCodeSequence;
    typedef gdcm::Attribute<0x0040, 0xe008> DocumentClassCodeSequence;
    typedef gdcm::Attribute<0x0040, 0xe010> RetrieveURI;
    typedef gdcm::Attribute<0x0040, 0xe011> RetrieveLocationUID;
    typedef gdcm::Attribute<0x0040, 0xe020> TypeOfInstances;
    typedef gdcm::Attribute<0x0040, 0xe021> DICOMRetrievalSequence;
    typedef gdcm::Attribute<0x0040, 0xe022> DICOMMediaRetrievalSequence;
    typedef gdcm::Attribute<0x0040, 0xe023> WADORetrievalSequence;
    typedef gdcm::Attribute<0x0040, 0xe024> XDSRetrievalSequence;
    typedef gdcm::Attribute<0x0040, 0xe030> RepositoryUniqueID;
    typedef gdcm::Attribute<0x0040, 0xe031> HomeCommunityID;
    typedef gdcm::Attribute<0x0042, 0x0010> DocumentTitle;
    typedef gdcm::Attribute<0x0042, 0x0011> EncapsulatedDocument;
    typedef gdcm::Attribute<0x0042, 0x0012> MIMETypeOfEncapsulatedDocument;
    typedef gdcm::Attribute<0x0042, 0x0013> SourceInstanceSequence;
    typedef gdcm::Attribute<0x0042, 0x0014> ListOfMIMETypes;
    typedef gdcm::Attribute<0x0044, 0x0001> ProductPackageIdentifier;
    typedef gdcm::Attribute<0x0044, 0x0002> SubstanceAdministrationApproval;
    typedef gdcm::Attribute<0x0044, 0x0003> ApprovalStatusFurtherDescription;
    typedef gdcm::Attribute<0x0044, 0x0004> ApprovalStatusDateTime;
    typedef gdcm::Attribute<0x0044, 0x0007> ProductTypeCodeSequence;
    typedef gdcm::Attribute<0x0044, 0x0008> ProductName;
    typedef gdcm::Attribute<0x0044, 0x0009> ProductDescription;
    typedef gdcm::Attribute<0x0044, 0x000a> ProductLotIdentifier;
    typedef gdcm::Attribute<0x0044, 0x000b> ProductExpirationDateTime;
    typedef gdcm::Attribute<0x0044, 0x0010> SubstanceAdministrationDateTime;
    typedef gdcm::Attribute<0x0044, 0x0011> SubstanceAdministrationNotes;
    typedef gdcm::Attribute<0x0044, 0x0012> SubstanceAdministrationDeviceID;
    typedef gdcm::Attribute<0x0044, 0x0013> ProductParameterSequence;
    typedef gdcm::Attribute<0x0044, 0x0019> SubstanceAdministrationParameterSequence;
    typedef gdcm::Attribute<0x0046, 0x0012> LensDescription;
    typedef gdcm::Attribute<0x0046, 0x0014> RightLensSequence;
    typedef gdcm::Attribute<0x0046, 0x0015> LeftLensSequence;
    typedef gdcm::Attribute<0x0046, 0x0016> UnspecifiedLateralityLensSequence;
    typedef gdcm::Attribute<0x0046, 0x0018> CylinderSequence;
    typedef gdcm::Attribute<0x0046, 0x0028> PrismSequence;
    typedef gdcm::Attribute<0x0046, 0x0030> HorizontalPrismPower;
    typedef gdcm::Attribute<0x0046, 0x0032> HorizontalPrismBase;
    typedef gdcm::Attribute<0x0046, 0x0034> VerticalPrismPower;
    typedef gdcm::Attribute<0x0046, 0x0036> VerticalPrismBase;
    typedef gdcm::Attribute<0x0046, 0x0038> LensSegmentType;
    typedef gdcm::Attribute<0x0046, 0x0040> OpticalTransmittance;
    typedef gdcm::Attribute<0x0046, 0x0042> ChannelWidth;
    typedef gdcm::Attribute<0x0046, 0x0044> PupilSize;
    typedef gdcm::Attribute<0x0046, 0x0046> CornealSize;
    typedef gdcm::Attribute<0x0046, 0x0050> AutorefractionRightEyeSequence;
    typedef gdcm::Attribute<0x0046, 0x0052> AutorefractionLeftEyeSequence;
    typedef gdcm::Attribute<0x0046, 0x0060> DistancePupillaryDistance;
    typedef gdcm::Attribute<0x0046, 0x0062> NearPupillaryDistance;
    typedef gdcm::Attribute<0x0046, 0x0063> IntermediatePupillaryDistance;
    typedef gdcm::Attribute<0x0046, 0x0064> OtherPupillaryDistance;
    typedef gdcm::Attribute<0x0046, 0x0070> KeratometryRightEyeSequence;
    typedef gdcm::Attribute<0x0046, 0x0071> KeratometryLeftEyeSequence;
    typedef gdcm::Attribute<0x0046, 0x0074> SteepKeratometricAxisSequence;
    typedef gdcm::Attribute<0x0046, 0x0075> RadiusOfCurvature;
    typedef gdcm::Attribute<0x0046, 0x0076> KeratometricPower;
    typedef gdcm::Attribute<0x0046, 0x0077> KeratometricAxis;
    typedef gdcm::Attribute<0x0046, 0x0080> FlatKeratometricAxisSequence;
    typedef gdcm::Attribute<0x0046, 0x0092> BackgroundColor;
    typedef gdcm::Attribute<0x0046, 0x0094> Optotype;
    typedef gdcm::Attribute<0x0046, 0x0095> OptotypePresentation;
    typedef gdcm::Attribute<0x0046, 0x0097> SubjectiveRefractionRightEyeSequence;
    typedef gdcm::Attribute<0x0046, 0x0098> SubjectiveRefractionLeftEyeSequence;
    typedef gdcm::Attribute<0x0046, 0x0100> AddNearSequence;
    typedef gdcm::Attribute<0x0046, 0x0101> AddIntermediateSequence;
    typedef gdcm::Attribute<0x0046, 0x0102> AddOtherSequence;
    typedef gdcm::Attribute<0x0046, 0x0104> AddPower;
    typedef gdcm::Attribute<0x0046, 0x0106> ViewingDistance;
    typedef gdcm::Attribute<0x0046, 0x0121> VisualAcuityTypeCodeSequence;
    typedef gdcm::Attribute<0x0046, 0x0122> VisualAcuityRightEyeSequence;
    typedef gdcm::Attribute<0x0046, 0x0123> VisualAcuityLeftEyeSequence;
    typedef gdcm::Attribute<0x0046, 0x0124> VisualAcuityBothEyesOpenSequence;
    typedef gdcm::Attribute<0x0046, 0x0125> ViewingDistanceType;
    typedef gdcm::Attribute<0x0046, 0x0135> VisualAcuityModifiers;
    typedef gdcm::Attribute<0x0046, 0x0137> DecimalVisualAcuity;
    typedef gdcm::Attribute<0x0046, 0x0139> OptotypeDetailedDefinition;
    typedef gdcm::Attribute<0x0046, 0x0145> ReferencedRefractiveMeasurementsSequence;
    typedef gdcm::Attribute<0x0046, 0x0146> SpherePower;
    typedef gdcm::Attribute<0x0046, 0x0147> CylinderPower;
    typedef gdcm::Attribute<0x0048, 0x0001> ImagedVolumeWidth;
    typedef gdcm::Attribute<0x0048, 0x0002> ImagedVolumeHeight;
    typedef gdcm::Attribute<0x0048, 0x0003> ImagedVolumeDepth;
    typedef gdcm::Attribute<0x0048, 0x0006> TotalPixelMatrixColumns;
    typedef gdcm::Attribute<0x0048, 0x0007> TotalPixelMatrixRows;
    typedef gdcm::Attribute<0x0048, 0x0008> TotalPixelMatrixOriginSequence;
    typedef gdcm::Attribute<0x0048, 0x0010> SpecimenLabelInImage;
    typedef gdcm::Attribute<0x0048, 0x0011> FocusMethod;
    typedef gdcm::Attribute<0x0048, 0x0012> ExtendedDepthOfField;
    typedef gdcm::Attribute<0x0048, 0x0013> NumberOfFocalPlanes;
    typedef gdcm::Attribute<0x0048, 0x0014> DistanceBetweenFocalPlanes;
    typedef gdcm::Attribute<0x0048, 0x0015> RecommendedAbsentPixelCIELabValue;
    typedef gdcm::Attribute<0x0048, 0x0100> IlluminatorTypeCodeSequence;
    typedef gdcm::Attribute<0x0048, 0x0102> ImageOrientationSlide;
    typedef gdcm::Attribute<0x0048, 0x0105> OpticalPathSequence;
    typedef gdcm::Attribute<0x0048, 0x0106> OpticalPathIdentifier;
    typedef gdcm::Attribute<0x0048, 0x0107> OpticalPathDescription;
    typedef gdcm::Attribute<0x0048, 0x0108> IlluminationColorCodeSequence;
    typedef gdcm::Attribute<0x0048, 0x0110> SpecimenReferenceSequence;
    typedef gdcm::Attribute<0x0048, 0x0111> CondenserLensPower;
    typedef gdcm::Attribute<0x0048, 0x0112> ObjectiveLensPower;
    typedef gdcm::Attribute<0x0048, 0x0113> ObjectiveLensNumericalAperture;
    typedef gdcm::Attribute<0x0048, 0x0120> PaletteColorLookupTableSequence;
    typedef gdcm::Attribute<0x0048, 0x0200> ReferencedImageNavigationSequence;
    typedef gdcm::Attribute<0x0048, 0x0201> TopLeftHandCornerOfLocalizerArea;
    typedef gdcm::Attribute<0x0048, 0x0202> BottomRightHandCornerOfLocalizerArea;
    typedef gdcm::Attribute<0x0048, 0x0207> OpticalPathIdentificationSequence;
    typedef gdcm::Attribute<0x0048, 0x021a> PlanePositionSlideSequence;
    typedef gdcm::Attribute<0x0048, 0x021e> ColumnPositionInTotalImagePixelMatrix;
    typedef gdcm::Attribute<0x0048, 0x021f> RowPositionInTotalImagePixelMatrix;
    typedef gdcm::Attribute<0x0048, 0x0301> PixelOriginInterpretation;
    typedef gdcm::Attribute<0x0050, 0x0004> CalibrationImage;
    typedef gdcm::Attribute<0x0050, 0x0010> DeviceSequence;
    typedef gdcm::Attribute<0x0050, 0x0012> ContainerComponentTypeCodeSequence;
    typedef gdcm::Attribute<0x0050, 0x0013> ContainerComponentThickness;
    typedef gdcm::Attribute<0x0050, 0x0014> DeviceLength;
    typedef gdcm::Attribute<0x0050, 0x0015> ContainerComponentWidth;
    typedef gdcm::Attribute<0x0050, 0x0016> DeviceDiameter;
    typedef gdcm::Attribute<0x0050, 0x0017> DeviceDiameterUnits;
    typedef gdcm::Attribute<0x0050, 0x0018> DeviceVolume;
    typedef gdcm::Attribute<0x0050, 0x0019> InterMarkerDistance;
    typedef gdcm::Attribute<0x0050, 0x001a> ContainerComponentMaterial;
    typedef gdcm::Attribute<0x0050, 0x001b> ContainerComponentID;
    typedef gdcm::Attribute<0x0050, 0x001c> ContainerComponentLength;
    typedef gdcm::Attribute<0x0050, 0x001d> ContainerComponentDiameter;
    typedef gdcm::Attribute<0x0050, 0x001e> ContainerComponentDescription;
    typedef gdcm::Attribute<0x0050, 0x0020> DeviceDescription;
    typedef gdcm::Attribute<0x0052, 0x0001> ContrastBolusIngredientPercentByVolume;
    typedef gdcm::Attribute<0x0052, 0x0002> OCTFocalDistance;
    typedef gdcm::Attribute<0x0052, 0x0003> BeamSpotSize;
    typedef gdcm::Attribute<0x0052, 0x0004> EffectiveRefractiveIndex;
    typedef gdcm::Attribute<0x0052, 0x0006> OCTAcquisitionDomain;
    typedef gdcm::Attribute<0x0052, 0x0007> OCTOpticalCenterWavelength;
    typedef gdcm::Attribute<0x0052, 0x0008> AxialResolution;
    typedef gdcm::Attribute<0x0052, 0x0009> RangingDepth;
    typedef gdcm::Attribute<0x0052, 0x0011> ALineRate;
    typedef gdcm::Attribute<0x0052, 0x0012> ALinesPerFrame;
    typedef gdcm::Attribute<0x0052, 0x0013> CatheterRotationalRate;
    typedef gdcm::Attribute<0x0052, 0x0014> ALinePixelSpacing;
    typedef gdcm::Attribute<0x0052, 0x0016> ModeOfPercutaneousAccessSequence;
    typedef gdcm::Attribute<0x0052, 0x0025> IntravascularOCTFrameTypeSequence;
    typedef gdcm::Attribute<0x0052, 0x0026> OCTZOffsetApplied;
    typedef gdcm::Attribute<0x0052, 0x0027> IntravascularFrameContentSequence;
    typedef gdcm::Attribute<0x0052, 0x0028> IntravascularLongitudinalDistance;
    typedef gdcm::Attribute<0x0052, 0x0029> IntravascularOCTFrameContentSequence;
    typedef gdcm::Attribute<0x0052, 0x0030> OCTZOffsetCorrection;
    typedef gdcm::Attribute<0x0052, 0x0031> CatheterDirectionOfRotation;
    typedef gdcm::Attribute<0x0052, 0x0033> SeamLineLocation;
    typedef gdcm::Attribute<0x0052, 0x0034> FirstALineLocation;
    typedef gdcm::Attribute<0x0052, 0x0036> SeamLineIndex;
    typedef gdcm::Attribute<0x0052, 0x0038> NumberOfPaddedAlines;
    typedef gdcm::Attribute<0x0052, 0x0039> InterpolationType;
    typedef gdcm::Attribute<0x0052, 0x003a> RefractiveIndexApplied;
    typedef gdcm::Attribute<0x0054, 0x0010> EnergyWindowVector;
    typedef gdcm::Attribute<0x0054, 0x0011> NumberOfEnergyWindows;
    typedef gdcm::Attribute<0x0054, 0x0012> EnergyWindowInformationSequence;
    typedef gdcm::Attribute<0x0054, 0x0013> EnergyWindowRangeSequence;
    typedef gdcm::Attribute<0x0054, 0x0014> EnergyWindowLowerLimit;
    typedef gdcm::Attribute<0x0054, 0x0015> EnergyWindowUpperLimit;
    typedef gdcm::Attribute<0x0054, 0x0016> RadiopharmaceuticalInformationSequence;
    typedef gdcm::Attribute<0x0054, 0x0017> ResidualSyringeCounts;
    typedef gdcm::Attribute<0x0054, 0x0018> EnergyWindowName;
    typedef gdcm::Attribute<0x0054, 0x0020> DetectorVector;
    typedef gdcm::Attribute<0x0054, 0x0021> NumberOfDetectors;
    typedef gdcm::Attribute<0x0054, 0x0022> DetectorInformationSequence;
    typedef gdcm::Attribute<0x0054, 0x0030> PhaseVector;
    typedef gdcm::Attribute<0x0054, 0x0031> NumberOfPhases;
    typedef gdcm::Attribute<0x0054, 0x0032> PhaseInformationSequence;
    typedef gdcm::Attribute<0x0054, 0x0033> NumberOfFramesInPhase;
    typedef gdcm::Attribute<0x0054, 0x0036> PhaseDelay;
    typedef gdcm::Attribute<0x0054, 0x0038> PauseBetweenFrames;
    typedef gdcm::Attribute<0x0054, 0x0039> PhaseDescription;
    typedef gdcm::Attribute<0x0054, 0x0050> RotationVector;
    typedef gdcm::Attribute<0x0054, 0x0051> NumberOfRotations;
    typedef gdcm::Attribute<0x0054, 0x0052> RotationInformationSequence;
    typedef gdcm::Attribute<0x0054, 0x0053> NumberOfFramesInRotation;
    typedef gdcm::Attribute<0x0054, 0x0060> RRIntervalVector;
    typedef gdcm::Attribute<0x0054, 0x0061> NumberOfRRIntervals;
    typedef gdcm::Attribute<0x0054, 0x0062> GatedInformationSequence;
    typedef gdcm::Attribute<0x0054, 0x0063> DataInformationSequence;
    typedef gdcm::Attribute<0x0054, 0x0070> TimeSlotVector;
    typedef gdcm::Attribute<0x0054, 0x0071> NumberOfTimeSlots;
    typedef gdcm::Attribute<0x0054, 0x0072> TimeSlotInformationSequence;
    typedef gdcm::Attribute<0x0054, 0x0073> TimeSlotTime;
    typedef gdcm::Attribute<0x0054, 0x0080> SliceVector;
    typedef gdcm::Attribute<0x0054, 0x0081> NumberOfSlices;
    typedef gdcm::Attribute<0x0054, 0x0090> AngularViewVector;
    typedef gdcm::Attribute<0x0054, 0x0100> TimeSliceVector;
    typedef gdcm::Attribute<0x0054, 0x0101> NumberOfTimeSlices;
    typedef gdcm::Attribute<0x0054, 0x0200> StartAngle;
    typedef gdcm::Attribute<0x0054, 0x0202> TypeOfDetectorMotion;
    typedef gdcm::Attribute<0x0054, 0x0210> TriggerVector;
    typedef gdcm::Attribute<0x0054, 0x0211> NumberOfTriggersInPhase;
    typedef gdcm::Attribute<0x0054, 0x0220> ViewCodeSequence;
    typedef gdcm::Attribute<0x0054, 0x0222> ViewModifierCodeSequence;
    typedef gdcm::Attribute<0x0054, 0x0300> RadionuclideCodeSequence;
    typedef gdcm::Attribute<0x0054, 0x0302> AdministrationRouteCodeSequence;
    typedef gdcm::Attribute<0x0054, 0x0304> RadiopharmaceuticalCodeSequence;
    typedef gdcm::Attribute<0x0054, 0x0306> CalibrationDataSequence;
    typedef gdcm::Attribute<0x0054, 0x0308> EnergyWindowNumber;
    typedef gdcm::Attribute<0x0054, 0x0400> ImageID;
    typedef gdcm::Attribute<0x0054, 0x0410> PatientOrientationCodeSequence;
    typedef gdcm::Attribute<0x0054, 0x0412> PatientOrientationModifierCodeSequence;
    typedef gdcm::Attribute<0x0054, 0x0414> PatientGantryRelationshipCodeSequence;
    typedef gdcm::Attribute<0x0054, 0x0500> SliceProgressionDirection;
    typedef gdcm::Attribute<0x0054, 0x1000> SeriesType;
    typedef gdcm::Attribute<0x0054, 0x1001> Units;
    typedef gdcm::Attribute<0x0054, 0x1002> CountsSource;
    typedef gdcm::Attribute<0x0054, 0x1004> ReprojectionMethod;
    typedef gdcm::Attribute<0x0054, 0x1006> SUVType;
    typedef gdcm::Attribute<0x0054, 0x1100> RandomsCorrectionMethod;
    typedef gdcm::Attribute<0x0054, 0x1101> AttenuationCorrectionMethod;
    typedef gdcm::Attribute<0x0054, 0x1102> DecayCorrection;
    typedef gdcm::Attribute<0x0054, 0x1103> ReconstructionMethod;
    typedef gdcm::Attribute<0x0054, 0x1104> DetectorLinesOfResponseUsed;
    typedef gdcm::Attribute<0x0054, 0x1105> ScatterCorrectionMethod;
    typedef gdcm::Attribute<0x0054, 0x1200> AxialAcceptance;
    typedef gdcm::Attribute<0x0054, 0x1201> AxialMash;
    typedef gdcm::Attribute<0x0054, 0x1202> TransverseMash;
    typedef gdcm::Attribute<0x0054, 0x1203> DetectorElementSize;
    typedef gdcm::Attribute<0x0054, 0x1210> CoincidenceWindowWidth;
    typedef gdcm::Attribute<0x0054, 0x1220> SecondaryCountsType;
    typedef gdcm::Attribute<0x0054, 0x1300> FrameReferenceTime;
    typedef gdcm::Attribute<0x0054, 0x1310> PrimaryPromptsCountsAccumulated;
    typedef gdcm::Attribute<0x0054, 0x1311> SecondaryCountsAccumulated;
    typedef gdcm::Attribute<0x0054, 0x1320> SliceSensitivityFactor;
    typedef gdcm::Attribute<0x0054, 0x1321> DecayFactor;
    typedef gdcm::Attribute<0x0054, 0x1322> DoseCalibrationFactor;
    typedef gdcm::Attribute<0x0054, 0x1323> ScatterFractionFactor;
    typedef gdcm::Attribute<0x0054, 0x1324> DeadTimeFactor;
    typedef gdcm::Attribute<0x0054, 0x1330> ImageIndex;
    typedef gdcm::Attribute<0x0054, 0x1400> CountsIncluded;
    typedef gdcm::Attribute<0x0054, 0x1401> DeadTimeCorrectionFlag;
    typedef gdcm::Attribute<0x0060, 0x3000> HistogramSequence;
    typedef gdcm::Attribute<0x0060, 0x3002> HistogramNumberOfBins;
    typedef gdcm::Attribute<0x0060, 0x3004> HistogramFirstBinValue;
    typedef gdcm::Attribute<0x0060, 0x3006> HistogramLastBinValue;
    typedef gdcm::Attribute<0x0060, 0x3008> HistogramBinWidth;
    typedef gdcm::Attribute<0x0060, 0x3010> HistogramExplanation;
    typedef gdcm::Attribute<0x0060, 0x3020> HistogramData;
    typedef gdcm::Attribute<0x0062, 0x0001> SegmentationType;
    typedef gdcm::Attribute<0x0062, 0x0002> SegmentSequence;
    typedef gdcm::Attribute<0x0062, 0x0003> SegmentedPropertyCategoryCodeSequence;
    typedef gdcm::Attribute<0x0062, 0x0004> SegmentNumber;
    typedef gdcm::Attribute<0x0062, 0x0005> SegmentLabel;
    typedef gdcm::Attribute<0x0062, 0x0006> SegmentDescription;
    typedef gdcm::Attribute<0x0062, 0x0008> SegmentAlgorithmType;
    typedef gdcm::Attribute<0x0062, 0x0009> SegmentAlgorithmName;
    typedef gdcm::Attribute<0x0062, 0x000a> SegmentIdentificationSequence;
    typedef gdcm::Attribute<0x0062, 0x000b> ReferencedSegmentNumber;
    typedef gdcm::Attribute<0x0062, 0x000c> RecommendedDisplayGrayscaleValue;
    typedef gdcm::Attribute<0x0062, 0x000d> RecommendedDisplayCIELabValue;
    typedef gdcm::Attribute<0x0062, 0x000e> MaximumFractionalValue;
    typedef gdcm::Attribute<0x0062, 0x000f> SegmentedPropertyTypeCodeSequence;
    typedef gdcm::Attribute<0x0062, 0x0010> SegmentationFractionalType;
    typedef gdcm::Attribute<0x0064, 0x0002> DeformableRegistrationSequence;
    typedef gdcm::Attribute<0x0064, 0x0003> SourceFrameOfReferenceUID;
    typedef gdcm::Attribute<0x0064, 0x0005> DeformableRegistrationGridSequence;
    typedef gdcm::Attribute<0x0064, 0x0007> GridDimensions;
    typedef gdcm::Attribute<0x0064, 0x0008> GridResolution;
    typedef gdcm::Attribute<0x0064, 0x0009> VectorGridData;
    typedef gdcm::Attribute<0x0064, 0x000f> PreDeformationMatrixRegistrationSequence;
    typedef gdcm::Attribute<0x0064, 0x0010> PostDeformationMatrixRegistrationSequence;
    typedef gdcm::Attribute<0x0066, 0x0001> NumberOfSurfaces;
    typedef gdcm::Attribute<0x0066, 0x0002> SurfaceSequence;
    typedef gdcm::Attribute<0x0066, 0x0003> SurfaceNumber;
    typedef gdcm::Attribute<0x0066, 0x0004> SurfaceComments;
    typedef gdcm::Attribute<0x0066, 0x0009> SurfaceProcessing;
    typedef gdcm::Attribute<0x0066, 0x000a> SurfaceProcessingRatio;
    typedef gdcm::Attribute<0x0066, 0x000b> SurfaceProcessingDescription;
    typedef gdcm::Attribute<0x0066, 0x000c> RecommendedPresentationOpacity;
    typedef gdcm::Attribute<0x0066, 0x000d> RecommendedPresentationType;
    typedef gdcm::Attribute<0x0066, 0x000e> FiniteVolume;
    typedef gdcm::Attribute<0x0066, 0x0010> Manifold;
    typedef gdcm::Attribute<0x0066, 0x0011> SurfacePointsSequence;
    typedef gdcm::Attribute<0x0066, 0x0012> SurfacePointsNormalsSequence;
    typedef gdcm::Attribute<0x0066, 0x0013> SurfaceMeshPrimitivesSequence;
    typedef gdcm::Attribute<0x0066, 0x0015> NumberOfSurfacePoints;
    typedef gdcm::Attribute<0x0066, 0x0016> PointCoordinatesData;
    typedef gdcm::Attribute<0x0066, 0x0017> PointPositionAccuracy;
    typedef gdcm::Attribute<0x0066, 0x0018> MeanPointDistance;
    typedef gdcm::Attribute<0x0066, 0x0019> MaximumPointDistance;
    typedef gdcm::Attribute<0x0066, 0x001a> PointsBoundingBoxCoordinates;
    typedef gdcm::Attribute<0x0066, 0x001b> AxisOfRotation;
    typedef gdcm::Attribute<0x0066, 0x001c> CenterOfRotation;
    typedef gdcm::Attribute<0x0066, 0x001e> NumberOfVectors;
    typedef gdcm::Attribute<0x0066, 0x001f> VectorDimensionality;
    typedef gdcm::Attribute<0x0066, 0x0020> VectorAccuracy;
    typedef gdcm::Attribute<0x0066, 0x0021> VectorCoordinateData;
    typedef gdcm::Attribute<0x0066, 0x0023> TrianglePointIndexList;
    typedef gdcm::Attribute<0x0066, 0x0024> EdgePointIndexList;
    typedef gdcm::Attribute<0x0066, 0x0025> VertexPointIndexList;
    typedef gdcm::Attribute<0x0066, 0x0026> TriangleStripSequence;
    typedef gdcm::Attribute<0x0066, 0x0027> TriangleFanSequence;
    typedef gdcm::Attribute<0x0066, 0x0028> LineSequence;
    typedef gdcm::Attribute<0x0066, 0x0029> PrimitivePointIndexList;
    typedef gdcm::Attribute<0x0066, 0x002a> SurfaceCount;
    typedef gdcm::Attribute<0x0066, 0x002b> ReferencedSurfaceSequence;
    typedef gdcm::Attribute<0x0066, 0x002c> ReferencedSurfaceNumber;
    typedef gdcm::Attribute<0x0066, 0x002d> SegmentSurfaceGenerationAlgorithmIdentificationSequence;
    typedef gdcm::Attribute<0x0066, 0x002e> SegmentSurfaceSourceInstanceSequence;
    typedef gdcm::Attribute<0x0066, 0x002f> AlgorithmFamilyCodeSequence;
    typedef gdcm::Attribute<0x0066, 0x0030> AlgorithmNameCodeSequence;
    typedef gdcm::Attribute<0x0066, 0x0031> AlgorithmVersion;
    typedef gdcm::Attribute<0x0066, 0x0032> AlgorithmParameters;
    typedef gdcm::Attribute<0x0066, 0x0034> FacetSequence;
    typedef gdcm::Attribute<0x0066, 0x0035> SurfaceProcessingAlgorithmIdentificationSequence;
    typedef gdcm::Attribute<0x0066, 0x0036> AlgorithmName;
    typedef gdcm::Attribute<0x0068, 0x6210> ImplantSize;
    typedef gdcm::Attribute<0x0068, 0x6221> ImplantTemplateVersion;
    typedef gdcm::Attribute<0x0068, 0x6222> ReplacedImplantTemplateSequence;
    typedef gdcm::Attribute<0x0068, 0x6223> ImplantType;
    typedef gdcm::Attribute<0x0068, 0x6224> DerivationImplantTemplateSequence;
    typedef gdcm::Attribute<0x0068, 0x6225> OriginalImplantTemplateSequence;
    typedef gdcm::Attribute<0x0068, 0x6226> EffectiveDateTime;
    typedef gdcm::Attribute<0x0068, 0x6230> ImplantTargetAnatomySequence;
    typedef gdcm::Attribute<0x0068, 0x6260> InformationFromManufacturerSequence;
    typedef gdcm::Attribute<0x0068, 0x6265> NotificationFromManufacturerSequence;
    typedef gdcm::Attribute<0x0068, 0x6270> InformationIssueDateTime;
    typedef gdcm::Attribute<0x0068, 0x6280> InformationSummary;
    typedef gdcm::Attribute<0x0068, 0x62a0> ImplantRegulatoryDisapprovalCodeSequence;
    typedef gdcm::Attribute<0x0068, 0x62a5> OverallTemplateSpatialTolerance;
    typedef gdcm::Attribute<0x0068, 0x62c0> HPGLDocumentSequence;
    typedef gdcm::Attribute<0x0068, 0x62d0> HPGLDocumentID;
    typedef gdcm::Attribute<0x0068, 0x62d5> HPGLDocumentLabel;
    typedef gdcm::Attribute<0x0068, 0x62e0> ViewOrientationCodeSequence;
    typedef gdcm::Attribute<0x0068, 0x62f0> ViewOrientationModifier;
    typedef gdcm::Attribute<0x0068, 0x62f2> HPGLDocumentScaling;
    typedef gdcm::Attribute<0x0068, 0x6300> HPGLDocument;
    typedef gdcm::Attribute<0x0068, 0x6310> HPGLContourPenNumber;
    typedef gdcm::Attribute<0x0068, 0x6320> HPGLPenSequence;
    typedef gdcm::Attribute<0x0068, 0x6330> HPGLPenNumber;
    typedef gdcm::Attribute<0x0068, 0x6340> HPGLPenLabel;
    typedef gdcm::Attribute<0x0068, 0x6345> HPGLPenDescription;
    typedef gdcm::Attribute<0x0068, 0x6346> RecommendedRotationPoint;
    typedef gdcm::Attribute<0x0068, 0x6347> BoundingRectangle;
    typedef gdcm::Attribute<0x0068, 0x6350> ImplantTemplate3DModelSurfaceNumber;
    typedef gdcm::Attribute<0x0068, 0x6360> SurfaceModelDescriptionSequence;
    typedef gdcm::Attribute<0x0068, 0x6380> SurfaceModelLabel;
    typedef gdcm::Attribute<0x0068, 0x6390> SurfaceModelScalingFactor;
    typedef gdcm::Attribute<0x0068, 0x63a0> MaterialsCodeSequence;
    typedef gdcm::Attribute<0x0068, 0x63a4> CoatingMaterialsCodeSequence;
    typedef gdcm::Attribute<0x0068, 0x63a8> ImplantTypeCodeSequence;
    typedef gdcm::Attribute<0x0068, 0x63ac> FixationMethodCodeSequence;
    typedef gdcm::Attribute<0x0068, 0x63b0> MatingFeatureSetsSequence;
    typedef gdcm::Attribute<0x0068, 0x63c0> MatingFeatureSetID;
    typedef gdcm::Attribute<0x0068, 0x63d0> MatingFeatureSetLabel;
    typedef gdcm::Attribute<0x0068, 0x63e0> MatingFeatureSequence;
    typedef gdcm::Attribute<0x0068, 0x63f0> MatingFeatureID;
    typedef gdcm::Attribute<0x0068, 0x6400> MatingFeatureDegreeOfFreedomSequence;
    typedef gdcm::Attribute<0x0068, 0x6410> DegreeOfFreedomID;
    typedef gdcm::Attribute<0x0068, 0x6420> DegreeOfFreedomType;
    typedef gdcm::Attribute<0x0068, 0x6430> TwoDMatingFeatureCoordinatesSequence;
    typedef gdcm::Attribute<0x0068, 0x6440> ReferencedHPGLDocumentID;
    typedef gdcm::Attribute<0x0068, 0x6450> TwoDMatingPoint;
    typedef gdcm::Attribute<0x0068, 0x6460> TwoDMatingAxes;
    typedef gdcm::Attribute<0x0068, 0x6470> TwoDDegreeOfFreedomSequence;
    typedef gdcm::Attribute<0x0068, 0x6490> ThreeDDegreeOfFreedomAxis;
    typedef gdcm::Attribute<0x0068, 0x64a0> RangeOfFreedom;
    typedef gdcm::Attribute<0x0068, 0x64c0> ThreeDMatingPoint;
    typedef gdcm::Attribute<0x0068, 0x64d0> ThreeDMatingAxes;
    typedef gdcm::Attribute<0x0068, 0x64f0> TwoDDegreeOfFreedomAxis;
    typedef gdcm::Attribute<0x0068, 0x6500> PlanningLandmarkPointSequence;
    typedef gdcm::Attribute<0x0068, 0x6510> PlanningLandmarkLineSequence;
    typedef gdcm::Attribute<0x0068, 0x6520> PlanningLandmarkPlaneSequence;
    typedef gdcm::Attribute<0x0068, 0x6530> PlanningLandmarkID;
    typedef gdcm::Attribute<0x0068, 0x6540> PlanningLandmarkDescription;
    typedef gdcm::Attribute<0x0068, 0x6545> PlanningLandmarkIdentificationCodeSequence;
    typedef gdcm::Attribute<0x0068, 0x6550> TwoDPointCoordinatesSequence;
    typedef gdcm::Attribute<0x0068, 0x6560> TwoDPointCoordinates;
    typedef gdcm::Attribute<0x0068, 0x6590> ThreeDPointCoordinates;
    typedef gdcm::Attribute<0x0068, 0x65a0> TwoDLineCoordinatesSequence;
    typedef gdcm::Attribute<0x0068, 0x65b0> TwoDLineCoordinates;
    typedef gdcm::Attribute<0x0068, 0x65d0> ThreeDLineCoordinates;
    typedef gdcm::Attribute<0x0068, 0x65e0> TwoDPlaneCoordinatesSequence;
    typedef gdcm::Attribute<0x0068, 0x65f0> TwoDPlaneIntersection;
    typedef gdcm::Attribute<0x0068, 0x6610> ThreeDPlaneOrigin;
    typedef gdcm::Attribute<0x0068, 0x6620> ThreeDPlaneNormal;
    typedef gdcm::Attribute<0x0070, 0x0001> GraphicAnnotationSequence;
    typedef gdcm::Attribute<0x0070, 0x0002> GraphicLayer;
    typedef gdcm::Attribute<0x0070, 0x0003> BoundingBoxAnnotationUnits;
    typedef gdcm::Attribute<0x0070, 0x0004> AnchorPointAnnotationUnits;
    typedef gdcm::Attribute<0x0070, 0x0005> GraphicAnnotationUnits;
    typedef gdcm::Attribute<0x0070, 0x0006> UnformattedTextValue;
    typedef gdcm::Attribute<0x0070, 0x0008> TextObjectSequence;
    typedef gdcm::Attribute<0x0070, 0x0009> GraphicObjectSequence;
    typedef gdcm::Attribute<0x0070, 0x0010> BoundingBoxTopLeftHandCorner;
    typedef gdcm::Attribute<0x0070, 0x0011> BoundingBoxBottomRightHandCorner;
    typedef gdcm::Attribute<0x0070, 0x0012> BoundingBoxTextHorizontalJustification;
    typedef gdcm::Attribute<0x0070, 0x0014> AnchorPoint;
    typedef gdcm::Attribute<0x0070, 0x0015> AnchorPointVisibility;
    typedef gdcm::Attribute<0x0070, 0x0020> GraphicDimensions;
    typedef gdcm::Attribute<0x0070, 0x0021> NumberOfGraphicPoints;
    typedef gdcm::Attribute<0x0070, 0x0022> GraphicData;
    typedef gdcm::Attribute<0x0070, 0x0023> GraphicType;
    typedef gdcm::Attribute<0x0070, 0x0024> GraphicFilled;
    typedef gdcm::Attribute<0x0070, 0x0040> ImageRotationRetired;
    typedef gdcm::Attribute<0x0070, 0x0041> ImageHorizontalFlip;
    typedef gdcm::Attribute<0x0070, 0x0042> ImageRotation;
    typedef gdcm::Attribute<0x0070, 0x0050> DisplayedAreaTopLeftHandCornerTrial;
    typedef gdcm::Attribute<0x0070, 0x0051> DisplayedAreaBottomRightHandCornerTrial;
    typedef gdcm::Attribute<0x0070, 0x0052> DisplayedAreaTopLeftHandCorner;
    typedef gdcm::Attribute<0x0070, 0x0053> DisplayedAreaBottomRightHandCorner;
    typedef gdcm::Attribute<0x0070, 0x005a> DisplayedAreaSelectionSequence;
    typedef gdcm::Attribute<0x0070, 0x0060> GraphicLayerSequence;
    typedef gdcm::Attribute<0x0070, 0x0062> GraphicLayerOrder;
    typedef gdcm::Attribute<0x0070, 0x0066> GraphicLayerRecommendedDisplayGrayscaleValue;
    typedef gdcm::Attribute<0x0070, 0x0067> GraphicLayerRecommendedDisplayRGBValue;
    typedef gdcm::Attribute<0x0070, 0x0068> GraphicLayerDescription;
    typedef gdcm::Attribute<0x0070, 0x0080> ContentLabel;
    typedef gdcm::Attribute<0x0070, 0x0081> ContentDescription;
    typedef gdcm::Attribute<0x0070, 0x0082> PresentationCreationDate;
    typedef gdcm::Attribute<0x0070, 0x0083> PresentationCreationTime;
    typedef gdcm::Attribute<0x0070, 0x0084> ContentCreatorName;
    typedef gdcm::Attribute<0x0070, 0x0086> ContentCreatorIdentificationCodeSequence;
    typedef gdcm::Attribute<0x0070, 0x0087> AlternateContentDescriptionSequence;
    typedef gdcm::Attribute<0x0070, 0x0100> PresentationSizeMode;
    typedef gdcm::Attribute<0x0070, 0x0101> PresentationPixelSpacing;
    typedef gdcm::Attribute<0x0070, 0x0102> PresentationPixelAspectRatio;
    typedef gdcm::Attribute<0x0070, 0x0103> PresentationPixelMagnificationRatio;
    typedef gdcm::Attribute<0x0070, 0x0207> GraphicGroupLabel;
    typedef gdcm::Attribute<0x0070, 0x0208> GraphicGroupDescription;
    typedef gdcm::Attribute<0x0070, 0x0209> CompoundGraphicSequence;
    typedef gdcm::Attribute<0x0070, 0x0226> CompoundGraphicInstanceID;
    typedef gdcm::Attribute<0x0070, 0x0227> FontName;
    typedef gdcm::Attribute<0x0070, 0x0228> FontNameType;
    typedef gdcm::Attribute<0x0070, 0x0229> CSSFontName;
    typedef gdcm::Attribute<0x0070, 0x0230> RotationAngle;
    typedef gdcm::Attribute<0x0070, 0x0231> TextStyleSequence;
    typedef gdcm::Attribute<0x0070, 0x0232> LineStyleSequence;
    typedef gdcm::Attribute<0x0070, 0x0233> FillStyleSequence;
    typedef gdcm::Attribute<0x0070, 0x0234> GraphicGroupSequence;
    typedef gdcm::Attribute<0x0070, 0x0241> TextColorCIELabValue;
    typedef gdcm::Attribute<0x0070, 0x0242> HorizontalAlignment;
    typedef gdcm::Attribute<0x0070, 0x0243> VerticalAlignment;
    typedef gdcm::Attribute<0x0070, 0x0244> ShadowStyle;
    typedef gdcm::Attribute<0x0070, 0x0245> ShadowOffsetX;
    typedef gdcm::Attribute<0x0070, 0x0246> ShadowOffsetY;
    typedef gdcm::Attribute<0x0070, 0x0247> ShadowColorCIELabValue;
    typedef gdcm::Attribute<0x0070, 0x0248> Underlined;
    typedef gdcm::Attribute<0x0070, 0x0249> Bold;
    typedef gdcm::Attribute<0x0070, 0x0250> Italic;
    typedef gdcm::Attribute<0x0070, 0x0251> PatternOnColorCIELabValue;
    typedef gdcm::Attribute<0x0070, 0x0252> PatternOffColorCIELabValue;
    typedef gdcm::Attribute<0x0070, 0x0253> LineThickness;
    typedef gdcm::Attribute<0x0070, 0x0254> LineDashingStyle;
    typedef gdcm::Attribute<0x0070, 0x0255> LinePattern;
    typedef gdcm::Attribute<0x0070, 0x0256> FillPattern;
    typedef gdcm::Attribute<0x0070, 0x0257> FillMode;
    typedef gdcm::Attribute<0x0070, 0x0258> ShadowOpacity;
    typedef gdcm::Attribute<0x0070, 0x0261> GapLength;
    typedef gdcm::Attribute<0x0070, 0x0262> DiameterOfVisibility;
    typedef gdcm::Attribute<0x0070, 0x0273> RotationPoint;
    typedef gdcm::Attribute<0x0070, 0x0274> TickAlignment;
    typedef gdcm::Attribute<0x0070, 0x0278> ShowTickLabel;
    typedef gdcm::Attribute<0x0070, 0x0279> TickLabelAlignment;
    typedef gdcm::Attribute<0x0070, 0x0282> CompoundGraphicUnits;
    typedef gdcm::Attribute<0x0070, 0x0284> PatternOnOpacity;
    typedef gdcm::Attribute<0x0070, 0x0285> PatternOffOpacity;
    typedef gdcm::Attribute<0x0070, 0x0287> MajorTicksSequence;
    typedef gdcm::Attribute<0x0070, 0x0288> TickPosition;
    typedef gdcm::Attribute<0x0070, 0x0289> TickLabel;
    typedef gdcm::Attribute<0x0070, 0x0294> CompoundGraphicType;
    typedef gdcm::Attribute<0x0070, 0x0295> GraphicGroupID;
    typedef gdcm::Attribute<0x0070, 0x0306> ShapeType;
    typedef gdcm::Attribute<0x0070, 0x0308> RegistrationSequence;
    typedef gdcm::Attribute<0x0070, 0x0309> MatrixRegistrationSequence;
    typedef gdcm::Attribute<0x0070, 0x030a> MatrixSequence;
    typedef gdcm::Attribute<0x0070, 0x030c> FrameOfReferenceTransformationMatrixType;
    typedef gdcm::Attribute<0x0070, 0x030d> RegistrationTypeCodeSequence;
    typedef gdcm::Attribute<0x0070, 0x030f> FiducialDescription;
    typedef gdcm::Attribute<0x0070, 0x0310> FiducialIdentifier;
    typedef gdcm::Attribute<0x0070, 0x0311> FiducialIdentifierCodeSequence;
    typedef gdcm::Attribute<0x0070, 0x0312> ContourUncertaintyRadius;
    typedef gdcm::Attribute<0x0070, 0x0314> UsedFiducialsSequence;
    typedef gdcm::Attribute<0x0070, 0x0318> GraphicCoordinatesDataSequence;
    typedef gdcm::Attribute<0x0070, 0x031a> FiducialUID;
    typedef gdcm::Attribute<0x0070, 0x031c> FiducialSetSequence;
    typedef gdcm::Attribute<0x0070, 0x031e> FiducialSequence;
    typedef gdcm::Attribute<0x0070, 0x0401> GraphicLayerRecommendedDisplayCIELabValue;
    typedef gdcm::Attribute<0x0070, 0x0402> BlendingSequence;
    typedef gdcm::Attribute<0x0070, 0x0403> RelativeOpacity;
    typedef gdcm::Attribute<0x0070, 0x0404> ReferencedSpatialRegistrationSequence;
    typedef gdcm::Attribute<0x0070, 0x0405> BlendingPosition;
    typedef gdcm::Attribute<0x0072, 0x0002> HangingProtocolName;
    typedef gdcm::Attribute<0x0072, 0x0004> HangingProtocolDescription;
    typedef gdcm::Attribute<0x0072, 0x0006> HangingProtocolLevel;
    typedef gdcm::Attribute<0x0072, 0x0008> HangingProtocolCreator;
    typedef gdcm::Attribute<0x0072, 0x000a> HangingProtocolCreationDateTime;
    typedef gdcm::Attribute<0x0072, 0x000c> HangingProtocolDefinitionSequence;
    typedef gdcm::Attribute<0x0072, 0x000e> HangingProtocolUserIdentificationCodeSequence;
    typedef gdcm::Attribute<0x0072, 0x0010> HangingProtocolUserGroupName;
    typedef gdcm::Attribute<0x0072, 0x0012> SourceHangingProtocolSequence;
    typedef gdcm::Attribute<0x0072, 0x0014> NumberOfPriorsReferenced;
    typedef gdcm::Attribute<0x0072, 0x0020> ImageSetsSequence;
    typedef gdcm::Attribute<0x0072, 0x0022> ImageSetSelectorSequence;
    typedef gdcm::Attribute<0x0072, 0x0024> ImageSetSelectorUsageFlag;
    typedef gdcm::Attribute<0x0072, 0x0026> SelectorAttribute;
    typedef gdcm::Attribute<0x0072, 0x0028> SelectorValueNumber;
    typedef gdcm::Attribute<0x0072, 0x0030> TimeBasedImageSetsSequence;
    typedef gdcm::Attribute<0x0072, 0x0032> ImageSetNumber;
    typedef gdcm::Attribute<0x0072, 0x0034> ImageSetSelectorCategory;
    typedef gdcm::Attribute<0x0072, 0x0038> RelativeTime;
    typedef gdcm::Attribute<0x0072, 0x003a> RelativeTimeUnits;
    typedef gdcm::Attribute<0x0072, 0x003c> AbstractPriorValue;
    typedef gdcm::Attribute<0x0072, 0x003e> AbstractPriorCodeSequence;
    typedef gdcm::Attribute<0x0072, 0x0040> ImageSetLabel;
    typedef gdcm::Attribute<0x0072, 0x0050> SelectorAttributeVR;
    typedef gdcm::Attribute<0x0072, 0x0052> SelectorSequencePointer;
    typedef gdcm::Attribute<0x0072, 0x0054> SelectorSequencePointerPrivateCreator;
    typedef gdcm::Attribute<0x0072, 0x0056> SelectorAttributePrivateCreator;
    typedef gdcm::Attribute<0x0072, 0x0060> SelectorATValue;
    typedef gdcm::Attribute<0x0072, 0x0062> SelectorCSValue;
    typedef gdcm::Attribute<0x0072, 0x0064> SelectorISValue;
    typedef gdcm::Attribute<0x0072, 0x0066> SelectorLOValue;
    typedef gdcm::Attribute<0x0072, 0x0068> SelectorLTValue;
    typedef gdcm::Attribute<0x0072, 0x006a> SelectorPNValue;
    typedef gdcm::Attribute<0x0072, 0x006c> SelectorSHValue;
    typedef gdcm::Attribute<0x0072, 0x006e> SelectorSTValue;
    typedef gdcm::Attribute<0x0072, 0x0070> SelectorUTValue;
    typedef gdcm::Attribute<0x0072, 0x0072> SelectorDSValue;
    typedef gdcm::Attribute<0x0072, 0x0074> SelectorFDValue;
    typedef gdcm::Attribute<0x0072, 0x0076> SelectorFLValue;
    typedef gdcm::Attribute<0x0072, 0x0078> SelectorULValue;
    typedef gdcm::Attribute<0x0072, 0x007a> SelectorUSValue;
    typedef gdcm::Attribute<0x0072, 0x007c> SelectorSLValue;
    typedef gdcm::Attribute<0x0072, 0x007e> SelectorSSValue;
    typedef gdcm::Attribute<0x0072, 0x0080> SelectorCodeSequenceValue;
    typedef gdcm::Attribute<0x0072, 0x0100> NumberOfScreens;
    typedef gdcm::Attribute<0x0072, 0x0102> NominalScreenDefinitionSequence;
    typedef gdcm::Attribute<0x0072, 0x0104> NumberOfVerticalPixels;
    typedef gdcm::Attribute<0x0072, 0x0106> NumberOfHorizontalPixels;
    typedef gdcm::Attribute<0x0072, 0x0108> DisplayEnvironmentSpatialPosition;
    typedef gdcm::Attribute<0x0072, 0x010a> ScreenMinimumGrayscaleBitDepth;
    typedef gdcm::Attribute<0x0072, 0x010c> ScreenMinimumColorBitDepth;
    typedef gdcm::Attribute<0x0072, 0x010e> ApplicationMaximumRepaintTime;
    typedef gdcm::Attribute<0x0072, 0x0200> DisplaySetsSequence;
    typedef gdcm::Attribute<0x0072, 0x0202> DisplaySetNumber;
    typedef gdcm::Attribute<0x0072, 0x0203> DisplaySetLabel;
    typedef gdcm::Attribute<0x0072, 0x0204> DisplaySetPresentationGroup;
    typedef gdcm::Attribute<0x0072, 0x0206> DisplaySetPresentationGroupDescription;
    typedef gdcm::Attribute<0x0072, 0x0208> PartialDataDisplayHandling;
    typedef gdcm::Attribute<0x0072, 0x0210> SynchronizedScrollingSequence;
    typedef gdcm::Attribute<0x0072, 0x0212> DisplaySetScrollingGroup;
    typedef gdcm::Attribute<0x0072, 0x0214> NavigationIndicatorSequence;
    typedef gdcm::Attribute<0x0072, 0x0216> NavigationDisplaySet;
    typedef gdcm::Attribute<0x0072, 0x0218> ReferenceDisplaySets;
    typedef gdcm::Attribute<0x0072, 0x0300> ImageBoxesSequence;
    typedef gdcm::Attribute<0x0072, 0x0302> ImageBoxNumber;
    typedef gdcm::Attribute<0x0072, 0x0304> ImageBoxLayoutType;
    typedef gdcm::Attribute<0x0072, 0x0306> ImageBoxTileHorizontalDimension;
    typedef gdcm::Attribute<0x0072, 0x0308> ImageBoxTileVerticalDimension;
    typedef gdcm::Attribute<0x0072, 0x0310> ImageBoxScrollDirection;
    typedef gdcm::Attribute<0x0072, 0x0312> ImageBoxSmallScrollType;
    typedef gdcm::Attribute<0x0072, 0x0314> ImageBoxSmallScrollAmount;
    typedef gdcm::Attribute<0x0072, 0x0316> ImageBoxLargeScrollType;
    typedef gdcm::Attribute<0x0072, 0x0318> ImageBoxLargeScrollAmount;
    typedef gdcm::Attribute<0x0072, 0x0320> ImageBoxOverlapPriority;
    typedef gdcm::Attribute<0x0072, 0x0330> CineRelativeToRealTime;
    typedef gdcm::Attribute<0x0072, 0x0400> FilterOperationsSequence;
    typedef gdcm::Attribute<0x0072, 0x0402> FilterByCategory;
    typedef gdcm::Attribute<0x0072, 0x0404> FilterByAttributePresence;
    typedef gdcm::Attribute<0x0072, 0x0406> FilterByOperator;
    typedef gdcm::Attribute<0x0072, 0x0420> StructuredDisplayBackgroundCIELabValue;
    typedef gdcm::Attribute<0x0072, 0x0421> EmptyImageBoxCIELabValue;
    typedef gdcm::Attribute<0x0072, 0x0422> StructuredDisplayImageBoxSequence;
    typedef gdcm::Attribute<0x0072, 0x0424> StructuredDisplayTextBoxSequence;
    typedef gdcm::Attribute<0x0072, 0x0427> ReferencedFirstFrameSequence;
    typedef gdcm::Attribute<0x0072, 0x0430> ImageBoxSynchronizationSequence;
    typedef gdcm::Attribute<0x0072, 0x0432> SynchronizedImageBoxList;
    typedef gdcm::Attribute<0x0072, 0x0434> TypeOfSynchronization;
    typedef gdcm::Attribute<0x0072, 0x0500> BlendingOperationType;
    typedef gdcm::Attribute<0x0072, 0x0510> ReformattingOperationType;
    typedef gdcm::Attribute<0x0072, 0x0512> ReformattingThickness;
    typedef gdcm::Attribute<0x0072, 0x0514> ReformattingInterval;
    typedef gdcm::Attribute<0x0072, 0x0516> ReformattingOperationInitialViewDirection;
    typedef gdcm::Attribute<0x0072, 0x0520> ThreeDRenderingType;
    typedef gdcm::Attribute<0x0072, 0x0600> SortingOperationsSequence;
    typedef gdcm::Attribute<0x0072, 0x0602> SortByCategory;
    typedef gdcm::Attribute<0x0072, 0x0604> SortingDirection;
    typedef gdcm::Attribute<0x0072, 0x0700> DisplaySetPatientOrientation;
    typedef gdcm::Attribute<0x0072, 0x0702> VOIType;
    typedef gdcm::Attribute<0x0072, 0x0704> PseudoColorType;
    typedef gdcm::Attribute<0x0072, 0x0705> PseudoColorPaletteInstanceReferenceSequence;
    typedef gdcm::Attribute<0x0072, 0x0706> ShowGrayscaleInverted;
    typedef gdcm::Attribute<0x0072, 0x0710> ShowImageTrueSizeFlag;
    typedef gdcm::Attribute<0x0072, 0x0712> ShowGraphicAnnotationFlag;
    typedef gdcm::Attribute<0x0072, 0x0714> ShowPatientDemographicsFlag;
    typedef gdcm::Attribute<0x0072, 0x0716> ShowAcquisitionTechniquesFlag;
    typedef gdcm::Attribute<0x0072, 0x0717> DisplaySetHorizontalJustification;
    typedef gdcm::Attribute<0x0072, 0x0718> DisplaySetVerticalJustification;
    typedef gdcm::Attribute<0x0074, 0x0120> ContinuationStartMeterset;
    typedef gdcm::Attribute<0x0074, 0x0121> ContinuationEndMeterset;
    typedef gdcm::Attribute<0x0074, 0x1000> ProcedureStepState;
    typedef gdcm::Attribute<0x0074, 0x1002> ProcedureStepProgressInformationSequence;
    typedef gdcm::Attribute<0x0074, 0x1004> ProcedureStepProgress;
    typedef gdcm::Attribute<0x0074, 0x1006> ProcedureStepProgressDescription;
    typedef gdcm::Attribute<0x0074, 0x1008> ProcedureStepCommunicationsURISequence;
    typedef gdcm::Attribute<0x0074, 0x100a> ContactURI;
    typedef gdcm::Attribute<0x0074, 0x100c> ContactDisplayName;
    typedef gdcm::Attribute<0x0074, 0x100e> ProcedureStepDiscontinuationReasonCodeSequence;
    typedef gdcm::Attribute<0x0074, 0x1020> BeamTaskSequence;
    typedef gdcm::Attribute<0x0074, 0x1022> BeamTaskType;
    typedef gdcm::Attribute<0x0074, 0x1024> BeamOrderIndexTrial;
    typedef gdcm::Attribute<0x0074, 0x1026> TableTopVerticalAdjustedPosition;
    typedef gdcm::Attribute<0x0074, 0x1027> TableTopLongitudinalAdjustedPosition;
    typedef gdcm::Attribute<0x0074, 0x1028> TableTopLateralAdjustedPosition;
    typedef gdcm::Attribute<0x0074, 0x102a> PatientSupportAdjustedAngle;
    typedef gdcm::Attribute<0x0074, 0x102b> TableTopEccentricAdjustedAngle;
    typedef gdcm::Attribute<0x0074, 0x102c> TableTopPitchAdjustedAngle;
    typedef gdcm::Attribute<0x0074, 0x102d> TableTopRollAdjustedAngle;
    typedef gdcm::Attribute<0x0074, 0x1030> DeliveryVerificationImageSequence;
    typedef gdcm::Attribute<0x0074, 0x1032> VerificationImageTiming;
    typedef gdcm::Attribute<0x0074, 0x1034> DoubleExposureFlag;
    typedef gdcm::Attribute<0x0074, 0x1036> DoubleExposureOrdering;
    typedef gdcm::Attribute<0x0074, 0x1038> DoubleExposureMetersetTrial;
    typedef gdcm::Attribute<0x0074, 0x103a> DoubleExposureFieldDeltaTrial;
    typedef gdcm::Attribute<0x0074, 0x1040> RelatedReferenceRTImageSequence;
    typedef gdcm::Attribute<0x0074, 0x1042> GeneralMachineVerificationSequence;
    typedef gdcm::Attribute<0x0074, 0x1044> ConventionalMachineVerificationSequence;
    typedef gdcm::Attribute<0x0074, 0x1046> IonMachineVerificationSequence;
    typedef gdcm::Attribute<0x0074, 0x1048> FailedAttributesSequence;
    typedef gdcm::Attribute<0x0074, 0x104a> OverriddenAttributesSequence;
    typedef gdcm::Attribute<0x0074, 0x104c> ConventionalControlPointVerificationSequence;
    typedef gdcm::Attribute<0x0074, 0x104e> IonControlPointVerificationSequence;
    typedef gdcm::Attribute<0x0074, 0x1050> AttributeOccurrenceSequence;
    typedef gdcm::Attribute<0x0074, 0x1052> AttributeOccurrencePointer;
    typedef gdcm::Attribute<0x0074, 0x1054> AttributeItemSelector;
    typedef gdcm::Attribute<0x0074, 0x1056> AttributeOccurrencePrivateCreator;
    typedef gdcm::Attribute<0x0074, 0x1057> SelectorSequencePointerItems;
    typedef gdcm::Attribute<0x0074, 0x1200> ScheduledProcedureStepPriority;
    typedef gdcm::Attribute<0x0074, 0x1202> WorklistLabel;
    typedef gdcm::Attribute<0x0074, 0x1204> ProcedureStepLabel;
    typedef gdcm::Attribute<0x0074, 0x1210> ScheduledProcessingParametersSequence;
    typedef gdcm::Attribute<0x0074, 0x1212> PerformedProcessingParametersSequence;
    typedef gdcm::Attribute<0x0074, 0x1216> UnifiedProcedureStepPerformedProcedureSequence;
    typedef gdcm::Attribute<0x0074, 0x1220> RelatedProcedureStepSequence;
    typedef gdcm::Attribute<0x0074, 0x1222> ProcedureStepRelationshipType;
    typedef gdcm::Attribute<0x0074, 0x1224> ReplacedProcedureStepSequence;
    typedef gdcm::Attribute<0x0074, 0x1230> DeletionLock;
    typedef gdcm::Attribute<0x0074, 0x1234> ReceivingAE;
    typedef gdcm::Attribute<0x0074, 0x1236> RequestingAE;
    typedef gdcm::Attribute<0x0074, 0x1238> ReasonForCancellation;
    typedef gdcm::Attribute<0x0074, 0x1242> SCPStatus;
    typedef gdcm::Attribute<0x0074, 0x1244> SubscriptionListStatus;
    typedef gdcm::Attribute<0x0074, 0x1246> UnifiedProcedureStepListStatus;
    typedef gdcm::Attribute<0x0074, 0x1324> BeamOrderIndex;
    typedef gdcm::Attribute<0x0074, 0x1338> DoubleExposureMeterset;
    typedef gdcm::Attribute<0x0074, 0x133a> DoubleExposureFieldDelta;
    typedef gdcm::Attribute<0x0076, 0x0001> ImplantAssemblyTemplateName;
    typedef gdcm::Attribute<0x0076, 0x0003> ImplantAssemblyTemplateIssuer;
    typedef gdcm::Attribute<0x0076, 0x0006> ImplantAssemblyTemplateVersion;
    typedef gdcm::Attribute<0x0076, 0x0008> ReplacedImplantAssemblyTemplateSequence;
    typedef gdcm::Attribute<0x0076, 0x000a> ImplantAssemblyTemplateType;
    typedef gdcm::Attribute<0x0076, 0x000c> OriginalImplantAssemblyTemplateSequence;
    typedef gdcm::Attribute<0x0076, 0x000e> DerivationImplantAssemblyTemplateSequence;
    typedef gdcm::Attribute<0x0076, 0x0010> ImplantAssemblyTemplateTargetAnatomySequence;
    typedef gdcm::Attribute<0x0076, 0x0020> ProcedureTypeCodeSequence;
    typedef gdcm::Attribute<0x0076, 0x0030> SurgicalTechnique;
    typedef gdcm::Attribute<0x0076, 0x0032> ComponentTypesSequence;
    typedef gdcm::Attribute<0x0076, 0x0034> ComponentTypeCodeSequence;
    typedef gdcm::Attribute<0x0076, 0x0036> ExclusiveComponentType;
    typedef gdcm::Attribute<0x0076, 0x0038> MandatoryComponentType;
    typedef gdcm::Attribute<0x0076, 0x0040> ComponentSequence;
    typedef gdcm::Attribute<0x0076, 0x0055> ComponentID;
    typedef gdcm::Attribute<0x0076, 0x0060> ComponentAssemblySequence;
    typedef gdcm::Attribute<0x0076, 0x0070> Component1ReferencedID;
    typedef gdcm::Attribute<0x0076, 0x0080> Component1ReferencedMatingFeatureSetID;
    typedef gdcm::Attribute<0x0076, 0x0090> Component1ReferencedMatingFeatureID;
    typedef gdcm::Attribute<0x0076, 0x00a0> Component2ReferencedID;
    typedef gdcm::Attribute<0x0076, 0x00b0> Component2ReferencedMatingFeatureSetID;
    typedef gdcm::Attribute<0x0076, 0x00c0> Component2ReferencedMatingFeatureID;
    typedef gdcm::Attribute<0x0078, 0x0001> ImplantTemplateGroupName;
    typedef gdcm::Attribute<0x0078, 0x0010> ImplantTemplateGroupDescription;
    typedef gdcm::Attribute<0x0078, 0x0020> ImplantTemplateGroupIssuer;
    typedef gdcm::Attribute<0x0078, 0x0024> ImplantTemplateGroupVersion;
    typedef gdcm::Attribute<0x0078, 0x0026> ReplacedImplantTemplateGroupSequence;
    typedef gdcm::Attribute<0x0078, 0x0028> ImplantTemplateGroupTargetAnatomySequence;
    typedef gdcm::Attribute<0x0078, 0x002a> ImplantTemplateGroupMembersSequence;
    typedef gdcm::Attribute<0x0078, 0x002e> ImplantTemplateGroupMemberID;
    typedef gdcm::Attribute<0x0078, 0x0050> ThreeDImplantTemplateGroupMemberMatchingPoint;
    typedef gdcm::Attribute<0x0078, 0x0060> ThreeDImplantTemplateGroupMemberMatchingAxes;
    typedef gdcm::Attribute<0x0078, 0x0070> ImplantTemplateGroupMemberMatching2DCoordinatesSequence;
    typedef gdcm::Attribute<0x0078, 0x0090> TwoDImplantTemplateGroupMemberMatchingPoint;
    typedef gdcm::Attribute<0x0078, 0x00a0> TwoDImplantTemplateGroupMemberMatchingAxes;
    typedef gdcm::Attribute<0x0078, 0x00b0> ImplantTemplateGroupVariationDimensionSequence;
    typedef gdcm::Attribute<0x0078, 0x00b2> ImplantTemplateGroupVariationDimensionName;
    typedef gdcm::Attribute<0x0078, 0x00b4> ImplantTemplateGroupVariationDimensionRankSequence;
    typedef gdcm::Attribute<0x0078, 0x00b6> ReferencedImplantTemplateGroupMemberID;
    typedef gdcm::Attribute<0x0078, 0x00b8> ImplantTemplateGroupVariationDimensionRank;
    typedef gdcm::Attribute<0x0088, 0x0130> StorageMediaFileSetID;
    typedef gdcm::Attribute<0x0088, 0x0140> StorageMediaFileSetUID;
    typedef gdcm::Attribute<0x0088, 0x0200> IconImageSequence;
    typedef gdcm::Attribute<0x0088, 0x0904> TopicTitle;
    typedef gdcm::Attribute<0x0088, 0x0906> TopicSubject;
    typedef gdcm::Attribute<0x0088, 0x0910> TopicAuthor;
    typedef gdcm::Attribute<0x0088, 0x0912> TopicKeywords;
    typedef gdcm::Attribute<0x0100, 0x0410> SOPInstanceStatus;
    typedef gdcm::Attribute<0x0100, 0x0420> SOPAuthorizationDateTime;
    typedef gdcm::Attribute<0x0100, 0x0424> SOPAuthorizationComment;
    typedef gdcm::Attribute<0x0100, 0x0426> AuthorizationEquipmentCertificationNumber;
    typedef gdcm::Attribute<0x0400, 0x0005> MACIDNumber;
    typedef gdcm::Attribute<0x0400, 0x0010> MACCalculationTransferSyntaxUID;
    typedef gdcm::Attribute<0x0400, 0x0015> MACAlgorithm;
    typedef gdcm::Attribute<0x0400, 0x0020> DataElementsSigned;
    typedef gdcm::Attribute<0x0400, 0x0100> DigitalSignatureUID;
    typedef gdcm::Attribute<0x0400, 0x0105> DigitalSignatureDateTime;
    typedef gdcm::Attribute<0x0400, 0x0110> CertificateType;
    typedef gdcm::Attribute<0x0400, 0x0115> CertificateOfSigner;
    typedef gdcm::Attribute<0x0400, 0x0120> Signature;
    typedef gdcm::Attribute<0x0400, 0x0305> CertifiedTimestampType;
    typedef gdcm::Attribute<0x0400, 0x0310> CertifiedTimestamp;
    typedef gdcm::Attribute<0x0400, 0x0401> DigitalSignaturePurposeCodeSequence;
    typedef gdcm::Attribute<0x0400, 0x0402> ReferencedDigitalSignatureSequence;
    typedef gdcm::Attribute<0x0400, 0x0403> ReferencedSOPInstanceMACSequence;
    typedef gdcm::Attribute<0x0400, 0x0404> MAC;
    typedef gdcm::Attribute<0x0400, 0x0500> EncryptedAttributesSequence;
    typedef gdcm::Attribute<0x0400, 0x0510> EncryptedContentTransferSyntaxUID;
    typedef gdcm::Attribute<0x0400, 0x0520> EncryptedContent;
    typedef gdcm::Attribute<0x0400, 0x0550> ModifiedAttributesSequence;
    typedef gdcm::Attribute<0x0400, 0x0561> OriginalAttributesSequence;
    typedef gdcm::Attribute<0x0400, 0x0562> AttributeModificationDateTime;
    typedef gdcm::Attribute<0x0400, 0x0563> ModifyingSystem;
    typedef gdcm::Attribute<0x0400, 0x0564> SourceOfPreviousValues;
    typedef gdcm::Attribute<0x0400, 0x0565> ReasonForTheAttributeModification;
    typedef gdcm::Attribute<0x1000, 0x0000> EscapeTriplet;
    typedef gdcm::Attribute<0x1000, 0x0001> RunLengthTriplet;
    typedef gdcm::Attribute<0x1000, 0x0002> HuffmanTableSize;
    typedef gdcm::Attribute<0x1000, 0x0003> HuffmanTableTriplet;
    typedef gdcm::Attribute<0x1000, 0x0004> ShiftTableSize;
    typedef gdcm::Attribute<0x1000, 0x0005> ShiftTableTriplet;
    typedef gdcm::Attribute<0x1010, 0x0000> ZonalMap;
    typedef gdcm::Attribute<0x2000, 0x0010> NumberOfCopies;
    typedef gdcm::Attribute<0x2000, 0x001e> PrinterConfigurationSequence;
    typedef gdcm::Attribute<0x2000, 0x0020> PrintPriority;
    typedef gdcm::Attribute<0x2000, 0x0030> MediumType;
    typedef gdcm::Attribute<0x2000, 0x0040> FilmDestination;
    typedef gdcm::Attribute<0x2000, 0x0050> FilmSessionLabel;
    typedef gdcm::Attribute<0x2000, 0x0060> MemoryAllocation;
    typedef gdcm::Attribute<0x2000, 0x0061> MaximumMemoryAllocation;
    typedef gdcm::Attribute<0x2000, 0x0062> ColorImagePrintingFlag;
    typedef gdcm::Attribute<0x2000, 0x0063> CollationFlag;
    typedef gdcm::Attribute<0x2000, 0x0065> AnnotationFlag;
    typedef gdcm::Attribute<0x2000, 0x0067> ImageOverlayFlag;
    typedef gdcm::Attribute<0x2000, 0x0069> PresentationLUTFlag;
    typedef gdcm::Attribute<0x2000, 0x006a> ImageBoxPresentationLUTFlag;
    typedef gdcm::Attribute<0x2000, 0x00a0> MemoryBitDepth;
    typedef gdcm::Attribute<0x2000, 0x00a1> PrintingBitDepth;
    typedef gdcm::Attribute<0x2000, 0x00a2> MediaInstalledSequence;
    typedef gdcm::Attribute<0x2000, 0x00a4> OtherMediaAvailableSequence;
    typedef gdcm::Attribute<0x2000, 0x00a8> SupportedImageDisplayFormatsSequence;
    typedef gdcm::Attribute<0x2000, 0x0500> ReferencedFilmBoxSequence;
    typedef gdcm::Attribute<0x2000, 0x0510> ReferencedStoredPrintSequence;
    typedef gdcm::Attribute<0x2010, 0x0010> ImageDisplayFormat;
    typedef gdcm::Attribute<0x2010, 0x0030> AnnotationDisplayFormatID;
    typedef gdcm::Attribute<0x2010, 0x0040> FilmOrientation;
    typedef gdcm::Attribute<0x2010, 0x0050> FilmSizeID;
    typedef gdcm::Attribute<0x2010, 0x0052> PrinterResolutionID;
    typedef gdcm::Attribute<0x2010, 0x0054> DefaultPrinterResolutionID;
    typedef gdcm::Attribute<0x2010, 0x0060> MagnificationType;
    typedef gdcm::Attribute<0x2010, 0x0080> SmoothingType;
    typedef gdcm::Attribute<0x2010, 0x00a6> DefaultMagnificationType;
    typedef gdcm::Attribute<0x2010, 0x00a7> OtherMagnificationTypesAvailable;
    typedef gdcm::Attribute<0x2010, 0x00a8> DefaultSmoothingType;
    typedef gdcm::Attribute<0x2010, 0x00a9> OtherSmoothingTypesAvailable;
    typedef gdcm::Attribute<0x2010, 0x0100> BorderDensity;
    typedef gdcm::Attribute<0x2010, 0x0110> EmptyImageDensity;
    typedef gdcm::Attribute<0x2010, 0x0120> MinDensity;
    typedef gdcm::Attribute<0x2010, 0x0130> MaxDensity;
    typedef gdcm::Attribute<0x2010, 0x0140> Trim;
    typedef gdcm::Attribute<0x2010, 0x0150> ConfigurationInformation;
    typedef gdcm::Attribute<0x2010, 0x0152> ConfigurationInformationDescription;
    typedef gdcm::Attribute<0x2010, 0x0154> MaximumCollatedFilms;
    typedef gdcm::Attribute<0x2010, 0x015e> Illumination;
    typedef gdcm::Attribute<0x2010, 0x0160> ReflectedAmbientLight;
    typedef gdcm::Attribute<0x2010, 0x0376> PrinterPixelSpacing;
    typedef gdcm::Attribute<0x2010, 0x0500> ReferencedFilmSessionSequence;
    typedef gdcm::Attribute<0x2010, 0x0510> ReferencedImageBoxSequence;
    typedef gdcm::Attribute<0x2010, 0x0520> ReferencedBasicAnnotationBoxSequence;
    typedef gdcm::Attribute<0x2020, 0x0010> ImageBoxPosition;
    typedef gdcm::Attribute<0x2020, 0x0020> Polarity;
    typedef gdcm::Attribute<0x2020, 0x0030> RequestedImageSize;
    typedef gdcm::Attribute<0x2020, 0x0040> RequestedDecimateCropBehavior;
    typedef gdcm::Attribute<0x2020, 0x0050> RequestedResolutionID;
    typedef gdcm::Attribute<0x2020, 0x00a0> RequestedImageSizeFlag;
    typedef gdcm::Attribute<0x2020, 0x00a2> DecimateCropResult;
    typedef gdcm::Attribute<0x2020, 0x0110> BasicGrayscaleImageSequence;
    typedef gdcm::Attribute<0x2020, 0x0111> BasicColorImageSequence;
    typedef gdcm::Attribute<0x2020, 0x0130> ReferencedImageOverlayBoxSequence;
    typedef gdcm::Attribute<0x2020, 0x0140> ReferencedVOILUTBoxSequence;
    typedef gdcm::Attribute<0x2030, 0x0010> AnnotationPosition;
    typedef gdcm::Attribute<0x2030, 0x0020> TextString;
    typedef gdcm::Attribute<0x2040, 0x0010> ReferencedOverlayPlaneSequence;
    typedef gdcm::Attribute<0x2040, 0x0011> ReferencedOverlayPlaneGroups;
    typedef gdcm::Attribute<0x2040, 0x0020> OverlayPixelDataSequence;
    typedef gdcm::Attribute<0x2040, 0x0060> OverlayMagnificationType;
    typedef gdcm::Attribute<0x2040, 0x0070> OverlaySmoothingType;
    typedef gdcm::Attribute<0x2040, 0x0072> OverlayOrImageMagnification;
    typedef gdcm::Attribute<0x2040, 0x0074> MagnifyToNumberOfColumns;
    typedef gdcm::Attribute<0x2040, 0x0080> OverlayForegroundDensity;
    typedef gdcm::Attribute<0x2040, 0x0082> OverlayBackgroundDensity;
    typedef gdcm::Attribute<0x2040, 0x0090> OverlayMode;
    typedef gdcm::Attribute<0x2040, 0x0100> ThresholdDensity;
    typedef gdcm::Attribute<0x2040, 0x0500> ReferencedImageBoxSequenceRetired;
    typedef gdcm::Attribute<0x2050, 0x0010> PresentationLUTSequence;
    typedef gdcm::Attribute<0x2050, 0x0020> PresentationLUTShape;
    typedef gdcm::Attribute<0x2050, 0x0500> ReferencedPresentationLUTSequence;
    typedef gdcm::Attribute<0x2100, 0x0010> PrintJobID;
    typedef gdcm::Attribute<0x2100, 0x0020> ExecutionStatus;
    typedef gdcm::Attribute<0x2100, 0x0030> ExecutionStatusInfo;
    typedef gdcm::Attribute<0x2100, 0x0040> CreationDate;
    typedef gdcm::Attribute<0x2100, 0x0050> CreationTime;
    typedef gdcm::Attribute<0x2100, 0x0070> Originator;
    typedef gdcm::Attribute<0x2100, 0x0140> DestinationAE;
    typedef gdcm::Attribute<0x2100, 0x0160> OwnerID;
    typedef gdcm::Attribute<0x2100, 0x0170> NumberOfFilms;
    typedef gdcm::Attribute<0x2100, 0x0500> ReferencedPrintJobSequencePullStoredPrint;
    typedef gdcm::Attribute<0x2110, 0x0010> PrinterStatus;
    typedef gdcm::Attribute<0x2110, 0x0020> PrinterStatusInfo;
    typedef gdcm::Attribute<0x2110, 0x0030> PrinterName;
    typedef gdcm::Attribute<0x2110, 0x0099> PrintQueueID;
    typedef gdcm::Attribute<0x2120, 0x0010> QueueStatus;
    typedef gdcm::Attribute<0x2120, 0x0050> PrintJobDescriptionSequence;
    typedef gdcm::Attribute<0x2120, 0x0070> ReferencedPrintJobSequence;
    typedef gdcm::Attribute<0x2130, 0x0010> PrintManagementCapabilitiesSequence;
    typedef gdcm::Attribute<0x2130, 0x0015> PrinterCharacteristicsSequence;
    typedef gdcm::Attribute<0x2130, 0x0030> FilmBoxContentSequence;
    typedef gdcm::Attribute<0x2130, 0x0040> ImageBoxContentSequence;
    typedef gdcm::Attribute<0x2130, 0x0050> AnnotationContentSequence;
    typedef gdcm::Attribute<0x2130, 0x0060> ImageOverlayBoxContentSequence;
    typedef gdcm::Attribute<0x2130, 0x0080> PresentationLUTContentSequence;
    typedef gdcm::Attribute<0x2130, 0x00a0> ProposedStudySequence;
    typedef gdcm::Attribute<0x2130, 0x00c0> OriginalImageSequence;
    typedef gdcm::Attribute<0x2200, 0x0001> LabelUsingInformationExtractedFromInstances;
    typedef gdcm::Attribute<0x2200, 0x0002> LabelText;
    typedef gdcm::Attribute<0x2200, 0x0003> LabelStyleSelection;
    typedef gdcm::Attribute<0x2200, 0x0004> MediaDisposition;
    typedef gdcm::Attribute<0x2200, 0x0005> BarcodeValue;
    typedef gdcm::Attribute<0x2200, 0x0006> BarcodeSymbology;
    typedef gdcm::Attribute<0x2200, 0x0007> AllowMediaSplitting;
    typedef gdcm::Attribute<0x2200, 0x0008> IncludeNonDICOMObjects;
    typedef gdcm::Attribute<0x2200, 0x0009> IncludeDisplayApplication;
    typedef gdcm::Attribute<0x2200, 0x000a> PreserveCompositeInstancesAfterMediaCreation;
    typedef gdcm::Attribute<0x2200, 0x000b> TotalNumberOfPiecesOfMediaCreated;
    typedef gdcm::Attribute<0x2200, 0x000c> RequestedMediaApplicationProfile;
    typedef gdcm::Attribute<0x2200, 0x000d> ReferencedStorageMediaSequence;
    typedef gdcm::Attribute<0x2200, 0x000e> FailureAttributes;
    typedef gdcm::Attribute<0x2200, 0x000f> AllowLossyCompression;
    typedef gdcm::Attribute<0x2200, 0x0020> RequestPriority;
    typedef gdcm::Attribute<0x3002, 0x0002> RTImageLabel;
    typedef gdcm::Attribute<0x3002, 0x0003> RTImageName;
    typedef gdcm::Attribute<0x3002, 0x0004> RTImageDescription;
    typedef gdcm::Attribute<0x3002, 0x000a> ReportedValuesOrigin;
    typedef gdcm::Attribute<0x3002, 0x000c> RTImagePlane;
    typedef gdcm::Attribute<0x3002, 0x000d> XRayImageReceptorTranslation;
    typedef gdcm::Attribute<0x3002, 0x000e> XRayImageReceptorAngle;
    typedef gdcm::Attribute<0x3002, 0x0010> RTImageOrientation;
    typedef gdcm::Attribute<0x3002, 0x0011> ImagePlanePixelSpacing;
    typedef gdcm::Attribute<0x3002, 0x0012> RTImagePosition;
    typedef gdcm::Attribute<0x3002, 0x0020> RadiationMachineName;
    typedef gdcm::Attribute<0x3002, 0x0022> RadiationMachineSAD;
    typedef gdcm::Attribute<0x3002, 0x0024> RadiationMachineSSD;
    typedef gdcm::Attribute<0x3002, 0x0026> RTImageSID;
    typedef gdcm::Attribute<0x3002, 0x0028> SourceToReferenceObjectDistance;
    typedef gdcm::Attribute<0x3002, 0x0029> FractionNumber;
    typedef gdcm::Attribute<0x3002, 0x0030> ExposureSequence;
    typedef gdcm::Attribute<0x3002, 0x0032> MetersetExposure;
    typedef gdcm::Attribute<0x3002, 0x0034> DiaphragmPosition;
    typedef gdcm::Attribute<0x3002, 0x0040> FluenceMapSequence;
    typedef gdcm::Attribute<0x3002, 0x0041> FluenceDataSource;
    typedef gdcm::Attribute<0x3002, 0x0042> FluenceDataScale;
    typedef gdcm::Attribute<0x3002, 0x0050> PrimaryFluenceModeSequence;
    typedef gdcm::Attribute<0x3002, 0x0051> FluenceMode;
    typedef gdcm::Attribute<0x3002, 0x0052> FluenceModeID;
    typedef gdcm::Attribute<0x3004, 0x0001> DVHType;
    typedef gdcm::Attribute<0x3004, 0x0002> DoseUnits;
    typedef gdcm::Attribute<0x3004, 0x0004> DoseType;
    typedef gdcm::Attribute<0x3004, 0x0006> DoseComment;
    typedef gdcm::Attribute<0x3004, 0x0008> NormalizationPoint;
    typedef gdcm::Attribute<0x3004, 0x000a> DoseSummationType;
    typedef gdcm::Attribute<0x3004, 0x000c> GridFrameOffsetVector;
    typedef gdcm::Attribute<0x3004, 0x000e> DoseGridScaling;
    typedef gdcm::Attribute<0x3004, 0x0010> RTDoseROISequence;
    typedef gdcm::Attribute<0x3004, 0x0012> DoseValue;
    typedef gdcm::Attribute<0x3004, 0x0014> TissueHeterogeneityCorrection;
    typedef gdcm::Attribute<0x3004, 0x0040> DVHNormalizationPoint;
    typedef gdcm::Attribute<0x3004, 0x0042> DVHNormalizationDoseValue;
    typedef gdcm::Attribute<0x3004, 0x0050> DVHSequence;
    typedef gdcm::Attribute<0x3004, 0x0052> DVHDoseScaling;
    typedef gdcm::Attribute<0x3004, 0x0054> DVHVolumeUnits;
    typedef gdcm::Attribute<0x3004, 0x0056> DVHNumberOfBins;
    typedef gdcm::Attribute<0x3004, 0x0058> DVHData;
    typedef gdcm::Attribute<0x3004, 0x0060> DVHReferencedROISequence;
    typedef gdcm::Attribute<0x3004, 0x0062> DVHROIContributionType;
    typedef gdcm::Attribute<0x3004, 0x0070> DVHMinimumDose;
    typedef gdcm::Attribute<0x3004, 0x0072> DVHMaximumDose;
    typedef gdcm::Attribute<0x3004, 0x0074> DVHMeanDose;
    typedef gdcm::Attribute<0x3006, 0x0002> StructureSetLabel;
    typedef gdcm::Attribute<0x3006, 0x0004> StructureSetName;
    typedef gdcm::Attribute<0x3006, 0x0006> StructureSetDescription;
    typedef gdcm::Attribute<0x3006, 0x0008> StructureSetDate;
    typedef gdcm::Attribute<0x3006, 0x0009> StructureSetTime;
    typedef gdcm::Attribute<0x3006, 0x0010> ReferencedFrameOfReferenceSequence;
    typedef gdcm::Attribute<0x3006, 0x0012> RTReferencedStudySequence;
    typedef gdcm::Attribute<0x3006, 0x0014> RTReferencedSeriesSequence;
    typedef gdcm::Attribute<0x3006, 0x0016> ContourImageSequence;
    typedef gdcm::Attribute<0x3006, 0x0020> StructureSetROISequence;
    typedef gdcm::Attribute<0x3006, 0x0022> ROINumber;
    typedef gdcm::Attribute<0x3006, 0x0024> ReferencedFrameOfReferenceUID;
    typedef gdcm::Attribute<0x3006, 0x0026> ROIName;
    typedef gdcm::Attribute<0x3006, 0x0028> ROIDescription;
    typedef gdcm::Attribute<0x3006, 0x002a> ROIDisplayColor;
    typedef gdcm::Attribute<0x3006, 0x002c> ROIVolume;
    typedef gdcm::Attribute<0x3006, 0x0030> RTRelatedROISequence;
    typedef gdcm::Attribute<0x3006, 0x0033> RTROIRelationship;
    typedef gdcm::Attribute<0x3006, 0x0036> ROIGenerationAlgorithm;
    typedef gdcm::Attribute<0x3006, 0x0038> ROIGenerationDescription;
    typedef gdcm::Attribute<0x3006, 0x0039> ROIContourSequence;
    typedef gdcm::Attribute<0x3006, 0x0040> ContourSequence;
    typedef gdcm::Attribute<0x3006, 0x0042> ContourGeometricType;
    typedef gdcm::Attribute<0x3006, 0x0044> ContourSlabThickness;
    typedef gdcm::Attribute<0x3006, 0x0045> ContourOffsetVector;
    typedef gdcm::Attribute<0x3006, 0x0046> NumberOfContourPoints;
    typedef gdcm::Attribute<0x3006, 0x0048> ContourNumber;
    typedef gdcm::Attribute<0x3006, 0x0049> AttachedContours;
    typedef gdcm::Attribute<0x3006, 0x0050> ContourData;
    typedef gdcm::Attribute<0x3006, 0x0080> RTROIObservationsSequence;
    typedef gdcm::Attribute<0x3006, 0x0082> ObservationNumber;
    typedef gdcm::Attribute<0x3006, 0x0084> ReferencedROINumber;
    typedef gdcm::Attribute<0x3006, 0x0085> ROIObservationLabel;
    typedef gdcm::Attribute<0x3006, 0x0086> RTROIIdentificationCodeSequence;
    typedef gdcm::Attribute<0x3006, 0x0088> ROIObservationDescription;
    typedef gdcm::Attribute<0x3006, 0x00a0> RelatedRTROIObservationsSequence;
    typedef gdcm::Attribute<0x3006, 0x00a4> RTROIInterpretedType;
    typedef gdcm::Attribute<0x3006, 0x00a6> ROIInterpreter;
    typedef gdcm::Attribute<0x3006, 0x00b0> ROIPhysicalPropertiesSequence;
    typedef gdcm::Attribute<0x3006, 0x00b2> ROIPhysicalProperty;
    typedef gdcm::Attribute<0x3006, 0x00b4> ROIPhysicalPropertyValue;
    typedef gdcm::Attribute<0x3006, 0x00b6> ROIElementalCompositionSequence;
    typedef gdcm::Attribute<0x3006, 0x00b7> ROIElementalCompositionAtomicNumber;
    typedef gdcm::Attribute<0x3006, 0x00b8> ROIElementalCompositionAtomicMassFraction;
    typedef gdcm::Attribute<0x3006, 0x00c0> FrameOfReferenceRelationshipSequence;
    typedef gdcm::Attribute<0x3006, 0x00c2> RelatedFrameOfReferenceUID;
    typedef gdcm::Attribute<0x3006, 0x00c4> FrameOfReferenceTransformationType;
    typedef gdcm::Attribute<0x3006, 0x00c6> FrameOfReferenceTransformationMatrix;
    typedef gdcm::Attribute<0x3006, 0x00c8> FrameOfReferenceTransformationComment;
    typedef gdcm::Attribute<0x3008, 0x0010> MeasuredDoseReferenceSequence;
    typedef gdcm::Attribute<0x3008, 0x0012> MeasuredDoseDescription;
    typedef gdcm::Attribute<0x3008, 0x0014> MeasuredDoseType;
    typedef gdcm::Attribute<0x3008, 0x0016> MeasuredDoseValue;
    typedef gdcm::Attribute<0x3008, 0x0020> TreatmentSessionBeamSequence;
    typedef gdcm::Attribute<0x3008, 0x0021> TreatmentSessionIonBeamSequence;
    typedef gdcm::Attribute<0x3008, 0x0022> CurrentFractionNumber;
    typedef gdcm::Attribute<0x3008, 0x0024> TreatmentControlPointDate;
    typedef gdcm::Attribute<0x3008, 0x0025> TreatmentControlPointTime;
    typedef gdcm::Attribute<0x3008, 0x002a> TreatmentTerminationStatus;
    typedef gdcm::Attribute<0x3008, 0x002b> TreatmentTerminationCode;
    typedef gdcm::Attribute<0x3008, 0x002c> TreatmentVerificationStatus;
    typedef gdcm::Attribute<0x3008, 0x0030> ReferencedTreatmentRecordSequence;
    typedef gdcm::Attribute<0x3008, 0x0032> SpecifiedPrimaryMeterset;
    typedef gdcm::Attribute<0x3008, 0x0033> SpecifiedSecondaryMeterset;
    typedef gdcm::Attribute<0x3008, 0x0036> DeliveredPrimaryMeterset;
    typedef gdcm::Attribute<0x3008, 0x0037> DeliveredSecondaryMeterset;
    typedef gdcm::Attribute<0x3008, 0x003a> SpecifiedTreatmentTime;
    typedef gdcm::Attribute<0x3008, 0x003b> DeliveredTreatmentTime;
    typedef gdcm::Attribute<0x3008, 0x0040> ControlPointDeliverySequence;
    typedef gdcm::Attribute<0x3008, 0x0041> IonControlPointDeliverySequence;
    typedef gdcm::Attribute<0x3008, 0x0042> SpecifiedMeterset;
    typedef gdcm::Attribute<0x3008, 0x0044> DeliveredMeterset;
    typedef gdcm::Attribute<0x3008, 0x0045> MetersetRateSet;
    typedef gdcm::Attribute<0x3008, 0x0046> MetersetRateDelivered;
    typedef gdcm::Attribute<0x3008, 0x0047> ScanSpotMetersetsDelivered;
    typedef gdcm::Attribute<0x3008, 0x0048> DoseRateDelivered;
    typedef gdcm::Attribute<0x3008, 0x0050> TreatmentSummaryCalculatedDoseReferenceSequence;
    typedef gdcm::Attribute<0x3008, 0x0052> CumulativeDoseToDoseReference;
    typedef gdcm::Attribute<0x3008, 0x0054> FirstTreatmentDate;
    typedef gdcm::Attribute<0x3008, 0x0056> MostRecentTreatmentDate;
    typedef gdcm::Attribute<0x3008, 0x005a> NumberOfFractionsDelivered;
    typedef gdcm::Attribute<0x3008, 0x0060> OverrideSequence;
    typedef gdcm::Attribute<0x3008, 0x0061> ParameterSequencePointer;
    typedef gdcm::Attribute<0x3008, 0x0062> OverrideParameterPointer;
    typedef gdcm::Attribute<0x3008, 0x0063> ParameterItemIndex;
    typedef gdcm::Attribute<0x3008, 0x0064> MeasuredDoseReferenceNumber;
    typedef gdcm::Attribute<0x3008, 0x0065> ParameterPointer;
    typedef gdcm::Attribute<0x3008, 0x0066> OverrideReason;
    typedef gdcm::Attribute<0x3008, 0x0068> CorrectedParameterSequence;
    typedef gdcm::Attribute<0x3008, 0x006a> CorrectionValue;
    typedef gdcm::Attribute<0x3008, 0x0070> CalculatedDoseReferenceSequence;
    typedef gdcm::Attribute<0x3008, 0x0072> CalculatedDoseReferenceNumber;
    typedef gdcm::Attribute<0x3008, 0x0074> CalculatedDoseReferenceDescription;
    typedef gdcm::Attribute<0x3008, 0x0076> CalculatedDoseReferenceDoseValue;
    typedef gdcm::Attribute<0x3008, 0x0078> StartMeterset;
    typedef gdcm::Attribute<0x3008, 0x007a> EndMeterset;
    typedef gdcm::Attribute<0x3008, 0x0080> ReferencedMeasuredDoseReferenceSequence;
    typedef gdcm::Attribute<0x3008, 0x0082> ReferencedMeasuredDoseReferenceNumber;
    typedef gdcm::Attribute<0x3008, 0x0090> ReferencedCalculatedDoseReferenceSequence;
    typedef gdcm::Attribute<0x3008, 0x0092> ReferencedCalculatedDoseReferenceNumber;
    typedef gdcm::Attribute<0x3008, 0x00a0> BeamLimitingDeviceLeafPairsSequence;
    typedef gdcm::Attribute<0x3008, 0x00b0> RecordedWedgeSequence;
    typedef gdcm::Attribute<0x3008, 0x00c0> RecordedCompensatorSequence;
    typedef gdcm::Attribute<0x3008, 0x00d0> RecordedBlockSequence;
    typedef gdcm::Attribute<0x3008, 0x00e0> TreatmentSummaryMeasuredDoseReferenceSequence;
    typedef gdcm::Attribute<0x3008, 0x00f0> RecordedSnoutSequence;
    typedef gdcm::Attribute<0x3008, 0x00f2> RecordedRangeShifterSequence;
    typedef gdcm::Attribute<0x3008, 0x00f4> RecordedLateralSpreadingDeviceSequence;
    typedef gdcm::Attribute<0x3008, 0x00f6> RecordedRangeModulatorSequence;
    typedef gdcm::Attribute<0x3008, 0x0100> RecordedSourceSequence;
    typedef gdcm::Attribute<0x3008, 0x0105> SourceSerialNumber;
    typedef gdcm::Attribute<0x3008, 0x0110> TreatmentSessionApplicationSetupSequence;
    typedef gdcm::Attribute<0x3008, 0x0116> ApplicationSetupCheck;
    typedef gdcm::Attribute<0x3008, 0x0120> RecordedBrachyAccessoryDeviceSequence;
    typedef gdcm::Attribute<0x3008, 0x0122> ReferencedBrachyAccessoryDeviceNumber;
    typedef gdcm::Attribute<0x3008, 0x0130> RecordedChannelSequence;
    typedef gdcm::Attribute<0x3008, 0x0132> SpecifiedChannelTotalTime;
    typedef gdcm::Attribute<0x3008, 0x0134> DeliveredChannelTotalTime;
    typedef gdcm::Attribute<0x3008, 0x0136> SpecifiedNumberOfPulses;
    typedef gdcm::Attribute<0x3008, 0x0138> DeliveredNumberOfPulses;
    typedef gdcm::Attribute<0x3008, 0x013a> SpecifiedPulseRepetitionInterval;
    typedef gdcm::Attribute<0x3008, 0x013c> DeliveredPulseRepetitionInterval;
    typedef gdcm::Attribute<0x3008, 0x0140> RecordedSourceApplicatorSequence;
    typedef gdcm::Attribute<0x3008, 0x0142> ReferencedSourceApplicatorNumber;
    typedef gdcm::Attribute<0x3008, 0x0150> RecordedChannelShieldSequence;
    typedef gdcm::Attribute<0x3008, 0x0152> ReferencedChannelShieldNumber;
    typedef gdcm::Attribute<0x3008, 0x0160> BrachyControlPointDeliveredSequence;
    typedef gdcm::Attribute<0x3008, 0x0162> SafePositionExitDate;
    typedef gdcm::Attribute<0x3008, 0x0164> SafePositionExitTime;
    typedef gdcm::Attribute<0x3008, 0x0166> SafePositionReturnDate;
    typedef gdcm::Attribute<0x3008, 0x0168> SafePositionReturnTime;
    typedef gdcm::Attribute<0x3008, 0x0200> CurrentTreatmentStatus;
    typedef gdcm::Attribute<0x3008, 0x0202> TreatmentStatusComment;
    typedef gdcm::Attribute<0x3008, 0x0220> FractionGroupSummarySequence;
    typedef gdcm::Attribute<0x3008, 0x0223> ReferencedFractionNumber;
    typedef gdcm::Attribute<0x3008, 0x0224> FractionGroupType;
    typedef gdcm::Attribute<0x3008, 0x0230> BeamStopperPosition;
    typedef gdcm::Attribute<0x3008, 0x0240> FractionStatusSummarySequence;
    typedef gdcm::Attribute<0x3008, 0x0250> TreatmentDate;
    typedef gdcm::Attribute<0x3008, 0x0251> TreatmentTime;
    typedef gdcm::Attribute<0x300a, 0x0002> RTPlanLabel;
    typedef gdcm::Attribute<0x300a, 0x0003> RTPlanName;
    typedef gdcm::Attribute<0x300a, 0x0004> RTPlanDescription;
    typedef gdcm::Attribute<0x300a, 0x0006> RTPlanDate;
    typedef gdcm::Attribute<0x300a, 0x0007> RTPlanTime;
    typedef gdcm::Attribute<0x300a, 0x0009> TreatmentProtocols;
    typedef gdcm::Attribute<0x300a, 0x000a> PlanIntent;
    typedef gdcm::Attribute<0x300a, 0x000b> TreatmentSites;
    typedef gdcm::Attribute<0x300a, 0x000c> RTPlanGeometry;
    typedef gdcm::Attribute<0x300a, 0x000e> PrescriptionDescription;
    typedef gdcm::Attribute<0x300a, 0x0010> DoseReferenceSequence;
    typedef gdcm::Attribute<0x300a, 0x0012> DoseReferenceNumber;
    typedef gdcm::Attribute<0x300a, 0x0013> DoseReferenceUID;
    typedef gdcm::Attribute<0x300a, 0x0014> DoseReferenceStructureType;
    typedef gdcm::Attribute<0x300a, 0x0015> NominalBeamEnergyUnit;
    typedef gdcm::Attribute<0x300a, 0x0016> DoseReferenceDescription;
    typedef gdcm::Attribute<0x300a, 0x0018> DoseReferencePointCoordinates;
    typedef gdcm::Attribute<0x300a, 0x001a> NominalPriorDose;
    typedef gdcm::Attribute<0x300a, 0x0020> DoseReferenceType;
    typedef gdcm::Attribute<0x300a, 0x0021> ConstraintWeight;
    typedef gdcm::Attribute<0x300a, 0x0022> DeliveryWarningDose;
    typedef gdcm::Attribute<0x300a, 0x0023> DeliveryMaximumDose;
    typedef gdcm::Attribute<0x300a, 0x0025> TargetMinimumDose;
    typedef gdcm::Attribute<0x300a, 0x0026> TargetPrescriptionDose;
    typedef gdcm::Attribute<0x300a, 0x0027> TargetMaximumDose;
    typedef gdcm::Attribute<0x300a, 0x0028> TargetUnderdoseVolumeFraction;
    typedef gdcm::Attribute<0x300a, 0x002a> OrganAtRiskFullVolumeDose;
    typedef gdcm::Attribute<0x300a, 0x002b> OrganAtRiskLimitDose;
    typedef gdcm::Attribute<0x300a, 0x002c> OrganAtRiskMaximumDose;
    typedef gdcm::Attribute<0x300a, 0x002d> OrganAtRiskOverdoseVolumeFraction;
    typedef gdcm::Attribute<0x300a, 0x0040> ToleranceTableSequence;
    typedef gdcm::Attribute<0x300a, 0x0042> ToleranceTableNumber;
    typedef gdcm::Attribute<0x300a, 0x0043> ToleranceTableLabel;
    typedef gdcm::Attribute<0x300a, 0x0044> GantryAngleTolerance;
    typedef gdcm::Attribute<0x300a, 0x0046> BeamLimitingDeviceAngleTolerance;
    typedef gdcm::Attribute<0x300a, 0x0048> BeamLimitingDeviceToleranceSequence;
    typedef gdcm::Attribute<0x300a, 0x004a> BeamLimitingDevicePositionTolerance;
    typedef gdcm::Attribute<0x300a, 0x004b> SnoutPositionTolerance;
    typedef gdcm::Attribute<0x300a, 0x004c> PatientSupportAngleTolerance;
    typedef gdcm::Attribute<0x300a, 0x004e> TableTopEccentricAngleTolerance;
    typedef gdcm::Attribute<0x300a, 0x004f> TableTopPitchAngleTolerance;
    typedef gdcm::Attribute<0x300a, 0x0050> TableTopRollAngleTolerance;
    typedef gdcm::Attribute<0x300a, 0x0051> TableTopVerticalPositionTolerance;
    typedef gdcm::Attribute<0x300a, 0x0052> TableTopLongitudinalPositionTolerance;
    typedef gdcm::Attribute<0x300a, 0x0053> TableTopLateralPositionTolerance;
    typedef gdcm::Attribute<0x300a, 0x0055> RTPlanRelationship;
    typedef gdcm::Attribute<0x300a, 0x0070> FractionGroupSequence;
    typedef gdcm::Attribute<0x300a, 0x0071> FractionGroupNumber;
    typedef gdcm::Attribute<0x300a, 0x0072> FractionGroupDescription;
    typedef gdcm::Attribute<0x300a, 0x0078> NumberOfFractionsPlanned;
    typedef gdcm::Attribute<0x300a, 0x0079> NumberOfFractionPatternDigitsPerDay;
    typedef gdcm::Attribute<0x300a, 0x007a> RepeatFractionCycleLength;
    typedef gdcm::Attribute<0x300a, 0x007b> FractionPattern;
    typedef gdcm::Attribute<0x300a, 0x0080> NumberOfBeams;
    typedef gdcm::Attribute<0x300a, 0x0082> BeamDoseSpecificationPoint;
    typedef gdcm::Attribute<0x300a, 0x0084> BeamDose;
    typedef gdcm::Attribute<0x300a, 0x0086> BeamMeterset;
    typedef gdcm::Attribute<0x300a, 0x0088> BeamDosePointDepth;
    typedef gdcm::Attribute<0x300a, 0x0089> BeamDosePointEquivalentDepth;
    typedef gdcm::Attribute<0x300a, 0x008a> BeamDosePointSSD;
    typedef gdcm::Attribute<0x300a, 0x00a0> NumberOfBrachyApplicationSetups;
    typedef gdcm::Attribute<0x300a, 0x00a2> BrachyApplicationSetupDoseSpecificationPoint;
    typedef gdcm::Attribute<0x300a, 0x00a4> BrachyApplicationSetupDose;
    typedef gdcm::Attribute<0x300a, 0x00b0> BeamSequence;
    typedef gdcm::Attribute<0x300a, 0x00b2> TreatmentMachineName;
    typedef gdcm::Attribute<0x300a, 0x00b3> PrimaryDosimeterUnit;
    typedef gdcm::Attribute<0x300a, 0x00b4> SourceAxisDistance;
    typedef gdcm::Attribute<0x300a, 0x00b6> BeamLimitingDeviceSequence;
    typedef gdcm::Attribute<0x300a, 0x00b8> RTBeamLimitingDeviceType;
    typedef gdcm::Attribute<0x300a, 0x00ba> SourceToBeamLimitingDeviceDistance;
    typedef gdcm::Attribute<0x300a, 0x00bb> IsocenterToBeamLimitingDeviceDistance;
    typedef gdcm::Attribute<0x300a, 0x00bc> NumberOfLeafJawPairs;
    typedef gdcm::Attribute<0x300a, 0x00be> LeafPositionBoundaries;
    typedef gdcm::Attribute<0x300a, 0x00c0> BeamNumber;
    typedef gdcm::Attribute<0x300a, 0x00c2> BeamName;
    typedef gdcm::Attribute<0x300a, 0x00c3> BeamDescription;
    typedef gdcm::Attribute<0x300a, 0x00c4> BeamType;
    typedef gdcm::Attribute<0x300a, 0x00c6> RadiationType;
    typedef gdcm::Attribute<0x300a, 0x00c7> HighDoseTechniqueType;
    typedef gdcm::Attribute<0x300a, 0x00c8> ReferenceImageNumber;
    typedef gdcm::Attribute<0x300a, 0x00ca> PlannedVerificationImageSequence;
    typedef gdcm::Attribute<0x300a, 0x00cc> ImagingDeviceSpecificAcquisitionParameters;
    typedef gdcm::Attribute<0x300a, 0x00ce> TreatmentDeliveryType;
    typedef gdcm::Attribute<0x300a, 0x00d0> NumberOfWedges;
    typedef gdcm::Attribute<0x300a, 0x00d1> WedgeSequence;
    typedef gdcm::Attribute<0x300a, 0x00d2> WedgeNumber;
    typedef gdcm::Attribute<0x300a, 0x00d3> WedgeType;
    typedef gdcm::Attribute<0x300a, 0x00d4> WedgeID;
    typedef gdcm::Attribute<0x300a, 0x00d5> WedgeAngle;
    typedef gdcm::Attribute<0x300a, 0x00d6> WedgeFactor;
    typedef gdcm::Attribute<0x300a, 0x00d7> TotalWedgeTrayWaterEquivalentThickness;
    typedef gdcm::Attribute<0x300a, 0x00d8> WedgeOrientation;
    typedef gdcm::Attribute<0x300a, 0x00d9> IsocenterToWedgeTrayDistance;
    typedef gdcm::Attribute<0x300a, 0x00da> SourceToWedgeTrayDistance;
    typedef gdcm::Attribute<0x300a, 0x00db> WedgeThinEdgePosition;
    typedef gdcm::Attribute<0x300a, 0x00dc> BolusID;
    typedef gdcm::Attribute<0x300a, 0x00dd> BolusDescription;
    typedef gdcm::Attribute<0x300a, 0x00e0> NumberOfCompensators;
    typedef gdcm::Attribute<0x300a, 0x00e1> MaterialID;
    typedef gdcm::Attribute<0x300a, 0x00e2> TotalCompensatorTrayFactor;
    typedef gdcm::Attribute<0x300a, 0x00e3> CompensatorSequence;
    typedef gdcm::Attribute<0x300a, 0x00e4> CompensatorNumber;
    typedef gdcm::Attribute<0x300a, 0x00e5> CompensatorID;
    typedef gdcm::Attribute<0x300a, 0x00e6> SourceToCompensatorTrayDistance;
    typedef gdcm::Attribute<0x300a, 0x00e7> CompensatorRows;
    typedef gdcm::Attribute<0x300a, 0x00e8> CompensatorColumns;
    typedef gdcm::Attribute<0x300a, 0x00e9> CompensatorPixelSpacing;
    typedef gdcm::Attribute<0x300a, 0x00ea> CompensatorPosition;
    typedef gdcm::Attribute<0x300a, 0x00eb> CompensatorTransmissionData;
    typedef gdcm::Attribute<0x300a, 0x00ec> CompensatorThicknessData;
    typedef gdcm::Attribute<0x300a, 0x00ed> NumberOfBoli;
    typedef gdcm::Attribute<0x300a, 0x00ee> CompensatorType;
    typedef gdcm::Attribute<0x300a, 0x00f0> NumberOfBlocks;
    typedef gdcm::Attribute<0x300a, 0x00f2> TotalBlockTrayFactor;
    typedef gdcm::Attribute<0x300a, 0x00f3> TotalBlockTrayWaterEquivalentThickness;
    typedef gdcm::Attribute<0x300a, 0x00f4> BlockSequence;
    typedef gdcm::Attribute<0x300a, 0x00f5> BlockTrayID;
    typedef gdcm::Attribute<0x300a, 0x00f6> SourceToBlockTrayDistance;
    typedef gdcm::Attribute<0x300a, 0x00f7> IsocenterToBlockTrayDistance;
    typedef gdcm::Attribute<0x300a, 0x00f8> BlockType;
    typedef gdcm::Attribute<0x300a, 0x00f9> AccessoryCode;
    typedef gdcm::Attribute<0x300a, 0x00fa> BlockDivergence;
    typedef gdcm::Attribute<0x300a, 0x00fb> BlockMountingPosition;
    typedef gdcm::Attribute<0x300a, 0x00fc> BlockNumber;
    typedef gdcm::Attribute<0x300a, 0x00fe> BlockName;
    typedef gdcm::Attribute<0x300a, 0x0100> BlockThickness;
    typedef gdcm::Attribute<0x300a, 0x0102> BlockTransmission;
    typedef gdcm::Attribute<0x300a, 0x0104> BlockNumberOfPoints;
    typedef gdcm::Attribute<0x300a, 0x0106> BlockData;
    typedef gdcm::Attribute<0x300a, 0x0107> ApplicatorSequence;
    typedef gdcm::Attribute<0x300a, 0x0108> ApplicatorID;
    typedef gdcm::Attribute<0x300a, 0x0109> ApplicatorType;
    typedef gdcm::Attribute<0x300a, 0x010a> ApplicatorDescription;
    typedef gdcm::Attribute<0x300a, 0x010c> CumulativeDoseReferenceCoefficient;
    typedef gdcm::Attribute<0x300a, 0x010e> FinalCumulativeMetersetWeight;
    typedef gdcm::Attribute<0x300a, 0x0110> NumberOfControlPoints;
    typedef gdcm::Attribute<0x300a, 0x0111> ControlPointSequence;
    typedef gdcm::Attribute<0x300a, 0x0112> ControlPointIndex;
    typedef gdcm::Attribute<0x300a, 0x0114> NominalBeamEnergy;
    typedef gdcm::Attribute<0x300a, 0x0115> DoseRateSet;
    typedef gdcm::Attribute<0x300a, 0x0116> WedgePositionSequence;
    typedef gdcm::Attribute<0x300a, 0x0118> WedgePosition;
    typedef gdcm::Attribute<0x300a, 0x011a> BeamLimitingDevicePositionSequence;
    typedef gdcm::Attribute<0x300a, 0x011c> LeafJawPositions;
    typedef gdcm::Attribute<0x300a, 0x011e> GantryAngle;
    typedef gdcm::Attribute<0x300a, 0x011f> GantryRotationDirection;
    typedef gdcm::Attribute<0x300a, 0x0120> BeamLimitingDeviceAngle;
    typedef gdcm::Attribute<0x300a, 0x0121> BeamLimitingDeviceRotationDirection;
    typedef gdcm::Attribute<0x300a, 0x0122> PatientSupportAngle;
    typedef gdcm::Attribute<0x300a, 0x0123> PatientSupportRotationDirection;
    typedef gdcm::Attribute<0x300a, 0x0124> TableTopEccentricAxisDistance;
    typedef gdcm::Attribute<0x300a, 0x0125> TableTopEccentricAngle;
    typedef gdcm::Attribute<0x300a, 0x0126> TableTopEccentricRotationDirection;
    typedef gdcm::Attribute<0x300a, 0x0128> TableTopVerticalPosition;
    typedef gdcm::Attribute<0x300a, 0x0129> TableTopLongitudinalPosition;
    typedef gdcm::Attribute<0x300a, 0x012a> TableTopLateralPosition;
    typedef gdcm::Attribute<0x300a, 0x012c> IsocenterPosition;
    typedef gdcm::Attribute<0x300a, 0x012e> SurfaceEntryPoint;
    typedef gdcm::Attribute<0x300a, 0x0130> SourceToSurfaceDistance;
    typedef gdcm::Attribute<0x300a, 0x0134> CumulativeMetersetWeight;
    typedef gdcm::Attribute<0x300a, 0x0140> TableTopPitchAngle;
    typedef gdcm::Attribute<0x300a, 0x0142> TableTopPitchRotationDirection;
    typedef gdcm::Attribute<0x300a, 0x0144> TableTopRollAngle;
    typedef gdcm::Attribute<0x300a, 0x0146> TableTopRollRotationDirection;
    typedef gdcm::Attribute<0x300a, 0x0148> HeadFixationAngle;
    typedef gdcm::Attribute<0x300a, 0x014a> GantryPitchAngle;
    typedef gdcm::Attribute<0x300a, 0x014c> GantryPitchRotationDirection;
    typedef gdcm::Attribute<0x300a, 0x014e> GantryPitchAngleTolerance;
    typedef gdcm::Attribute<0x300a, 0x0180> PatientSetupSequence;
    typedef gdcm::Attribute<0x300a, 0x0182> PatientSetupNumber;
    typedef gdcm::Attribute<0x300a, 0x0183> PatientSetupLabel;
    typedef gdcm::Attribute<0x300a, 0x0184> PatientAdditionalPosition;
    typedef gdcm::Attribute<0x300a, 0x0190> FixationDeviceSequence;
    typedef gdcm::Attribute<0x300a, 0x0192> FixationDeviceType;
    typedef gdcm::Attribute<0x300a, 0x0194> FixationDeviceLabel;
    typedef gdcm::Attribute<0x300a, 0x0196> FixationDeviceDescription;
    typedef gdcm::Attribute<0x300a, 0x0198> FixationDevicePosition;
    typedef gdcm::Attribute<0x300a, 0x0199> FixationDevicePitchAngle;
    typedef gdcm::Attribute<0x300a, 0x019a> FixationDeviceRollAngle;
    typedef gdcm::Attribute<0x300a, 0x01a0> ShieldingDeviceSequence;
    typedef gdcm::Attribute<0x300a, 0x01a2> ShieldingDeviceType;
    typedef gdcm::Attribute<0x300a, 0x01a4> ShieldingDeviceLabel;
    typedef gdcm::Attribute<0x300a, 0x01a6> ShieldingDeviceDescription;
    typedef gdcm::Attribute<0x300a, 0x01a8> ShieldingDevicePosition;
    typedef gdcm::Attribute<0x300a, 0x01b0> SetupTechnique;
    typedef gdcm::Attribute<0x300a, 0x01b2> SetupTechniqueDescription;
    typedef gdcm::Attribute<0x300a, 0x01b4> SetupDeviceSequence;
    typedef gdcm::Attribute<0x300a, 0x01b6> SetupDeviceType;
    typedef gdcm::Attribute<0x300a, 0x01b8> SetupDeviceLabel;
    typedef gdcm::Attribute<0x300a, 0x01ba> SetupDeviceDescription;
    typedef gdcm::Attribute<0x300a, 0x01bc> SetupDeviceParameter;
    typedef gdcm::Attribute<0x300a, 0x01d0> SetupReferenceDescription;
    typedef gdcm::Attribute<0x300a, 0x01d2> TableTopVerticalSetupDisplacement;
    typedef gdcm::Attribute<0x300a, 0x01d4> TableTopLongitudinalSetupDisplacement;
    typedef gdcm::Attribute<0x300a, 0x01d6> TableTopLateralSetupDisplacement;
    typedef gdcm::Attribute<0x300a, 0x0200> BrachyTreatmentTechnique;
    typedef gdcm::Attribute<0x300a, 0x0202> BrachyTreatmentType;
    typedef gdcm::Attribute<0x300a, 0x0206> TreatmentMachineSequence;
    typedef gdcm::Attribute<0x300a, 0x0210> SourceSequence;
    typedef gdcm::Attribute<0x300a, 0x0212> SourceNumber;
    typedef gdcm::Attribute<0x300a, 0x0214> SourceType;
    typedef gdcm::Attribute<0x300a, 0x0216> SourceManufacturer;
    typedef gdcm::Attribute<0x300a, 0x0218> ActiveSourceDiameter;
    typedef gdcm::Attribute<0x300a, 0x021a> ActiveSourceLength;
    typedef gdcm::Attribute<0x300a, 0x0222> SourceEncapsulationNominalThickness;
    typedef gdcm::Attribute<0x300a, 0x0224> SourceEncapsulationNominalTransmission;
    typedef gdcm::Attribute<0x300a, 0x0226> SourceIsotopeName;
    typedef gdcm::Attribute<0x300a, 0x0228> SourceIsotopeHalfLife;
    typedef gdcm::Attribute<0x300a, 0x0229> SourceStrengthUnits;
    typedef gdcm::Attribute<0x300a, 0x022a> ReferenceAirKermaRate;
    typedef gdcm::Attribute<0x300a, 0x022b> SourceStrength;
    typedef gdcm::Attribute<0x300a, 0x022c> SourceStrengthReferenceDate;
    typedef gdcm::Attribute<0x300a, 0x022e> SourceStrengthReferenceTime;
    typedef gdcm::Attribute<0x300a, 0x0230> ApplicationSetupSequence;
    typedef gdcm::Attribute<0x300a, 0x0232> ApplicationSetupType;
    typedef gdcm::Attribute<0x300a, 0x0234> ApplicationSetupNumber;
    typedef gdcm::Attribute<0x300a, 0x0236> ApplicationSetupName;
    typedef gdcm::Attribute<0x300a, 0x0238> ApplicationSetupManufacturer;
    typedef gdcm::Attribute<0x300a, 0x0240> TemplateNumber;
    typedef gdcm::Attribute<0x300a, 0x0242> TemplateType;
    typedef gdcm::Attribute<0x300a, 0x0244> TemplateName;
    typedef gdcm::Attribute<0x300a, 0x0250> TotalReferenceAirKerma;
    typedef gdcm::Attribute<0x300a, 0x0260> BrachyAccessoryDeviceSequence;
    typedef gdcm::Attribute<0x300a, 0x0262> BrachyAccessoryDeviceNumber;
    typedef gdcm::Attribute<0x300a, 0x0263> BrachyAccessoryDeviceID;
    typedef gdcm::Attribute<0x300a, 0x0264> BrachyAccessoryDeviceType;
    typedef gdcm::Attribute<0x300a, 0x0266> BrachyAccessoryDeviceName;
    typedef gdcm::Attribute<0x300a, 0x026a> BrachyAccessoryDeviceNominalThickness;
    typedef gdcm::Attribute<0x300a, 0x026c> BrachyAccessoryDeviceNominalTransmission;
    typedef gdcm::Attribute<0x300a, 0x0280> ChannelSequence;
    typedef gdcm::Attribute<0x300a, 0x0282> ChannelNumber;
    typedef gdcm::Attribute<0x300a, 0x0284> ChannelLength;
    typedef gdcm::Attribute<0x300a, 0x0286> ChannelTotalTime;
    typedef gdcm::Attribute<0x300a, 0x0288> SourceMovementType;
    typedef gdcm::Attribute<0x300a, 0x028a> NumberOfPulses;
    typedef gdcm::Attribute<0x300a, 0x028c> PulseRepetitionInterval;
    typedef gdcm::Attribute<0x300a, 0x0290> SourceApplicatorNumber;
    typedef gdcm::Attribute<0x300a, 0x0291> SourceApplicatorID;
    typedef gdcm::Attribute<0x300a, 0x0292> SourceApplicatorType;
    typedef gdcm::Attribute<0x300a, 0x0294> SourceApplicatorName;
    typedef gdcm::Attribute<0x300a, 0x0296> SourceApplicatorLength;
    typedef gdcm::Attribute<0x300a, 0x0298> SourceApplicatorManufacturer;
    typedef gdcm::Attribute<0x300a, 0x029c> SourceApplicatorWallNominalThickness;
    typedef gdcm::Attribute<0x300a, 0x029e> SourceApplicatorWallNominalTransmission;
    typedef gdcm::Attribute<0x300a, 0x02a0> SourceApplicatorStepSize;
    typedef gdcm::Attribute<0x300a, 0x02a2> TransferTubeNumber;
    typedef gdcm::Attribute<0x300a, 0x02a4> TransferTubeLength;
    typedef gdcm::Attribute<0x300a, 0x02b0> ChannelShieldSequence;
    typedef gdcm::Attribute<0x300a, 0x02b2> ChannelShieldNumber;
    typedef gdcm::Attribute<0x300a, 0x02b3> ChannelShieldID;
    typedef gdcm::Attribute<0x300a, 0x02b4> ChannelShieldName;
    typedef gdcm::Attribute<0x300a, 0x02b8> ChannelShieldNominalThickness;
    typedef gdcm::Attribute<0x300a, 0x02ba> ChannelShieldNominalTransmission;
    typedef gdcm::Attribute<0x300a, 0x02c8> FinalCumulativeTimeWeight;
    typedef gdcm::Attribute<0x300a, 0x02d0> BrachyControlPointSequence;
    typedef gdcm::Attribute<0x300a, 0x02d2> ControlPointRelativePosition;
    typedef gdcm::Attribute<0x300a, 0x02d4> ControlPoint3DPosition;
    typedef gdcm::Attribute<0x300a, 0x02d6> CumulativeTimeWeight;
    typedef gdcm::Attribute<0x300a, 0x02e0> CompensatorDivergence;
    typedef gdcm::Attribute<0x300a, 0x02e1> CompensatorMountingPosition;
    typedef gdcm::Attribute<0x300a, 0x02e2> SourceToCompensatorDistance;
    typedef gdcm::Attribute<0x300a, 0x02e3> TotalCompensatorTrayWaterEquivalentThickness;
    typedef gdcm::Attribute<0x300a, 0x02e4> IsocenterToCompensatorTrayDistance;
    typedef gdcm::Attribute<0x300a, 0x02e5> CompensatorColumnOffset;
    typedef gdcm::Attribute<0x300a, 0x02e6> IsocenterToCompensatorDistances;
    typedef gdcm::Attribute<0x300a, 0x02e7> CompensatorRelativeStoppingPowerRatio;
    typedef gdcm::Attribute<0x300a, 0x02e8> CompensatorMillingToolDiameter;
    typedef gdcm::Attribute<0x300a, 0x02ea> IonRangeCompensatorSequence;
    typedef gdcm::Attribute<0x300a, 0x02eb> CompensatorDescription;
    typedef gdcm::Attribute<0x300a, 0x0302> RadiationMassNumber;
    typedef gdcm::Attribute<0x300a, 0x0304> RadiationAtomicNumber;
    typedef gdcm::Attribute<0x300a, 0x0306> RadiationChargeState;
    typedef gdcm::Attribute<0x300a, 0x0308> ScanMode;
    typedef gdcm::Attribute<0x300a, 0x030a> VirtualSourceAxisDistances;
    typedef gdcm::Attribute<0x300a, 0x030c> SnoutSequence;
    typedef gdcm::Attribute<0x300a, 0x030d> SnoutPosition;
    typedef gdcm::Attribute<0x300a, 0x030f> SnoutID;
    typedef gdcm::Attribute<0x300a, 0x0312> NumberOfRangeShifters;
    typedef gdcm::Attribute<0x300a, 0x0314> RangeShifterSequence;
    typedef gdcm::Attribute<0x300a, 0x0316> RangeShifterNumber;
    typedef gdcm::Attribute<0x300a, 0x0318> RangeShifterID;
    typedef gdcm::Attribute<0x300a, 0x0320> RangeShifterType;
    typedef gdcm::Attribute<0x300a, 0x0322> RangeShifterDescription;
    typedef gdcm::Attribute<0x300a, 0x0330> NumberOfLateralSpreadingDevices;
    typedef gdcm::Attribute<0x300a, 0x0332> LateralSpreadingDeviceSequence;
    typedef gdcm::Attribute<0x300a, 0x0334> LateralSpreadingDeviceNumber;
    typedef gdcm::Attribute<0x300a, 0x0336> LateralSpreadingDeviceID;
    typedef gdcm::Attribute<0x300a, 0x0338> LateralSpreadingDeviceType;
    typedef gdcm::Attribute<0x300a, 0x033a> LateralSpreadingDeviceDescription;
    typedef gdcm::Attribute<0x300a, 0x033c> LateralSpreadingDeviceWaterEquivalentThickness;
    typedef gdcm::Attribute<0x300a, 0x0340> NumberOfRangeModulators;
    typedef gdcm::Attribute<0x300a, 0x0342> RangeModulatorSequence;
    typedef gdcm::Attribute<0x300a, 0x0344> RangeModulatorNumber;
    typedef gdcm::Attribute<0x300a, 0x0346> RangeModulatorID;
    typedef gdcm::Attribute<0x300a, 0x0348> RangeModulatorType;
    typedef gdcm::Attribute<0x300a, 0x034a> RangeModulatorDescription;
    typedef gdcm::Attribute<0x300a, 0x034c> BeamCurrentModulationID;
    typedef gdcm::Attribute<0x300a, 0x0350> PatientSupportType;
    typedef gdcm::Attribute<0x300a, 0x0352> PatientSupportID;
    typedef gdcm::Attribute<0x300a, 0x0354> PatientSupportAccessoryCode;
    typedef gdcm::Attribute<0x300a, 0x0356> FixationLightAzimuthalAngle;
    typedef gdcm::Attribute<0x300a, 0x0358> FixationLightPolarAngle;
    typedef gdcm::Attribute<0x300a, 0x035a> MetersetRate;
    typedef gdcm::Attribute<0x300a, 0x0360> RangeShifterSettingsSequence;
    typedef gdcm::Attribute<0x300a, 0x0362> RangeShifterSetting;
    typedef gdcm::Attribute<0x300a, 0x0364> IsocenterToRangeShifterDistance;
    typedef gdcm::Attribute<0x300a, 0x0366> RangeShifterWaterEquivalentThickness;
    typedef gdcm::Attribute<0x300a, 0x0370> LateralSpreadingDeviceSettingsSequence;
    typedef gdcm::Attribute<0x300a, 0x0372> LateralSpreadingDeviceSetting;
    typedef gdcm::Attribute<0x300a, 0x0374> IsocenterToLateralSpreadingDeviceDistance;
    typedef gdcm::Attribute<0x300a, 0x0380> RangeModulatorSettingsSequence;
    typedef gdcm::Attribute<0x300a, 0x0382> RangeModulatorGatingStartValue;
    typedef gdcm::Attribute<0x300a, 0x0384> RangeModulatorGatingStopValue;
    typedef gdcm::Attribute<0x300a, 0x0386> RangeModulatorGatingStartWaterEquivalentThickness;
    typedef gdcm::Attribute<0x300a, 0x0388> RangeModulatorGatingStopWaterEquivalentThickness;
    typedef gdcm::Attribute<0x300a, 0x038a> IsocenterToRangeModulatorDistance;
    typedef gdcm::Attribute<0x300a, 0x0390> ScanSpotTuneID;
    typedef gdcm::Attribute<0x300a, 0x0392> NumberOfScanSpotPositions;
    typedef gdcm::Attribute<0x300a, 0x0394> ScanSpotPositionMap;
    typedef gdcm::Attribute<0x300a, 0x0396> ScanSpotMetersetWeights;
    typedef gdcm::Attribute<0x300a, 0x0398> ScanningSpotSize;
    typedef gdcm::Attribute<0x300a, 0x039a> NumberOfPaintings;
    typedef gdcm::Attribute<0x300a, 0x03a0> IonToleranceTableSequence;
    typedef gdcm::Attribute<0x300a, 0x03a2> IonBeamSequence;
    typedef gdcm::Attribute<0x300a, 0x03a4> IonBeamLimitingDeviceSequence;
    typedef gdcm::Attribute<0x300a, 0x03a6> IonBlockSequence;
    typedef gdcm::Attribute<0x300a, 0x03a8> IonControlPointSequence;
    typedef gdcm::Attribute<0x300a, 0x03aa> IonWedgeSequence;
    typedef gdcm::Attribute<0x300a, 0x03ac> IonWedgePositionSequence;
    typedef gdcm::Attribute<0x300a, 0x0401> ReferencedSetupImageSequence;
    typedef gdcm::Attribute<0x300a, 0x0402> SetupImageComment;
    typedef gdcm::Attribute<0x300a, 0x0410> MotionSynchronizationSequence;
    typedef gdcm::Attribute<0x300a, 0x0412> ControlPointOrientation;
    typedef gdcm::Attribute<0x300a, 0x0420> GeneralAccessorySequence;
    typedef gdcm::Attribute<0x300a, 0x0421> GeneralAccessoryID;
    typedef gdcm::Attribute<0x300a, 0x0422> GeneralAccessoryDescription;
    typedef gdcm::Attribute<0x300a, 0x0423> GeneralAccessoryType;
    typedef gdcm::Attribute<0x300a, 0x0424> GeneralAccessoryNumber;
    typedef gdcm::Attribute<0x300a, 0x0431> ApplicatorGeometrySequence;
    typedef gdcm::Attribute<0x300a, 0x0432> ApplicatorApertureShape;
    typedef gdcm::Attribute<0x300a, 0x0433> ApplicatorOpening;
    typedef gdcm::Attribute<0x300a, 0x0434> ApplicatorOpeningX;
    typedef gdcm::Attribute<0x300a, 0x0435> ApplicatorOpeningY;
    typedef gdcm::Attribute<0x300a, 0x0436> SourceToApplicatorMountingPositionDistance;
    typedef gdcm::Attribute<0x300c, 0x0002> ReferencedRTPlanSequence;
    typedef gdcm::Attribute<0x300c, 0x0004> ReferencedBeamSequence;
    typedef gdcm::Attribute<0x300c, 0x0006> ReferencedBeamNumber;
    typedef gdcm::Attribute<0x300c, 0x0007> ReferencedReferenceImageNumber;
    typedef gdcm::Attribute<0x300c, 0x0008> StartCumulativeMetersetWeight;
    typedef gdcm::Attribute<0x300c, 0x0009> EndCumulativeMetersetWeight;
    typedef gdcm::Attribute<0x300c, 0x000a> ReferencedBrachyApplicationSetupSequence;
    typedef gdcm::Attribute<0x300c, 0x000c> ReferencedBrachyApplicationSetupNumber;
    typedef gdcm::Attribute<0x300c, 0x000e> ReferencedSourceNumber;
    typedef gdcm::Attribute<0x300c, 0x0020> ReferencedFractionGroupSequence;
    typedef gdcm::Attribute<0x300c, 0x0022> ReferencedFractionGroupNumber;
    typedef gdcm::Attribute<0x300c, 0x0040> ReferencedVerificationImageSequence;
    typedef gdcm::Attribute<0x300c, 0x0042> ReferencedReferenceImageSequence;
    typedef gdcm::Attribute<0x300c, 0x0050> ReferencedDoseReferenceSequence;
    typedef gdcm::Attribute<0x300c, 0x0051> ReferencedDoseReferenceNumber;
    typedef gdcm::Attribute<0x300c, 0x0055> BrachyReferencedDoseReferenceSequence;
    typedef gdcm::Attribute<0x300c, 0x0060> ReferencedStructureSetSequence;
    typedef gdcm::Attribute<0x300c, 0x006a> ReferencedPatientSetupNumber;
    typedef gdcm::Attribute<0x300c, 0x0080> ReferencedDoseSequence;
    typedef gdcm::Attribute<0x300c, 0x00a0> ReferencedToleranceTableNumber;
    typedef gdcm::Attribute<0x300c, 0x00b0> ReferencedBolusSequence;
    typedef gdcm::Attribute<0x300c, 0x00c0> ReferencedWedgeNumber;
    typedef gdcm::Attribute<0x300c, 0x00d0> ReferencedCompensatorNumber;
    typedef gdcm::Attribute<0x300c, 0x00e0> ReferencedBlockNumber;
    typedef gdcm::Attribute<0x300c, 0x00f0> ReferencedControlPointIndex;
    typedef gdcm::Attribute<0x300c, 0x00f2> ReferencedControlPointSequence;
    typedef gdcm::Attribute<0x300c, 0x00f4> ReferencedStartControlPointIndex;
    typedef gdcm::Attribute<0x300c, 0x00f6> ReferencedStopControlPointIndex;
    typedef gdcm::Attribute<0x300c, 0x0100> ReferencedRangeShifterNumber;
    typedef gdcm::Attribute<0x300c, 0x0102> ReferencedLateralSpreadingDeviceNumber;
    typedef gdcm::Attribute<0x300c, 0x0104> ReferencedRangeModulatorNumber;
    typedef gdcm::Attribute<0x300e, 0x0002> ApprovalStatus;
    typedef gdcm::Attribute<0x300e, 0x0004> ReviewDate;
    typedef gdcm::Attribute<0x300e, 0x0005> ReviewTime;
    typedef gdcm::Attribute<0x300e, 0x0008> ReviewerName;
    typedef gdcm::Attribute<0x4000, 0x0010> Arbitrary;
    typedef gdcm::Attribute<0x4000, 0x4000> TextComments;
    typedef gdcm::Attribute<0x4008, 0x0040> ResultsID;
    typedef gdcm::Attribute<0x4008, 0x0042> ResultsIDIssuer;
    typedef gdcm::Attribute<0x4008, 0x0050> ReferencedInterpretationSequence;
    typedef gdcm::Attribute<0x4008, 0x00ff> ReportProductionStatusTrial;
    typedef gdcm::Attribute<0x4008, 0x0100> InterpretationRecordedDate;
    typedef gdcm::Attribute<0x4008, 0x0101> InterpretationRecordedTime;
    typedef gdcm::Attribute<0x4008, 0x0102> InterpretationRecorder;
    typedef gdcm::Attribute<0x4008, 0x0103> ReferenceToRecordedSound;
    typedef gdcm::Attribute<0x4008, 0x0108> InterpretationTranscriptionDate;
    typedef gdcm::Attribute<0x4008, 0x0109> InterpretationTranscriptionTime;
    typedef gdcm::Attribute<0x4008, 0x010a> InterpretationTranscriber;
    typedef gdcm::Attribute<0x4008, 0x010b> InterpretationText;
    typedef gdcm::Attribute<0x4008, 0x010c> InterpretationAuthor;
    typedef gdcm::Attribute<0x4008, 0x0111> InterpretationApproverSequence;
    typedef gdcm::Attribute<0x4008, 0x0112> InterpretationApprovalDate;
    typedef gdcm::Attribute<0x4008, 0x0113> InterpretationApprovalTime;
    typedef gdcm::Attribute<0x4008, 0x0114> PhysicianApprovingInterpretation;
    typedef gdcm::Attribute<0x4008, 0x0115> InterpretationDiagnosisDescription;
    typedef gdcm::Attribute<0x4008, 0x0117> InterpretationDiagnosisCodeSequence;
    typedef gdcm::Attribute<0x4008, 0x0118> ResultsDistributionListSequence;
    typedef gdcm::Attribute<0x4008, 0x0119> DistributionName;
    typedef gdcm::Attribute<0x4008, 0x011a> DistributionAddress;
    typedef gdcm::Attribute<0x4008, 0x0200> InterpretationID;
    typedef gdcm::Attribute<0x4008, 0x0202> InterpretationIDIssuer;
    typedef gdcm::Attribute<0x4008, 0x0210> InterpretationTypeID;
    typedef gdcm::Attribute<0x4008, 0x0212> InterpretationStatusID;
    typedef gdcm::Attribute<0x4008, 0x0300> Impressions;
    typedef gdcm::Attribute<0x4008, 0x4000> ResultsComments;
    typedef gdcm::Attribute<0x4010, 0x0001> LowEnergyDetectors;
    typedef gdcm::Attribute<0x4010, 0x0002> HighEnergyDetectors;
    typedef gdcm::Attribute<0x4010, 0x0004> DetectorGeometrySequence;
    typedef gdcm::Attribute<0x4010, 0x1001> ThreatROIVoxelSequence;
    typedef gdcm::Attribute<0x4010, 0x1004> ThreatROIBase;
    typedef gdcm::Attribute<0x4010, 0x1005> ThreatROIExtents;
    typedef gdcm::Attribute<0x4010, 0x1006> ThreatROIBitmap;
    typedef gdcm::Attribute<0x4010, 0x1007> RouteSegmentID;
    typedef gdcm::Attribute<0x4010, 0x1008> GantryType;
    typedef gdcm::Attribute<0x4010, 0x1009> OOIOwnerType;
    typedef gdcm::Attribute<0x4010, 0x100a> RouteSegmentSequence;
    typedef gdcm::Attribute<0x4010, 0x1010> PotentialThreatObjectID;
    typedef gdcm::Attribute<0x4010, 0x1011> ThreatSequence;
    typedef gdcm::Attribute<0x4010, 0x1012> ThreatCategory;
    typedef gdcm::Attribute<0x4010, 0x1013> ThreatCategoryDescription;
    typedef gdcm::Attribute<0x4010, 0x1014> ATDAbilityAssessment;
    typedef gdcm::Attribute<0x4010, 0x1015> ATDAssessmentFlag;
    typedef gdcm::Attribute<0x4010, 0x1016> ATDAssessmentProbability;
    typedef gdcm::Attribute<0x4010, 0x1017> Mass;
    typedef gdcm::Attribute<0x4010, 0x1018> Density;
    typedef gdcm::Attribute<0x4010, 0x1019> ZEffective;
    typedef gdcm::Attribute<0x4010, 0x101a> BoardingPassID;
    typedef gdcm::Attribute<0x4010, 0x101b> CenterOfMass;
    typedef gdcm::Attribute<0x4010, 0x101c> CenterOfPTO;
    typedef gdcm::Attribute<0x4010, 0x101d> BoundingPolygon;
    typedef gdcm::Attribute<0x4010, 0x101e> RouteSegmentStartLocationID;
    typedef gdcm::Attribute<0x4010, 0x101f> RouteSegmentEndLocationID;
    typedef gdcm::Attribute<0x4010, 0x1020> RouteSegmentLocationIDType;
    typedef gdcm::Attribute<0x4010, 0x1021> AbortReason;
    typedef gdcm::Attribute<0x4010, 0x1023> VolumeOfPTO;
    typedef gdcm::Attribute<0x4010, 0x1024> AbortFlag;
    typedef gdcm::Attribute<0x4010, 0x1025> RouteSegmentStartTime;
    typedef gdcm::Attribute<0x4010, 0x1026> RouteSegmentEndTime;
    typedef gdcm::Attribute<0x4010, 0x1027> TDRType;
    typedef gdcm::Attribute<0x4010, 0x1028> InternationalRouteSegment;
    typedef gdcm::Attribute<0x4010, 0x1029> ThreatDetectionAlgorithmandVersion;
    typedef gdcm::Attribute<0x4010, 0x102a> AssignedLocation;
    typedef gdcm::Attribute<0x4010, 0x102b> AlarmDecisionTime;
    typedef gdcm::Attribute<0x4010, 0x1031> AlarmDecision;
    typedef gdcm::Attribute<0x4010, 0x1033> NumberOfTotalObjects;
    typedef gdcm::Attribute<0x4010, 0x1034> NumberOfAlarmObjects;
    typedef gdcm::Attribute<0x4010, 0x1037> PTORepresentationSequence;
    typedef gdcm::Attribute<0x4010, 0x1038> ATDAssessmentSequence;
    typedef gdcm::Attribute<0x4010, 0x1039> TIPType;
    typedef gdcm::Attribute<0x4010, 0x103a> DICOSVersion;
    typedef gdcm::Attribute<0x4010, 0x1041> OOIOwnerCreationTime;
    typedef gdcm::Attribute<0x4010, 0x1042> OOIType;
    typedef gdcm::Attribute<0x4010, 0x1043> OOISize;
    typedef gdcm::Attribute<0x4010, 0x1044> AcquisitionStatus;
    typedef gdcm::Attribute<0x4010, 0x1045> BasisMaterialsCodeSequence;
    typedef gdcm::Attribute<0x4010, 0x1046> PhantomType;
    typedef gdcm::Attribute<0x4010, 0x1047> OOIOwnerSequence;
    typedef gdcm::Attribute<0x4010, 0x1048> ScanType;
    typedef gdcm::Attribute<0x4010, 0x1051> ItineraryID;
    typedef gdcm::Attribute<0x4010, 0x1052> ItineraryIDType;
    typedef gdcm::Attribute<0x4010, 0x1053> ItineraryIDAssigningAuthority;
    typedef gdcm::Attribute<0x4010, 0x1054> RouteID;
    typedef gdcm::Attribute<0x4010, 0x1055> RouteIDAssigningAuthority;
    typedef gdcm::Attribute<0x4010, 0x1056> InboundArrivalType;
    typedef gdcm::Attribute<0x4010, 0x1058> CarrierID;
    typedef gdcm::Attribute<0x4010, 0x1059> CarrierIDAssigningAuthority;
    typedef gdcm::Attribute<0x4010, 0x1060> SourceOrientation;
    typedef gdcm::Attribute<0x4010, 0x1061> SourcePosition;
    typedef gdcm::Attribute<0x4010, 0x1062> BeltHeight;
    typedef gdcm::Attribute<0x4010, 0x1064> AlgorithmRoutingCodeSequence;
    typedef gdcm::Attribute<0x4010, 0x1067> TransportClassification;
    typedef gdcm::Attribute<0x4010, 0x1068> OOITypeDescriptor;
    typedef gdcm::Attribute<0x4010, 0x1069> TotalProcessingTime;
    typedef gdcm::Attribute<0x4010, 0x106c> DetectorCalibrationData;
    typedef gdcm::Attribute<0x4ffe, 0x0001> MACParametersSequence;
    typedef gdcm::Attribute<0x5000, 0x0005> CurveDimensions;
    typedef gdcm::Attribute<0x5000, 0x0010> NumberOfPoints;
    typedef gdcm::Attribute<0x5000, 0x0020> TypeOfData;
    typedef gdcm::Attribute<0x5000, 0x0022> CurveDescription;
    typedef gdcm::Attribute<0x5000, 0x0030> AxisUnits;
    typedef gdcm::Attribute<0x5000, 0x0040> AxisLabels;
    typedef gdcm::Attribute<0x5000, 0x0103> DataValueRepresentation;
    typedef gdcm::Attribute<0x5000, 0x0104> MinimumCoordinateValue;
    typedef gdcm::Attribute<0x5000, 0x0105> MaximumCoordinateValue;
    typedef gdcm::Attribute<0x5000, 0x0106> CurveRange;
    typedef gdcm::Attribute<0x5000, 0x0110> CurveDataDescriptor;
    typedef gdcm::Attribute<0x5000, 0x0112> CoordinateStartValue;
    typedef gdcm::Attribute<0x5000, 0x0114> CoordinateStepValue;
    typedef gdcm::Attribute<0x5000, 0x1001> CurveActivationLayer;
    typedef gdcm::Attribute<0x5000, 0x2000> AudioType;
    typedef gdcm::Attribute<0x5000, 0x2002> AudioSampleFormat;
    typedef gdcm::Attribute<0x5000, 0x2004> NumberOfChannels;
    typedef gdcm::Attribute<0x5000, 0x2006> NumberOfSamples;
    typedef gdcm::Attribute<0x5000, 0x2008> SampleRate;
    typedef gdcm::Attribute<0x5000, 0x200a> TotalTime;
    typedef gdcm::Attribute<0x5000, 0x200c> AudioSampleData;
    typedef gdcm::Attribute<0x5000, 0x200e> AudioComments;
    typedef gdcm::Attribute<0x5000, 0x2500> CurveLabel;
    typedef gdcm::Attribute<0x5000, 0x2600> CurveReferencedOverlaySequence;
    typedef gdcm::Attribute<0x5000, 0x2610> CurveReferencedOverlayGroup;
    typedef gdcm::Attribute<0x5000, 0x3000> CurveData;
    typedef gdcm::Attribute<0x5200, 0x9229> SharedFunctionalGroupsSequence;
    typedef gdcm::Attribute<0x5200, 0x9230> PerFrameFunctionalGroupsSequence;
    typedef gdcm::Attribute<0x5400, 0x0100> WaveformSequence;
    typedef gdcm::Attribute<0x5400, 0x0110> ChannelMinimumValue;
    typedef gdcm::Attribute<0x5400, 0x0112> ChannelMaximumValue;
    typedef gdcm::Attribute<0x5400, 0x1004> WaveformBitsAllocated;
    typedef gdcm::Attribute<0x5400, 0x1006> WaveformSampleInterpretation;
    typedef gdcm::Attribute<0x5400, 0x100a> WaveformPaddingValue;
    typedef gdcm::Attribute<0x5400, 0x1010> WaveformData;
    typedef gdcm::Attribute<0x5600, 0x0010> FirstOrderPhaseCorrectionAngle;
    typedef gdcm::Attribute<0x5600, 0x0020> SpectroscopyData;
    typedef gdcm::Attribute<0x6000, 0x0010> OverlayRows;
    typedef gdcm::Attribute<0x6000, 0x0011> OverlayColumns;
    typedef gdcm::Attribute<0x6000, 0x0012> OverlayPlanes;
    typedef gdcm::Attribute<0x6000, 0x0015> NumberOfFramesInOverlay;
    typedef gdcm::Attribute<0x6000, 0x0022> OverlayDescription;
    typedef gdcm::Attribute<0x6000, 0x0040> OverlayType;
    typedef gdcm::Attribute<0x6000, 0x0045> OverlaySubtype;
    typedef gdcm::Attribute<0x6000, 0x0050> OverlayOrigin;
    typedef gdcm::Attribute<0x6000, 0x0051> ImageFrameOrigin;
    typedef gdcm::Attribute<0x6000, 0x0052> OverlayPlaneOrigin;
    typedef gdcm::Attribute<0x6000, 0x0060> OverlayCompressionCode;
    typedef gdcm::Attribute<0x6000, 0x0061> OverlayCompressionOriginator;
    typedef gdcm::Attribute<0x6000, 0x0062> OverlayCompressionLabel;
    typedef gdcm::Attribute<0x6000, 0x0063> OverlayCompressionDescription;
    typedef gdcm::Attribute<0x6000, 0x0066> OverlayCompressionStepPointers;
    typedef gdcm::Attribute<0x6000, 0x0068> OverlayRepeatInterval;
    typedef gdcm::Attribute<0x6000, 0x0069> OverlayBitsGrouped;
    typedef gdcm::Attribute<0x6000, 0x0100> OverlayBitsAllocated;
    typedef gdcm::Attribute<0x6000, 0x0102> OverlayBitPosition;
    typedef gdcm::Attribute<0x6000, 0x0110> OverlayFormat;
    typedef gdcm::Attribute<0x6000, 0x0200> OverlayLocation;
    typedef gdcm::Attribute<0x6000, 0x0800> OverlayCodeLabel;
    typedef gdcm::Attribute<0x6000, 0x0802> OverlayNumberOfTables;
    typedef gdcm::Attribute<0x6000, 0x0803> OverlayCodeTableLocation;
    typedef gdcm::Attribute<0x6000, 0x0804> OverlayBitsForCodeWord;
    typedef gdcm::Attribute<0x6000, 0x1001> OverlayActivationLayer;
    typedef gdcm::Attribute<0x6000, 0x1100> OverlayDescriptorGray;
    typedef gdcm::Attribute<0x6000, 0x1101> OverlayDescriptorRed;
    typedef gdcm::Attribute<0x6000, 0x1102> OverlayDescriptorGreen;
    typedef gdcm::Attribute<0x6000, 0x1103> OverlayDescriptorBlue;
    typedef gdcm::Attribute<0x6000, 0x1200> OverlaysGray;
    typedef gdcm::Attribute<0x6000, 0x1201> OverlaysRed;
    typedef gdcm::Attribute<0x6000, 0x1202> OverlaysGreen;
    typedef gdcm::Attribute<0x6000, 0x1203> OverlaysBlue;
    typedef gdcm::Attribute<0x6000, 0x1301> ROIArea;
    typedef gdcm::Attribute<0x6000, 0x1302> ROIMean;
    typedef gdcm::Attribute<0x6000, 0x1303> ROIStandardDeviation;
    typedef gdcm::Attribute<0x6000, 0x1500> OverlayLabel;
    typedef gdcm::Attribute<0x6000, 0x3000> OverlayData;
    typedef gdcm::Attribute<0x6000, 0x4000> OverlayComments;
    typedef gdcm::Attribute<0x7fe0, 0x0010> PixelData;
    typedef gdcm::Attribute<0x7fe0, 0x0020> CoefficientsSDVN;
    typedef gdcm::Attribute<0x7fe0, 0x0030> CoefficientsSDHN;
    typedef gdcm::Attribute<0x7fe0, 0x0040> CoefficientsSDDN;
    typedef gdcm::Attribute<0x7f00, 0x0010> VariablePixelData;
    typedef gdcm::Attribute<0x7f00, 0x0011> VariableNextDataGroup;
    typedef gdcm::Attribute<0x7f00, 0x0020> VariableCoefficientsSDVN;
    typedef gdcm::Attribute<0x7f00, 0x0030> VariableCoefficientsSDHN;
    typedef gdcm::Attribute<0x7f00, 0x0040> VariableCoefficientsSDDN;
    typedef gdcm::Attribute<0xfffa, 0xfffa> DigitalSignaturesSequence;
    typedef gdcm::Attribute<0xfffc, 0xfffc> DataSetTrailingPadding;
    typedef gdcm::Attribute<0xfffe, 0xe000> Item;
    typedef gdcm::Attribute<0xfffe, 0xe00d> ItemDelimitationItem;
    typedef gdcm::Attribute<0xfffe, 0xe0dd> SequenceDelimitationItem;
    typedef gdcm::Attribute<0x0002, 0x0000> FileMetaInformationGroupLength;
    typedef gdcm::Attribute<0x0002, 0x0001> FileMetaInformationVersion;
    typedef gdcm::Attribute<0x0002, 0x0002> MediaStorageSOPClassUID;
    typedef gdcm::Attribute<0x0002, 0x0003> MediaStorageSOPInstanceUID;
    typedef gdcm::Attribute<0x0002, 0x0010> TransferSyntaxUID;
    typedef gdcm::Attribute<0x0002, 0x0012> ImplementationClassUID;
    typedef gdcm::Attribute<0x0002, 0x0013> ImplementationVersionName;
    typedef gdcm::Attribute<0x0002, 0x0016> SourceApplicationEntityTitle;
    typedef gdcm::Attribute<0x0002, 0x0100> PrivateInformationCreatorUID;
    typedef gdcm::Attribute<0x0002, 0x0102> PrivateInformation;
    typedef gdcm::Attribute<0x0004, 0x1130> FileSetID;
    typedef gdcm::Attribute<0x0004, 0x1141> FileSetDescriptorFileID;
    typedef gdcm::Attribute<0x0004, 0x1142> SpecificCharacterSetOfFileSetDescriptorFile;
    typedef gdcm::Attribute<0x0004, 0x1200> OffsetOfTheFirstDirectoryRecordOfTheRootDirectoryEntity;
    typedef gdcm::Attribute<0x0004, 0x1202> OffsetOfTheLastDirectoryRecordOfTheRootDirectoryEntity;
    typedef gdcm::Attribute<0x0004, 0x1212> FileSetConsistencyFlag;
    typedef gdcm::Attribute<0x0004, 0x1220> DirectoryRecordSequence;
    typedef gdcm::Attribute<0x0004, 0x1400> OffsetOfTheNextDirectoryRecord;
    typedef gdcm::Attribute<0x0004, 0x1410> RecordInUseFlag;
    typedef gdcm::Attribute<0x0004, 0x1420> OffsetOfReferencedLowerLevelDirectoryEntity;
    typedef gdcm::Attribute<0x0004, 0x1430> DirectoryRecordType;
    typedef gdcm::Attribute<0x0004, 0x1432> PrivateRecordUID;
    typedef gdcm::Attribute<0x0004, 0x1500> ReferencedFileID;
    typedef gdcm::Attribute<0x0004, 0x1504> MRDRDirectoryRecordOffset;
    typedef gdcm::Attribute<0x0004, 0x1510> ReferencedSOPClassUIDInFile;
    typedef gdcm::Attribute<0x0004, 0x1511> ReferencedSOPInstanceUIDInFile;
    typedef gdcm::Attribute<0x0004, 0x1512> ReferencedTransferSyntaxUIDInFile;
    typedef gdcm::Attribute<0x0004, 0x151a> ReferencedRelatedGeneralSOPClassUIDInFile;
    typedef gdcm::Attribute<0x0004, 0x1600> NumberOfReferences;

}
}

#endif
